##:cassert@cassert
#-assert@cassert
void assert (int expression)
/*-p-
@b:assert - Abort the program if assertion is false.
@r:No value is returned. ANSI - C assert() is implemented as  a  macro;  if  the  expression tested  has side - effects, program behaviour will be dif- ferent depending on whether NDEBUG is  defined.  This  may create  Heisenbugs  which go away when debugging is turned on.
assert()  prints  an  error message to standard output and terminates the program by calling abort() if expression is false  (i.e.,  compares equal to zero).  This only happens when the macro NDEBUG is undefined.
/*/

##:cctype@cctype
#-tolower@cctype
int tolower (int c)
/*-p-
@b:toupper, tolower - convert letter to upper or lower case
see: toupper
/*/

#-toupper@cctype
int toupper(int c)
/*-p-
@b:toupper, tolower - convert letter to upper or lower case
@r:The  value  returned is that of the converted letter, or c if the conversion was not possible. ANSI - C, BSD 4.3 The details of what constitutes an uppercase or  lowercase letter  depend  on  the  current locale.  For example, the default "C" locale does not know about umlauts, so no con- version is done for them. In some non - English locales, there are lowercase letters with no corresponding  uppercase  equivalent;  the  German sharp s is one example.
toupper()  converts  the letter c to upper case, if possi- ble. tolower() converts the letter c to lower case,  if  possi- ble.
/*/



#-isalnum@cctype
int isalnum(int c)
/*-p-
@b:Check if character is alphanumeric (function)
Checks if parameter c is either a decimal digit or an uppercase or lowercase letter.
The result is true if either isalpha or isdigit would also return true.
/*/

#-isalpha@cctype
int isalpha(int c)	
/*-p-
@b:Check if character is alphabetic (function)
Checks if parameter c is either an uppercase or a lowercase alphabetic letter.
/*/

#-iscntrl@cctype
int iscntrl(int c)
/*-p-
@b:Check if character is a control character (function)
For the standard ASCII character set, control characters are those between ASCII codes 0x00 (NUL) and 0x1f (US), plus 0x7f (DEL), although specific compiler implementations for certain platforms may define additional control characters in the extended character set (above 0x7f).
The opposite of iscntrl is isprint, which is true for all characters that are not control characters.

/*/

#-isdigit@cctype
int isdigit(int c)	
/*-p-
@b:Check if character is decimal digit (function)
Decimal digits are any of: 0 1 2 3 4 5 6 7 8 9
/*/

#-isgraph@cctype
int isgraph(int c)
/*-p-
@b:Check if character has graphical representation (function)
Checks if parameter c is a character with graphical representation. The characters with graphical representation are all those characters than can be printed (as determined by isprint) except for whitespace characters (like ' '), which are not considered isgraph characters.
/*/

#-islower@cctype
int islower(int c)
/*-p-
@b:Check if character is lowercase letter (function)
Notice that what is considered a letter may depend on the locale being used; In the default C locale, a lowercase letter is any of: a b c d e f g h i j k l m n o p q r s t u v w x y z.
/*/

#-isprint@cctype
int isprint(int c)
/*-p-
@b:Check if character is printable (function)
A printable character is any character that is not a control character.
For the standard ASCII character set, control characters are those between ASCII codes 0x00 (NUL) and 0x1f (US), plus 0x7f (DEL). Therefore, printable characters are all but these, although specific compiler implementations for certain platforms may define additional control characters in the extended character set (above 0x7f).

isgraph returns true for the same cases as isprint except for the whitespace characters (like ' '), which return true when checked with isprint but false when checked with isgraph.
/*/

#-ispunct@cctype
int ispunct(int c)
/*-p-
@b:Check if character is a punctuation character (function)
For the purpose of this function, a punctuation character is any graphic character (as in isgraph) that is not alphanumeric (as in isalnum).
/*/

#-isspace@cctype
int isspace(int c)
/*-p-
@b:Check if character is a white-space (function)
Checks if parameter c is a white-space character.
For the purpose of this function, standard white-space characters are:
' '	(0x20)	space (SPC)
'\\t'	(0x09)	horizontal tab (TAB)
'\\n'	(0x0a)	newline (LF)
'\\v'	(0x0b)	vertical tab (VT)
'\\f'	(0x0c)	feed (FF)
'\\r'	(0x0d)	carriage return (CR)

Specific compiler implementations or locales may define additional white-space characters in the extended character set (values over 0x7f).
/*/

#-isupper@cctype
int isupper(int c)
/*-p-
@b:Check if character is uppercase letter (function)
Notice that what is considered a letter may depend on the locale being used; In the default C locale, an uppercase letter is any of: A B C D E F G H I J K L M N O P Q R S T U V W X Y Z.
/*/

#-isxdigit@cctype
int isxdigit(int c)
/*-p-
@b:Check if character is hexadecimal digit (function)
Checks if parameter c is a hexdecimal digit character.
Hexadecimal digits are any of: 0 1 2 3 4 5 6 7 8 9 a b c d e f A B C D E F
/*/
##:cmath@cmath
#-acos@cmath
double acos(double x)
/*-p-
@b:acos - arc cosine function
@r:The  acos() function returns the arc cosine in radians and the value is mathematically defined to be between 0 and PI (inclusive).
The  acos()  function calculates the arc cosine of x; that is the value whose cosine is x.  If x  falls  outside  the range -1 to 1, acos() fails and errno is set.
/*/

#-asin@cmath
double asin(double x)
/*-p-
@b:asin - arc sine function
@r:The  asin()  function  returns the arc sine in radians and the value is mathematically defined to  be  between  -PI/2 and PI/2 (inclusive).
The  asin() function calculates the arc sine of x; that is the value whose sine is x.  If x falls outside  the  range -1 to 1, asin() fails and errno is set.
/*/

#-atan@cmath
double atan(double x)
/*-p-
@b:atan - arc tangent function
@r:The atan() function returns the arc tangent in radians and the  value  is  mathematically defined to be between -PI/2 and PI/2 (inclusive).
The  atan() function calculates the arc tangent of x; that is the value whose tangent is x.
/*/

#-atan2@cmath
double atan2(double y, double x)
/*-p-
@b:atan2 - arc tangent function of two variables
@r:The atan2() function returns the result in radians,  which is between -PI and PI (inclusive).
The atan2() function calculates the arc tangent of the two variables x and y.  It is similar to calculating  the  arc tangent  of y / x, except that the signs of both arguments are used to determine the quadrant of the result.
/*/

#-acosh@cmath
double acosh(double x)
/*-p-
@b:acosh - inverse hyperbolic cosine function
@r:
The  acosh()  function  calculates  the inverse hyperbolic cosine of x; that is the value whose hyperbolic cosine  is x.   If  x  is less than 1.0, acosh() returns not-a-number (NaN) and errno is set.
/*/

#-asinh@cmath
double asinh(double x)
/*-p-
@b:asinh - inverse hyperbolic sine function
@r:
The  asinh()  function  calculates  the inverse hyperbolic sine of x; that is the value whose hyperbolic sine is x.
/*/

#-ceil@cmath
double ceil (double x)
/*-p-
@b:ceil - smallest integral value not less than x
@r:
The  ceil() function rounds x upwards to the nearest inte- ger, returning that value as a double.
/*/

#-cos@cmath
double cos(double x)
/*-p-
@b:cos - cosine function
@r:The cos() function returns a value between -1 and 1.
The  cos()  function  returns  the cosine of x, where x is given in radians.
/*/

#-cosh@cmath
double cosh(double x)
/*-p-
@b:cosh - hyperbolic cosine function
@r:
The  cosh()  function  returns the hyperbolic cosine of x, which is defined mathematically as exp(x) + exp(-x) / 2.
/*/

#-copysign@cmath
double copysign(double x, double y)
/*-p-
@b:copysign - copy sign of a number
@r:
The  copysign()  function  returns  a value whose absolute value matches x, but whose sign matches that of y.
/*/

#-exp@cmath
double exp(double x); double log(double x); double log10(double x); double pow(double x, double y)
/*-p-
@b:exp,  log, log10, pow - exponential, logarithmic and powerfunctions
@r:
The exp() function returns the value of  e  (the  base  of natural logarithms) raised to the power of x. The log() function returns the natural logarithm of x. The log10() function returns the base-10 logarithm of x. The  pow()  function  returns the value of x raised to the power of y.
/*/

#-fabs@cmath
double fabs(double x)
/*-p-
@b:fabs - absolute value of floating-point number
@r:
The  fabs()  function  returns  the  absolute value of the floating-point number x.
/*/

#-floor@cmath
double floor(double x)
/*-p-
@b:floor - largest integral value not greater than x
@r:
The  floor()  function  rounds  x downwards to the nearest integer, returning that value as a double.
/*/

#-fmod@cmath
double fmod(double x, double y)
/*-p-
@b:fmod - floating-point remainder function
@r:The  fmod()  function  returns  the remainder, unless y is zero, when the function fails and errno is set.
The  modf()  function computes the remainder of dividing x by y.  The return value is x - n * y, where n is the  quo- tient of x / y, rounded towards zero to an integer.
/*/

#-frexp@cmath
double frexp(double x, int *exp)
/*-p-
@b:frexp  -  convert  floating-point number to fractional andintegral components
@r:The frexp() function returns the normalized fraction.   If the  argument  x is not zero, the normalized fraction is x times a power of two, and  is  always  in  the  range  1/2 (inclusive) to 1 (exclusive).  If x is zero, then the nor- malized fraction is zero and zero is stored in exp.
The frexp() function is used to split the number x into  a normalized  fraction  and  an  exponent which is stored in exp.
/*/

#-finite@cmath
int isinf(double value); int isnan(double value); int finite(double value)
/*-p-
@b:isinf,  isnan,  finite - test for infinity or not-a-number(NaN)
@r:
The isinf() function returns -1 if value represents  nega- tive  infinity,  1  if value represents positive infinity, and 0 otherwise. The isnan() function returns a non-zero value if value  is "not-a-number" (NaN), and 0 otherwise. The finite() function returns a non-zero value if value is finite or is not a "not-a-number" (NaN) value, and 0  oth- erwise.
/*/

#-hypot@cmath
double hypot(double x, double y)
/*-p-
@b:hypot - Euclidean distance function
@r:
The hypot() function returns the sqrt(x*x + y*y).  This is the length of the hypotenuse  of  a  right-angle  triangle with sides of length x and y, or the distance of the point (x, y) from the origin.
/*/

#-isinf@cmath
int isinf(double value); int isnan(double value); int finite(double value)
/*-p-
@b:isinf,  isnan,  finite - test for infinity or not-a-number(NaN)
@r:
The isinf() function returns -1 if value represents  nega- tive  infinity,  1  if value represents positive infinity, and 0 otherwise. The isnan() function returns a non-zero value if value  is "not-a-number" (NaN), and 0 otherwise. The finite() function returns a non-zero value if value is finite or is not a "not-a-number" (NaN) value, and 0  oth- erwise.
/*/

#-isnan@cmath
int isinf(double value); int isnan(double value); int finite(double value)
/*-p-
@b:isinf,  isnan,  finite - test for infinity or not-a-number(NaN)
@r:
The isinf() function returns -1 if value represents  nega- tive  infinity,  1  if value represents positive infinity, and 0 otherwise. The isnan() function returns a non-zero value if value  is "not-a-number" (NaN), and 0 otherwise. The finite() function returns a non-zero value if value is finite or is not a "not-a-number" (NaN) value, and 0  oth- erwise.
/*/

#-j0@cmath
double j0(double x); double j1(double x); double jn(int n, double x); double y0(double x); double y1(double x); double yn(int n, double x)
/*-p-
@b:j0, j1, jn, y0, y1, yn - Bessel functions
@r:
The  j0()  and j1() functions return Bessel functions of x of the first kind of orders 0 and  1,  respectively.   The jn()  function  returns  the  Bessel  function of x of the first kind of order n. The y0() and y1() functions return Bessel functions  of  x of  the  second kind of orders 0 and 1, respectively.  The yn() function returns the Bessel function of x of the sec- ond kind of order n. For the functions y0(), y1() and yn(), the value of x must be positive.  For negative values of  x,  these  functions return -HUGE_VAL.
/*/

#-j1@cmath
double j0(double x); double j1(double x); double jn(int n, double x); double y0(double x); double y1(double x); double yn(int n, double x)
/*-p-
@b:j0, j1, jn, y0, y1, yn - Bessel functions
@r:
The  j0()  and j1() functions return Bessel functions of x of the first kind of orders 0 and  1,  respectively.   The jn()  function  returns  the  Bessel  function of x of the first kind of order n. The y0() and y1() functions return Bessel functions  of  x of  the  second kind of orders 0 and 1, respectively.  The yn() function returns the Bessel function of x of the sec- ond kind of order n. For the functions y0(), y1() and yn(), the value of x must be positive.  For negative values of  x,  these  functions return -HUGE_VAL.
/*/

#-jn@cmath
double j0(double x); double j1(double x); double jn(int n, double x); double y0(double x); double y1(double x); double yn(int n, double x)
/*-p-
@b:j0, j1, jn, y0, y1, yn - Bessel functions
@r:
The  j0()  and j1() functions return Bessel functions of x of the first kind of orders 0 and  1,  respectively.   The jn()  function  returns  the  Bessel  function of x of the first kind of order n. The y0() and y1() functions return Bessel functions  of  x of  the  second kind of orders 0 and 1, respectively.  The yn() function returns the Bessel function of x of the sec- ond kind of order n. For the functions y0(), y1() and yn(), the value of x must be positive.  For negative values of  x,  these  functions return -HUGE_VAL.
/*/

#-j0, j1, jn, y0, y1, yn@cmath
double j0(double x); double j1(double x); double jn(int n, double x); double y0(double x); double y1(double x); double yn(int n, double x)
/*-p-
@b:j0, j1, jn, y0, y1, yn - Bessel functions
@r:
The  j0()  and j1() functions return Bessel functions of x of the first kind of orders 0 and  1,  respectively.   The jn()  function  returns  the  Bessel  function of x of the first kind of order n. The y0() and y1() functions return Bessel functions  of  x of  the  second kind of orders 0 and 1, respectively.  The yn() function returns the Bessel function of x of the sec- ond kind of order n. For the functions y0(), y1() and yn(), the value of x must be positive.  For negative values of  x,  these  functions return -HUGE_VAL.
/*/

#-ldexp@cmath
double ldexp(double x, int exp)
/*-p-
@b:ldexp  -  multiply floating-point number by integral powerof 2
@r:
The ldexp() function returns the result of multiplying the floating-point number x by 2 raised to the power exp.
/*/

#-log@cmath
double exp(double x); double log(double x); double log10(double x); double pow(double x, double y)
/*-p-
@b:exp,  log, log10, pow - exponential, logarithmic and powerfunctions
@r:
The exp() function returns the value of  e  (the  base  of natural logarithms) raised to the power of x. The log() function returns the natural logarithm of x. The log10() function returns the base-10 logarithm of x. The  pow()  function  returns the value of x raised to the power of y.
/*/

#-log10@cmath
double exp(double x); double log(double x); double log10(double x); double pow(double x, double y)
/*-p-
@b:exp,  log, log10, pow - exponential, logarithmic and powerfunctions
@r:
The exp() function returns the value of  e  (the  base  of natural logarithms) raised to the power of x. The log() function returns the natural logarithm of x. The log10() function returns the base-10 logarithm of x. The  pow()  function  returns the value of x raised to the power of y.
/*/

#-lgamma@cmath
double lgamma(double x)
/*-p-
@b:lgamma - log gamma function
@r:
The  lgamma()  function  returns  the  log of the absolute value of the Gamma function.  The sign of the Gamma  func- tion is returned in the external integer signgam. For   negative  integer  values  of  x,  lgamma()  returns HUGE_VAL and errno is set to ERANGE.
/*/

#-modf@cmath
double modf(double x, double *iptr)
/*-p-
@b:modf  - extract signed integral and fractional values fromfloating-point number
@r:The modf() function returns the fractional part of x.
The modf() function breaks the argument x into an integral part  and  a  fractional  part, each of which has the same sign as x.  The integral part is stored in iptr.
/*/

#-pow@cmath
double exp(double x); double log(double x); double log10(double x); double pow(double x, double y)
/*-p-
@b:exp,  log, log10, pow - exponential, logarithmic and powerfunctions
@r:
The exp() function returns the value of  e  (the  base  of natural logarithms) raised to the power of x. The log() function returns the natural logarithm of x. The log10() function returns the base-10 logarithm of x. The  pow()  function  returns the value of x raised to the power of y.
/*/

#-rint@cmath
double rint(double x)
/*-p-
@b:rint - round to closest integer
@r:The  rint() function returns the integer value as a float- ing-point number.
The rint() function rounds x to an integer value according to the prevalent rounding mode.  The default rounding mode is to round to the nearest integer.
/*/

#-sin@cmath
double sin(double x)
/*-p-
@b:sin - sine function
@r:The sin() function returns a value between -1 and 1.
The sin() function returns the sine of x, where x is given in radians.
/*/

#-sinh@cmath
double sinh(double x)
/*-p-
@b:sinh - hyperbolic sine function
@r:
The  sinh()  function  returns  the  hyperbolic sine of x, which is defined mathematically as exp(x) - exp(-x) / 2.
/*/

#-sqrt@cmath
double sqrt(double x)
/*-p-
@b:sqrt - square root function
@r:
The  sqrt()  function returns the non-negative square root of x.  It fails and sets errno to EDOM, if x is  negative.
/*/

#-tan@cmath
double tan(double x)
/*-p-
@b:tan - tangent function
@r:
The  tan()  function  returns the tangent of x, where x is given in radians.
/*/

#-tanh@cmath
double tanh(double x)
/*-p-
@b:tanh - hyperbolic tangent function
@r:
The  tanh()  function returns the hyperbolic tangent of x, which is defined mathematically as sinh(x) / cosh(x).
/*/

#-y0@cmath
double j0(double x); double j1(double x); double jn(int n, double x); double y0(double x); double y1(double x); double yn(int n, double x)
/*-p-
@b:j0, j1, jn, y0, y1, yn - Bessel functions
@r:
The  j0()  and j1() functions return Bessel functions of x of the first kind of orders 0 and  1,  respectively.   The jn()  function  returns  the  Bessel  function of x of the first kind of order n. The y0() and y1() functions return Bessel functions  of  x of  the  second kind of orders 0 and 1, respectively.  The yn() function returns the Bessel function of x of the sec- ond kind of order n. For the functions y0(), y1() and yn(), the value of x must be positive.  For negative values of  x,  these  functions return -HUGE_VAL.
/*/

#-y1@cmath
double j0(double x); double j1(double x); double jn(int n, double x); double y0(double x); double y1(double x); double yn(int n, double x)
/*-p-
@b:j0, j1, jn, y0, y1, yn - Bessel functions
@r:
The  j0()  and j1() functions return Bessel functions of x of the first kind of orders 0 and  1,  respectively.   The jn()  function  returns  the  Bessel  function of x of the first kind of order n. The y0() and y1() functions return Bessel functions  of  x of  the  second kind of orders 0 and 1, respectively.  The yn() function returns the Bessel function of x of the sec- ond kind of order n. For the functions y0(), y1() and yn(), the value of x must be positive.  For negative values of  x,  these  functions return -HUGE_VAL.
/*/

#-yn@cmath
double j0(double x); double j1(double x); double jn(int n, double x); double y0(double x); double y1(double x); double yn(int n, double x)
/*-p-
@b:j0, j1, jn, y0, y1, yn - Bessel functions
@r:
The  j0()  and j1() functions return Bessel functions of x of the first kind of orders 0 and  1,  respectively.   The jn()  function  returns  the  Bessel  function of x of the first kind of order n. The y0() and y1() functions return Bessel functions  of  x of  the  second kind of orders 0 and 1, respectively.  The yn() function returns the Bessel function of x of the sec- ond kind of order n. For the functions y0(), y1() and yn(), the value of x must be positive.  For negative values of  x,  these  functions return -HUGE_VAL.
/*/

#-yn@cmath
double j0(double x); double j1(double x); double jn(int n, double x); double y0(double x); double y1(double x); double yn(int n, double x)
/*-p-
@b:j0, j1, jn, y0, y1, yn - Bessel functions
@r:
The  j0()  and j1() functions return Bessel functions of x of the first kind of orders 0 and  1,  respectively.   The jn()  function  returns  the  Bessel  function of x of the first kind of order n. The y0() and y1() functions return Bessel functions  of  x of  the  second kind of orders 0 and 1, respectively.  The yn() function returns the Bessel function of x of the sec- ond kind of order n. For the functions y0(), y1() and yn(), the value of x must be positive.  For negative values of  x,  these  functions return -HUGE_VAL.
/*/

##:cstdio@cstdio
#-clearerr@cstdio
void clearerr( FILE *stream); int feof( FILE *stream); int ferror( FILE *stream); int fileno( FILE *stream)
/*-p-
@b:clearerr,  feof,  ferror,  fileno - check and reset streamstatus
@r:
The function clearerr clears  the  end-of-file  and  error indicators for the stream pointed to by stream. The  function feof tests the end-of-file indicator for the stream pointed to by stream, returning non-zero if  it  is set.  The end-of-file indicator can only be cleared by the function clearerr. The function ferror tests  the  error  indicator  for  the stream  pointed  to by stream, returning non-zero if it is set.  The error indicator can only be reset by the  clear- err function. The  function  fileno  examines  the  argument  stream and returns its integer descriptor.
/*/

#-fclose@cstdio
int fclose( FILE *stream)
/*-p-
@b:fclose - close a stream
@r:Upon successful completion 0 is returned.  Otherwise,  EOF is  returned and the global variable errno is set to indi- cate the error.  In either case no further access  to  the stream is possible.
The  fclose function dissociates the named stream from its underlying file or set of functions.  If  the  stream  was being used for output, any buffered data is written first, using <A HREF="/man3/fflush">fflush(3)</A>.
/*/

#-feof@cstdio
void clearerr( FILE *stream); int feof( FILE *stream); int ferror( FILE *stream); int fileno( FILE *stream)
/*-p-
@b:clearerr,  feof,  ferror,  fileno - check and reset streamstatus
@r:
The function clearerr clears  the  end-of-file  and  error indicators for the stream pointed to by stream. The  function feof tests the end-of-file indicator for the stream pointed to by stream, returning non-zero if  it  is set.  The end-of-file indicator can only be cleared by the function clearerr. The function ferror tests  the  error  indicator  for  the stream  pointed  to by stream, returning non-zero if it is set.  The error indicator can only be reset by the  clear- err function. The  function  fileno  examines  the  argument  stream and returns its integer descriptor.
/*/

#-ferror@cstdio
void clearerr( FILE *stream); int feof( FILE *stream); int ferror( FILE *stream); int fileno( FILE *stream)
/*-p-
@b:clearerr,  feof,  ferror,  fileno - check and reset streamstatus
@r:
The function clearerr clears  the  end-of-file  and  error indicators for the stream pointed to by stream. The  function feof tests the end-of-file indicator for the stream pointed to by stream, returning non-zero if  it  is set.  The end-of-file indicator can only be cleared by the function clearerr. The function ferror tests  the  error  indicator  for  the stream  pointed  to by stream, returning non-zero if it is set.  The error indicator can only be reset by the  clear- err function. The  function  fileno  examines  the  argument  stream and returns its integer descriptor.
/*/

#-fflush@cstdio
int fflush( FILE *stream); int fpurge( FILE *stream)
/*-p-
@b:fflush, fpurge - flush a stream
@r:Upon successful completion 0 is returned.  Otherwise,  EOF is  returned and the global variable errno is set to indi- cate the error.
The  function  fflush  forces a write of all buffered data for the given output or update  stream  via  the  stream's underlying  write function.  The open status of the stream is unaffected. If the stream argument is NULL, fflush  flushes  all  open output streams. (Does this happen under Linux?) The function fpurge erases any input or output buffered in the given stream.  For output streams  this  discards  any unwritten  output.   For  input  streams this discards any input read from the underlying object but not yet obtained via  <A HREF="/man3/getc">getc(3)</A>;  this  includes  any  text  pushed  back via ungetc.
/*/

#-fgetc@cstdio
int fgetc(FILE *stream); char *fgets(char *s, int size, FILE *stream); int getc(FILE *stream); int getchar(void); char *gets(char *s); int ungetc(int c, FILE *stream)
/*-p-
@b:fgetc, fgets, getc, getchar, gets, ungetc - input of char-acters and strings
@r:fgetc(), getc() and getchar() return the character read as an  unsigned  char cast to an int or EOF on end of file or error. gets() and fgets() return s on success, and NULL on end of file or error. ungetc() returns c on success, or EOF on error. ANSI - C, POSIX.1 Because  it is impossible to tell without knowing the data in advance how  many  characters  gets()  will  read,  and because  gets() will continue to store characters past the end of the buffer, it is extremely dangerous to  use.   It has  been  used  to  break computer security.  Use fgets() instead. It is not advisable to mix calls to input  functions  from the stdio library with low - level calls to read() for the file descriptor associated  with  the  input  stream;  the results  will  be undefined and very probably not what you want.
fgetc() reads the next character from stream  and  returns it  as  an  unsigned char cast to an int, or EOF on end of file or error. getc() is equivalent to fgetc()  except  that  it  may  be implemented  as  a  macro which evaluates stream more than once. getchar() is equivalent to getc(stdin). gets() reads a line from stdin into the buffer pointed  to by  s  until either a terminating newline or EOF, which it replaces with '\0'.  No check for buffer overrun  is  per- formed (see BUGS below). fgets()  reads  in at most one less than n characters from stream and stores them into the buffer pointed  to  by  s. Reading  stops after an EOF or a newline.  If a newline is read, it is stored into the  buffer.   A  '\0'  is  stored after the last character in the buffer. ungetc()  pushes  c back to stream, cast to unsigned char, where it is  available  for  subsequent  read  operations. Pushed  -  back  characters  will  be  returned in reverse order; only one pushback is guaranteed. Calls to the functions described here can  be  mixed  with each  other  and  with calls to other input functions from the stdio library for the same input stream.
/*/

#-fgetpos@cstdio
int fseek( FILE *stream, long offset, int whence); long ftell( FILE *stream); void rewind( FILE *stream); int fgetpos( FILE *stream, fpos_t *pos); int fsetpos( FILE *stream, fpos_t *pos)
/*-p-
@b:fgetpos,  fseek,  fsetpos,  ftell,  rewind  - reposition astream
@r:The  rewind  function  returns  no value.  Upon successful completion, fgetpos, fseek, fsetpos return  0,  and  ftell returns the current offset.  Otherwise, -1 is returned and the global variable errno is set to indicate the error.
The fseek function sets the file  position  indicator  for the  stream  pointed to by stream.  The new position, mea- sured in bytes, is obtained by adding offset bytes to  the position  specified  by  whence.   If  whence  is  set  to SEEK_SET, SEEK_CUR, or SEEK_END, the offset is relative to the  start of the file, the current position indicator, or end-of-file, respectively.  A successful call to the fseek function  clears  the end-of-file indicator for the stream and undoes any effects of the <A HREF="/man3/ungetc">ungetc(3)</A>  function  on  the same stream. The  ftell  function obtains the current value of the file position indicator for the stream pointed to by stream. The rewind function sets the file position  indicator  for the  stream  pointed  to by stream to the beginning of the file.  It is equivalent to: (void)fseek(stream, 0L, SEEK_SET) except that the error indicator for  the  stream  is  also cleared (see <A HREF="/man3/clearerr">clearerr(3)</A>. The fgetpos and fsetpos functions are alternate interfaces equivalent  to  ftell  and  fseek  (with  whence  set   to SEEK_SET),  setting  and  storing the current value of the file offset into or from the object referenced by pos.  On some  non-UNIX  systems  an fpos_t object may be a complex object and these routines may be the only way to  portably reposition a text stream.
/*/

#-fgets@cstdio
int fgetc(FILE *stream); char *fgets(char *s, int size, FILE *stream); int getc(FILE *stream); int getchar(void); char *gets(char *s); int ungetc(int c, FILE *stream)
/*-p-
@b:fgetc, fgets, getc, getchar, gets, ungetc - input of char-acters and strings
@r:fgetc(), getc() and getchar() return the character read as an  unsigned  char cast to an int or EOF on end of file or error. gets() and fgets() return s on success, and NULL on end of file or error. ungetc() returns c on success, or EOF on error. ANSI - C, POSIX.1 Because  it is impossible to tell without knowing the data in advance how  many  characters  gets()  will  read,  and because  gets() will continue to store characters past the end of the buffer, it is extremely dangerous to  use.   It has  been  used  to  break computer security.  Use fgets() instead. It is not advisable to mix calls to input  functions  from the stdio library with low - level calls to read() for the file descriptor associated  with  the  input  stream;  the results  will  be undefined and very probably not what you want.
fgetc() reads the next character from stream  and  returns it  as  an  unsigned char cast to an int, or EOF on end of file or error. getc() is equivalent to fgetc()  except  that  it  may  be implemented  as  a  macro which evaluates stream more than once. getchar() is equivalent to getc(stdin). gets() reads a line from stdin into the buffer pointed  to by  s  until either a terminating newline or EOF, which it replaces with '\0'.  No check for buffer overrun  is  per- formed (see BUGS below). fgets()  reads  in at most one less than n characters from stream and stores them into the buffer pointed  to  by  s. Reading  stops after an EOF or a newline.  If a newline is read, it is stored into the  buffer.   A  '\0'  is  stored after the last character in the buffer. ungetc()  pushes  c back to stream, cast to unsigned char, where it is  available  for  subsequent  read  operations. Pushed  -  back  characters  will  be  returned in reverse order; only one pushback is guaranteed. Calls to the functions described here can  be  mixed  with each  other  and  with calls to other input functions from the stdio library for the same input stream.
/*/

#-fopen@cstdio
FILE *fopen( char *path, char *mode); FILE *fdopen( int fildes, char *mode); FILE *freopen( char *path, char *mode, FILE *stream)
/*-p-
@b:fopen, fdopen, freopen - stream open functions
@r:Upon  successful  completion  fopen,  fdopen  and  freopen return a FILE pointer.  Otherwise, NULL  is  returned  and the global variable errno is set to indicate the error.
The fopen function opens the file whose name is the string pointed to by path and associates a stream with it. The argument mode points to a string beginning with one of the  following sequences (Additional characters may follow these sequences.): r      Open text file for reading.  The  stream  is  posi- tioned at the beginning of the file. r+     Open  for reading and writing.  The stream is posi- tioned at the beginning of the file. w      Truncate file to zero length or  create  text  file for  writing.   The  stream  is  positioned  at the beginning of the file. w+     Open for reading and writing.  The file is  created if  it  does  not exist, otherwise it is truncated. The stream is positioned at the  beginning  of  the file. a      Open  for  writing.  The file is created if it does not exist.  The stream is positioned at the end  of the file. a+     Open  for reading and writing.  The file is created if it does not exist.  The stream is positioned  at the end of the file. The  mode  string can also include the letter ``b'' either as a third character or as a character between the charac- ters  in any of the two-character strings described above. This is strictly for compatibility with  ANSI  C3.159-1989 (``ANSI  C'')  and  has  no  effect; the ``b'' is ignored. Linux may not behave this way. Any      created      files      will      have       mode S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH (0666), as modified by the process' umask value (see <A HREF="/man2/umask">umask(2)</A>. Reads and writes may be intermixed on  read/write  streams in  any  order, and do not require an intermediate seek as in previous versions of stdio.  This is  not  portable  to other  systems,  however,  and  may  not  work under Linux (someone should find out and fix  this  manpage);  ANSI  C requires   that  a  file  positioning  function  intervene between  output  and  input,  unless  an  input  operation encounters end-of-file. The  fdopen function associates a stream with the existing file descriptor, fildes.  The mode of the stream  must  be compatible with the mode of the file descriptor. The  freopen  function  opens  the  file whose name is the string pointed  to  by  path  and  associates  the  stream pointed  to by stream with it.  The original stream (if it exists) is closed.  The mode argument is used just  as  in the  fopen function.  The primary use of the freopen func- tion is to change the file associated with a standard text stream (stderr, stdin, or stdout).
/*/

#-fputc@cstdio
int fputc(int c, FILE *stream); int fputs(const char *s, FILE *stream); int putc(int c, FILE *stream); int putchar(int c); int puts(char *s); int ungetc(int c, FILE *stream)
/*-p-
@b:fputc,  fputs,  putc, putchar, puts - output of charactersand strings
@r:fputc(), putc() and putchar() return the character written as an unsigned char cast to an int or EOF on error. puts() and fputs() return a non - negative number on  suc- cess, or EOF on error. ANSI - C, POSIX.1 It  is not advisable to mix calls to output functions from the stdio library with low - level calls  to  write()  for the  file  descriptor  associated  with  the  same  output stream; the results will be undefined  and  very  probably not what you want.
fputc() writes the character c, cast to an unsigned  char, to stream. fputs()  writes the string s to stream, without its trail- ing '\0'. putc() is equivalent to fputc()  except  that  it  may  be implemented  as  a  macro which evaluates stream more than once. putchar(c); is equivalent to putc(c,stdout). puts() writes the string s and a trailing newline to  std- out. Calls  to  the  functions described here can be mixed with each other and with calls to other output  functions  from the stdio library for the same output stream.
/*/

#-fputs@cstdio
int fputc(int c, FILE *stream); int fputs(const char *s, FILE *stream); int putc(int c, FILE *stream); int putchar(int c); int puts(char *s); int ungetc(int c, FILE *stream)
/*-p-
@b:fputc,  fputs,  putc, putchar, puts - output of charactersand strings
@r:fputc(), putc() and putchar() return the character written as an unsigned char cast to an int or EOF on error. puts() and fputs() return a non - negative number on  suc- cess, or EOF on error. ANSI - C, POSIX.1 It  is not advisable to mix calls to output functions from the stdio library with low - level calls  to  write()  for the  file  descriptor  associated  with  the  same  output stream; the results will be undefined  and  very  probably not what you want.
fputc() writes the character c, cast to an unsigned  char, to stream. fputs()  writes the string s to stream, without its trail- ing '\0'. putc() is equivalent to fputc()  except  that  it  may  be implemented  as  a  macro which evaluates stream more than once. putchar(c); is equivalent to putc(c,stdout). puts() writes the string s and a trailing newline to  std- out. Calls  to  the  functions described here can be mixed with each other and with calls to other output  functions  from the stdio library for the same output stream.
/*/

#-fread@cstdio
int  fread(  void  *ptr,  size_t  size, size_t nmemb, FILE *stream); int fwrite( void *ptr, size_t  size,  size_t  nmemb,  FILE *stream)
/*-p-
@b:fread, fwrite - binary stream input/output
@r:fread  and  fwrite return the number of items successfully read or written (i.e., not the number of characters).   If an error occurs, or the end-of-file is reached, the return value is a short item count (or zero). fread does not distinguish between end-of-file and  error, and  callers  must  use <A HREF="/man3/feof">feof(3)</A> and <A HREF="/man3/ferror">ferror(3)</A> to determine which occurred.
The function fread reads nmemb elements of data, each size bytes long, from the stream pointed to by stream,  storing them at the location given by ptr. The  function  fwrite  writes nmemb elements of data, each size bytes long, to  the  stream  pointed  to  by  stream, obtaining them from the location given by ptr.
/*/

#-freopen@cstdio
FILE *fopen( char *path, char *mode); FILE *fdopen( int fildes, char *mode); FILE *freopen( char *path, char *mode, FILE *stream)
/*-p-
@b:fopen, fdopen, freopen - stream open functions
@r:Upon  successful  completion  fopen,  fdopen  and  freopen return a FILE pointer.  Otherwise, NULL  is  returned  and the global variable errno is set to indicate the error.
The fopen function opens the file whose name is the string pointed to by path and associates a stream with it. The argument mode points to a string beginning with one of the  following sequences (Additional characters may follow these sequences.): r      Open text file for reading.  The  stream  is  posi- tioned at the beginning of the file. r+     Open  for reading and writing.  The stream is posi- tioned at the beginning of the file. w      Truncate file to zero length or  create  text  file for  writing.   The  stream  is  positioned  at the beginning of the file. w+     Open for reading and writing.  The file is  created if  it  does  not exist, otherwise it is truncated. The stream is positioned at the  beginning  of  the file. a      Open  for  writing.  The file is created if it does not exist.  The stream is positioned at the end  of the file. a+     Open  for reading and writing.  The file is created if it does not exist.  The stream is positioned  at the end of the file. The  mode  string can also include the letter ``b'' either as a third character or as a character between the charac- ters  in any of the two-character strings described above. This is strictly for compatibility with  ANSI  C3.159-1989 (``ANSI  C'')  and  has  no  effect; the ``b'' is ignored. Linux may not behave this way. Any      created      files      will      have       mode S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH (0666), as modified by the process' umask value (see <A HREF="/man2/umask">umask(2)</A>. Reads and writes may be intermixed on  read/write  streams in  any  order, and do not require an intermediate seek as in previous versions of stdio.  This is  not  portable  to other  systems,  however,  and  may  not  work under Linux (someone should find out and fix  this  manpage);  ANSI  C requires   that  a  file  positioning  function  intervene between  output  and  input,  unless  an  input  operation encounters end-of-file. The  fdopen function associates a stream with the existing file descriptor, fildes.  The mode of the stream  must  be compatible with the mode of the file descriptor. The  freopen  function  opens  the  file whose name is the string pointed  to  by  path  and  associates  the  stream pointed  to by stream with it.  The original stream (if it exists) is closed.  The mode argument is used just  as  in the  fopen function.  The primary use of the freopen func- tion is to change the file associated with a standard text stream (stderr, stdin, or stdout).
/*/

#-fseek@cstdio
int fseek( FILE *stream, long offset, int whence); long ftell( FILE *stream); void rewind( FILE *stream); int fgetpos( FILE *stream, fpos_t *pos); int fsetpos( FILE *stream, fpos_t *pos)
/*-p-
@b:fgetpos,  fseek,  fsetpos,  ftell,  rewind  - reposition astream
@r:The  rewind  function  returns  no value.  Upon successful completion, fgetpos, fseek, fsetpos return  0,  and  ftell returns the current offset.  Otherwise, -1 is returned and the global variable errno is set to indicate the error.
The fseek function sets the file  position  indicator  for the  stream  pointed to by stream.  The new position, mea- sured in bytes, is obtained by adding offset bytes to  the position  specified  by  whence.   If  whence  is  set  to SEEK_SET, SEEK_CUR, or SEEK_END, the offset is relative to the  start of the file, the current position indicator, or end-of-file, respectively.  A successful call to the fseek function  clears  the end-of-file indicator for the stream and undoes any effects of the <A HREF="/man3/ungetc">ungetc(3)</A>  function  on  the same stream. The  ftell  function obtains the current value of the file position indicator for the stream pointed to by stream. The rewind function sets the file position  indicator  for the  stream  pointed  to by stream to the beginning of the file.  It is equivalent to: (void)fseek(stream, 0L, SEEK_SET) except that the error indicator for  the  stream  is  also cleared (see <A HREF="/man3/clearerr">clearerr(3)</A>. The fgetpos and fsetpos functions are alternate interfaces equivalent  to  ftell  and  fseek  (with  whence  set   to SEEK_SET),  setting  and  storing the current value of the file offset into or from the object referenced by pos.  On some  non-UNIX  systems  an fpos_t object may be a complex object and these routines may be the only way to  portably reposition a text stream.
/*/

#-fsetpos@cstdio
int fseek( FILE *stream, long offset, int whence); long ftell( FILE *stream); void rewind( FILE *stream); int fgetpos( FILE *stream, fpos_t *pos); int fsetpos( FILE *stream, fpos_t *pos)
/*-p-
@b:fgetpos,  fseek,  fsetpos,  ftell,  rewind  - reposition astream
@r:The  rewind  function  returns  no value.  Upon successful completion, fgetpos, fseek, fsetpos return  0,  and  ftell returns the current offset.  Otherwise, -1 is returned and the global variable errno is set to indicate the error.
The fseek function sets the file  position  indicator  for the  stream  pointed to by stream.  The new position, mea- sured in bytes, is obtained by adding offset bytes to  the position  specified  by  whence.   If  whence  is  set  to SEEK_SET, SEEK_CUR, or SEEK_END, the offset is relative to the  start of the file, the current position indicator, or end-of-file, respectively.  A successful call to the fseek function  clears  the end-of-file indicator for the stream and undoes any effects of the <A HREF="/man3/ungetc">ungetc(3)</A>  function  on  the same stream. The  ftell  function obtains the current value of the file position indicator for the stream pointed to by stream. The rewind function sets the file position  indicator  for the  stream  pointed  to by stream to the beginning of the file.  It is equivalent to: (void)fseek(stream, 0L, SEEK_SET) except that the error indicator for  the  stream  is  also cleared (see <A HREF="/man3/clearerr">clearerr(3)</A>. The fgetpos and fsetpos functions are alternate interfaces equivalent  to  ftell  and  fseek  (with  whence  set   to SEEK_SET),  setting  and  storing the current value of the file offset into or from the object referenced by pos.  On some  non-UNIX  systems  an fpos_t object may be a complex object and these routines may be the only way to  portably reposition a text stream.
/*/

#-ftell@cstdio
int fseek( FILE *stream, long offset, int whence); long ftell( FILE *stream); void rewind( FILE *stream); int fgetpos( FILE *stream, fpos_t *pos); int fsetpos( FILE *stream, fpos_t *pos)
/*-p-
@b:fgetpos,  fseek,  fsetpos,  ftell,  rewind  - reposition astream
@r:The  rewind  function  returns  no value.  Upon successful completion, fgetpos, fseek, fsetpos return  0,  and  ftell returns the current offset.  Otherwise, -1 is returned and the global variable errno is set to indicate the error.
The fseek function sets the file  position  indicator  for the  stream  pointed to by stream.  The new position, mea- sured in bytes, is obtained by adding offset bytes to  the position  specified  by  whence.   If  whence  is  set  to SEEK_SET, SEEK_CUR, or SEEK_END, the offset is relative to the  start of the file, the current position indicator, or end-of-file, respectively.  A successful call to the fseek function  clears  the end-of-file indicator for the stream and undoes any effects of the <A HREF="/man3/ungetc">ungetc(3)</A>  function  on  the same stream. The  ftell  function obtains the current value of the file position indicator for the stream pointed to by stream. The rewind function sets the file position  indicator  for the  stream  pointed  to by stream to the beginning of the file.  It is equivalent to: (void)fseek(stream, 0L, SEEK_SET) except that the error indicator for  the  stream  is  also cleared (see <A HREF="/man3/clearerr">clearerr(3)</A>. The fgetpos and fsetpos functions are alternate interfaces equivalent  to  ftell  and  fseek  (with  whence  set   to SEEK_SET),  setting  and  storing the current value of the file offset into or from the object referenced by pos.  On some  non-UNIX  systems  an fpos_t object may be a complex object and these routines may be the only way to  portably reposition a text stream.
/*/

#-fwrite@cstdio
int  fread(  void  *ptr,  size_t  size, size_t nmemb, FILE *stream); int fwrite( void *ptr, size_t  size,  size_t  nmemb,  FILE *stream)
/*-p-
@b:fread, fwrite - binary stream input/output
@r:fread  and  fwrite return the number of items successfully read or written (i.e., not the number of characters).   If an error occurs, or the end-of-file is reached, the return value is a short item count (or zero). fread does not distinguish between end-of-file and  error, and  callers  must  use <A HREF="/man3/feof">feof(3)</A> and <A HREF="/man3/ferror">ferror(3)</A> to determine which occurred.
The function fread reads nmemb elements of data, each size bytes long, from the stream pointed to by stream,  storing them at the location given by ptr. The  function  fwrite  writes nmemb elements of data, each size bytes long, to  the  stream  pointed  to  by  stream, obtaining them from the location given by ptr.
/*/

#-getc@cstdio
int fgetc(FILE *stream); char *fgets(char *s, int size, FILE *stream); int getc(FILE *stream); int getchar(void); char *gets(char *s); int ungetc(int c, FILE *stream)
/*-p-
@b:fgetc, fgets, getc, getchar, gets, ungetc - input of char-acters and strings
@r:fgetc(), getc() and getchar() return the character read as an  unsigned  char cast to an int or EOF on end of file or error. gets() and fgets() return s on success, and NULL on end of file or error. ungetc() returns c on success, or EOF on error. ANSI - C, POSIX.1 Because  it is impossible to tell without knowing the data in advance how  many  characters  gets()  will  read,  and because  gets() will continue to store characters past the end of the buffer, it is extremely dangerous to  use.   It has  been  used  to  break computer security.  Use fgets() instead. It is not advisable to mix calls to input  functions  from the stdio library with low - level calls to read() for the file descriptor associated  with  the  input  stream;  the results  will  be undefined and very probably not what you want.
fgetc() reads the next character from stream  and  returns it  as  an  unsigned char cast to an int, or EOF on end of file or error. getc() is equivalent to fgetc()  except  that  it  may  be implemented  as  a  macro which evaluates stream more than once. getchar() is equivalent to getc(stdin). gets() reads a line from stdin into the buffer pointed  to by  s  until either a terminating newline or EOF, which it replaces with '\0'.  No check for buffer overrun  is  per- formed (see BUGS below). fgets()  reads  in at most one less than n characters from stream and stores them into the buffer pointed  to  by  s. Reading  stops after an EOF or a newline.  If a newline is read, it is stored into the  buffer.   A  '\0'  is  stored after the last character in the buffer. ungetc()  pushes  c back to stream, cast to unsigned char, where it is  available  for  subsequent  read  operations. Pushed  -  back  characters  will  be  returned in reverse order; only one pushback is guaranteed. Calls to the functions described here can  be  mixed  with each  other  and  with calls to other input functions from the stdio library for the same input stream.
/*/

#-getchar@cstdio
int fgetc(FILE *stream); char *fgets(char *s, int size, FILE *stream); int getc(FILE *stream); int getchar(void); char *gets(char *s); int ungetc(int c, FILE *stream)
/*-p-
@b:fgetc, fgets, getc, getchar, gets, ungetc - input of char-acters and strings
@r:fgetc(), getc() and getchar() return the character read as an  unsigned  char cast to an int or EOF on end of file or error. gets() and fgets() return s on success, and NULL on end of file or error. ungetc() returns c on success, or EOF on error. ANSI - C, POSIX.1 Because  it is impossible to tell without knowing the data in advance how  many  characters  gets()  will  read,  and because  gets() will continue to store characters past the end of the buffer, it is extremely dangerous to  use.   It has  been  used  to  break computer security.  Use fgets() instead. It is not advisable to mix calls to input  functions  from the stdio library with low - level calls to read() for the file descriptor associated  with  the  input  stream;  the results  will  be undefined and very probably not what you want.
fgetc() reads the next character from stream  and  returns it  as  an  unsigned char cast to an int, or EOF on end of file or error. getc() is equivalent to fgetc()  except  that  it  may  be implemented  as  a  macro which evaluates stream more than once. getchar() is equivalent to getc(stdin). gets() reads a line from stdin into the buffer pointed  to by  s  until either a terminating newline or EOF, which it replaces with '\0'.  No check for buffer overrun  is  per- formed (see BUGS below). fgets()  reads  in at most one less than n characters from stream and stores them into the buffer pointed  to  by  s. Reading  stops after an EOF or a newline.  If a newline is read, it is stored into the  buffer.   A  '\0'  is  stored after the last character in the buffer. ungetc()  pushes  c back to stream, cast to unsigned char, where it is  available  for  subsequent  read  operations. Pushed  -  back  characters  will  be  returned in reverse order; only one pushback is guaranteed. Calls to the functions described here can  be  mixed  with each  other  and  with calls to other input functions from the stdio library for the same input stream.
/*/

#-gets@cstdio
int fgetc(FILE *stream); char *fgets(char *s, int size, FILE *stream); int getc(FILE *stream); int getchar(void); char *gets(char *s); int ungetc(int c, FILE *stream)
/*-p-
@b:fgetc, fgets, getc, getchar, gets, ungetc - input of char-acters and strings
@r:fgetc(), getc() and getchar() return the character read as an  unsigned  char cast to an int or EOF on end of file or error. gets() and fgets() return s on success, and NULL on end of file or error. ungetc() returns c on success, or EOF on error. ANSI - C, POSIX.1 Because  it is impossible to tell without knowing the data in advance how  many  characters  gets()  will  read,  and because  gets() will continue to store characters past the end of the buffer, it is extremely dangerous to  use.   It has  been  used  to  break computer security.  Use fgets() instead. It is not advisable to mix calls to input  functions  from the stdio library with low - level calls to read() for the file descriptor associated  with  the  input  stream;  the results  will  be undefined and very probably not what you want.
fgetc() reads the next character from stream  and  returns it  as  an  unsigned char cast to an int, or EOF on end of file or error. getc() is equivalent to fgetc()  except  that  it  may  be implemented  as  a  macro which evaluates stream more than once. getchar() is equivalent to getc(stdin). gets() reads a line from stdin into the buffer pointed  to by  s  until either a terminating newline or EOF, which it replaces with '\0'.  No check for buffer overrun  is  per- formed (see BUGS below). fgets()  reads  in at most one less than n characters from stream and stores them into the buffer pointed  to  by  s. Reading  stops after an EOF or a newline.  If a newline is read, it is stored into the  buffer.   A  '\0'  is  stored after the last character in the buffer. ungetc()  pushes  c back to stream, cast to unsigned char, where it is  available  for  subsequent  read  operations. Pushed  -  back  characters  will  be  returned in reverse order; only one pushback is guaranteed. Calls to the functions described here can  be  mixed  with each  other  and  with calls to other input functions from the stdio library for the same input stream.
/*/

#-perror@cstdio
void perror(const char *s)
/*-p-
@b:perror - print a system error message
@r:
perror  produces  a  message on the standard error output, describing the last error encountered during a call  to  a system  or  library  function.   The  argument string s is printed first, then a colon and a blank, then the  message and  a  new-line.   To be of most use, the argument string should include the name of the program that  incurred  the error.   The error number is taken from the external vari- able errno, which is set when errors occur but not cleared when non-erroneous calls are made.
/*/

#=printf,  fprintf,  sprintf,  vprintf, vfprintf, vsprintf@cstdio
int printf( const char *format, ...); int fprintf( FILE *stream, const char *format, ...); int sprintf( char *str, const char *format, ...);  int vprintf( const char *format, va_list ap); int vfprintf( FILE *stream, const  char  *format,  va_list ap); int vsprintf( char *str, char *format, va_list ap);</PRE>
/*-p-
@b:printf,  fprintf,  sprintf,  vprintf, vfprintf, vsprintf -formatted output conversion
@r:
The  printf  family of functions produces output according to a format as described below.  Printf and vprintf  write output  to stdout, the standard output stream; fprintf and vfprintf write output to the given output stream; sprintf, and  vsprintf  write  to  the character string str.  These functions write the output under the control of  a  format string  that  specifies how subsequent arguments (or argu- ments accessed via the variable-length argument facilities of  <A HREF="/man3/stdarg">stdarg(3)</A>  are  converted for output.  These functions return the number of characters printed (not including the trailing `\0' used to end output to strings). The  format string is composed of zero or more directives: ordinary characters (not %), which are copied unchanged to the  output stream; and conversion specifications, each of which results in fetching zero or  more  subsequent  argu- ments.  Each conversion specification is introduced by the character  %.   The  arguments  must  correspond  properly (after  type  promotion)  with  the  conversion specifier. After the %, the following appear in sequence: o      Zero or more of the following flags: #      specifying that the  value  should  be  con- verted  to an ``alternate form''.  For c, d, i, n, p, s, and u conversions,  this  option has  no effect.  For o conversions, the pre- cision of the number is increased  to  force the  first character of the output string to a zero (except if a zero  value  is  printed with  an explicit precision of zero).  For x and X conversions, a non-zero result has the string  `0x'  (or  `0X'  for  X conversions) prepended to it.  For e, E, f, g, and G con- versions,  the  result will always contain a decimal point, even if no digits  follow  it (normally,  a  decimal  point appears in the results of those conversions only if a digit follows).  For g and G conversions, trailing zeros are not removed  from  the  result  as they would otherwise be. 0      specifying  zero  padding.   For all conver- sions  except  n,  the  converted  value  is padded  on  the  left with zeros rather than blanks.  If a  precision  is  given  with  a numeric  conversion  (d,  i, o, u, i, x, and X), the 0 flag is ignored. -      (a negative field width flag) indicates  the converted  value  is  to be left adjusted on the field boundary.  Except  for  n  conver- sions,  the converted value is padded on the right with blanks, rather than on  the  left with  blanks or zeros.  A - overrides a 0 if both are given. (a space) specifying that a blank should  be left  before a positive number produced by a signed conversion (d, e, E, f, g, G, or  i). +      specifying  that  a  sign  always  be placed before a number produced by a signed conver- sion.   A  +  overrides  a space if both are used. o      An optional decimal digit string specifying a mini- mum  field width.  If the converted value has fewer characters than the field width, it will be  padded with  spaces  on  the  left (or right, if the left- adjustment flag has been given)  to  fill  out  the field width. o      An  optional  precision,  in  the  form of a period (`.')  followed by an optional  digit  string.   If the digit string is omitted, the precision is taken as zero.  This gives the minimum number  of  digits to appear for d, i, o, u, x, and X conversions, the number of digits to appear after the  decimal-point for  e, E, and f conversions, the maximum number of significant digits for g and G conversions, or  the maximum  number  of characters to be printed from a string for s conversions. o      The optional character h, specifying that a follow- ing d, i, o, u, x, or X conversion corresponds to a short int or unsigned short int argument, or that a following  n conversion corresponds to a pointer to a short int argument. o      The optional character l (ell)  specifying  that  a following d, i, o, u, x, or X conversion applies to a pointer to a long int or unsigned long int  argu- ment,  or that a following n conversion corresponds to a pointer to a long int argument. o      The character L specifying that a following  e,  E, f,  g, or G conversion corresponds to a long double argument. o      A character that specifies the type  of  conversion to be applied. A  field  width or precision, or both, may be indicated by an asterisk `*' instead of a digit string.  In this  case, an  int argument supplies the field width or precision.  A negative field width is treated as a left adjustment  flag followed  by  a positive field width; a negative precision is treated as though it were missing. The conversion specifiers and their meanings are: diouxX The int (or appropriate variant) argument  is  con- verted  to signed decimal (d and i), unsigned octal (o, unsigned decimal (u, or unsigned hexadecimal (x and X) notation.  The letters abcdef are used for x conversions; the letters ABCDEF are used for X con- versions.  The precision, if any, gives the minimum number of digits that must appear; if the converted value  requires  fewer  digits, it is padded on the left with zeros. DOU    The long int argument is converted to signed  deci- mal, unsigned octal, or unsigned decimal, as if the format had been ld, lo, or lu respectively.   These conversion  characters  are  deprecated,  and  will eventually disappear. eE     The double argument is rounded and converted in the style  [-]d.dddedd  where there is one digit before the decimal-point character and the number of  dig- its after it is equal to the precision; if the pre- cision is missing, it is taken as 6; if the  preci- sion  is  zero, no decimal-point character appears. An E conversion uses the letter E (rather  than  e) to  introduce  the  exponent.   The exponent always contains at least two digits; if the value is zero, the exponent is 00. f      The  double  argument  is  rounded and converted to decimal notation in the style [-]ddd.ddd, where the number  of digits after the decimal-point character is equal to the precision  specification.   If  the precision  is  missing,  it  is  taken as 6; if the precision  is  explicitly  zero,  no  decimal-point character  appears.  If a decimal point appears, at least one digit appears before it. g      The double argument is converted in style  f  or  e (or  E for G conversions).  The precision specifies the number of significant digits.  If the precision is missing, 6 digits are given; if the precision is zero, it is treated as 1.  Style e is used  if  the exponent  from  its  conversion  is less than -4 or greater than or equal to the  precision.   Trailing zeros  are  removed from the fractional part of the result; a decimal point appears only if it is  fol- lowed by at least one digit. c      The  int argument is converted to an unsigned char, and the resulting character is written. s      The ``char *'' argument is expected to be a pointer to  an  array  of  character  type  (pointer  to  a string).  Characters from the array are written  up to (but not including) a terminating NUL character; if a precision is specified, no more than the  num- ber  specified  are  written.   If  a  precision is given, no null character need be  present;  if  the precision  is not specified, or is greater than the size of the array, the array must contain a  termi- nating NUL character. p      The  ``void *'' pointer argument is printed in hex- adecimal (as if by %#x or %#lx). n      The number of characters written so far  is  stored into  the  integer  indicated  by the ``int *'' (or variant) pointer argument.   No  argument  is  con- verted. %      A  `%'  is  written.  No argument is converted. The complete conversion specification is `%%'. In no case does a non-existent or small field width  cause truncation  of  a  field; if the result of a conversion is wider than the field width, the field is expanded to  con- tain the conversion result.
/*/

#-putc@cstdio
int fputc(int c, FILE *stream); int fputs(const char *s, FILE *stream); int putc(int c, FILE *stream); int putchar(int c); int puts(char *s); int ungetc(int c, FILE *stream)
/*-p-
@b:fputc,  fputs,  putc, putchar, puts - output of charactersand strings
@r:fputc(), putc() and putchar() return the character written as an unsigned char cast to an int or EOF on error. puts() and fputs() return a non - negative number on  suc- cess, or EOF on error. ANSI - C, POSIX.1 It  is not advisable to mix calls to output functions from the stdio library with low - level calls  to  write()  for the  file  descriptor  associated  with  the  same  output stream; the results will be undefined  and  very  probably not what you want.
fputc() writes the character c, cast to an unsigned  char, to stream. fputs()  writes the string s to stream, without its trail- ing '\0'. putc() is equivalent to fputc()  except  that  it  may  be implemented  as  a  macro which evaluates stream more than once. putchar(c); is equivalent to putc(c,stdout). puts() writes the string s and a trailing newline to  std- out. Calls  to  the  functions described here can be mixed with each other and with calls to other output  functions  from the stdio library for the same output stream.
/*/

#-putchar@cstdio
int fputc(int c, FILE *stream); int fputs(const char *s, FILE *stream); int putc(int c, FILE *stream); int putchar(int c); int puts(char *s); int ungetc(int c, FILE *stream)
/*-p-
@b:fputc,  fputs,  putc, putchar, puts - output of charactersand strings
@r:fputc(), putc() and putchar() return the character written as an unsigned char cast to an int or EOF on error. puts() and fputs() return a non - negative number on  suc- cess, or EOF on error. ANSI - C, POSIX.1 It  is not advisable to mix calls to output functions from the stdio library with low - level calls  to  write()  for the  file  descriptor  associated  with  the  same  output stream; the results will be undefined  and  very  probably not what you want.
fputc() writes the character c, cast to an unsigned  char, to stream. fputs()  writes the string s to stream, without its trail- ing '\0'. putc() is equivalent to fputc()  except  that  it  may  be implemented  as  a  macro which evaluates stream more than once. putchar(c); is equivalent to putc(c,stdout). puts() writes the string s and a trailing newline to  std- out. Calls  to  the  functions described here can be mixed with each other and with calls to other output  functions  from the stdio library for the same output stream.
/*/

#-puts@cstdio
int fputc(int c, FILE *stream); int fputs(const char *s, FILE *stream); int putc(int c, FILE *stream); int putchar(int c); int puts(char *s); int ungetc(int c, FILE *stream)
/*-p-
@b:fputc,  fputs,  putc, putchar, puts - output of charactersand strings
@r:fputc(), putc() and putchar() return the character written as an unsigned char cast to an int or EOF on error. puts() and fputs() return a non - negative number on  suc- cess, or EOF on error. ANSI - C, POSIX.1 It  is not advisable to mix calls to output functions from the stdio library with low - level calls  to  write()  for the  file  descriptor  associated  with  the  same  output stream; the results will be undefined  and  very  probably not what you want.
fputc() writes the character c, cast to an unsigned  char, to stream. fputs()  writes the string s to stream, without its trail- ing '\0'. putc() is equivalent to fputc()  except  that  it  may  be implemented  as  a  macro which evaluates stream more than once. putchar(c); is equivalent to putc(c,stdout). puts() writes the string s and a trailing newline to  std- out. Calls  to  the  functions described here can be mixed with each other and with calls to other output  functions  from the stdio library for the same output stream.
/*/

#-remove@cstdio
int remove(const char *pathname)
/*-p-
@b:remove - delete a name and possibly the file it refers to
@r:On success, zero is returned.  On error, -1  is  returned, and errno is set appropriately.
remove  deletes  a  name from the filesystem. If that name was the last link to a file and no processes have the file open  the  file  is  deleted and the space it was using is made available for reuse. If the name was the last link to a file but any  processes still have the file open the file will remain in existence until the last file descriptor referring to it is  closed. If  the  name  referred  to  a  symbolic  link the link is removed. If the name referred to a socket, fifo or device the  name for it is removed but processes which have the object open may continue to use it.
/*/

#-rewind@cstdio
int fseek( FILE *stream, long offset, int whence); long ftell( FILE *stream); void rewind( FILE *stream); int fgetpos( FILE *stream, fpos_t *pos); int fsetpos( FILE *stream, fpos_t *pos)
/*-p-
@b:fgetpos,  fseek,  fsetpos,  ftell,  rewind  - reposition astream
@r:The  rewind  function  returns  no value.  Upon successful completion, fgetpos, fseek, fsetpos return  0,  and  ftell returns the current offset.  Otherwise, -1 is returned and the global variable errno is set to indicate the error.
The fseek function sets the file  position  indicator  for the  stream  pointed to by stream.  The new position, mea- sured in bytes, is obtained by adding offset bytes to  the position  specified  by  whence.   If  whence  is  set  to SEEK_SET, SEEK_CUR, or SEEK_END, the offset is relative to the  start of the file, the current position indicator, or end-of-file, respectively.  A successful call to the fseek function  clears  the end-of-file indicator for the stream and undoes any effects of the <A HREF="/man3/ungetc">ungetc(3)</A>  function  on  the same stream. The  ftell  function obtains the current value of the file position indicator for the stream pointed to by stream. The rewind function sets the file position  indicator  for the  stream  pointed  to by stream to the beginning of the file.  It is equivalent to: (void)fseek(stream, 0L, SEEK_SET) except that the error indicator for  the  stream  is  also cleared (see <A HREF="/man3/clearerr">clearerr(3)</A>. The fgetpos and fsetpos functions are alternate interfaces equivalent  to  ftell  and  fseek  (with  whence  set   to SEEK_SET),  setting  and  storing the current value of the file offset into or from the object referenced by pos.  On some  non-UNIX  systems  an fpos_t object may be a complex object and these routines may be the only way to  portably reposition a text stream.
/*/

#-scanf,  fscanf,  sscanf,  vscanf, vsscanf, vfscanf@cstdio
int scanf( const char *format, ...); int fscanf( FILE *stream, const char *format, ...);       int sscanf( const char *str, const char *format, ...<  int vscanf( const char *format, va_list ap); int vsscanf( const char *str, const char *format, va_list ap); int vfscanf( FILE *stream, const char *format, va_list ap)
/*-p-
@b:scanf,  fscanf,  sscanf,  vscanf, vsscanf, vfscanf - inputformat conversion
@r:These functions return the number of input items assigned, which can be fewer than provided for, or even zero, in the event  of  a matching failure.  Zero indicates that, while there was input available, no conversions  were  assigned; typically  this is due to an invalid input character, such as an alphabetic character for  a  `%d'  conversion.   The value  EOF  is  returned if an input failure occurs before any conversion such as an end-of-file occurs. If an  error or end-of-file occurs after conversion has begun, the num- ber of conversions which were  successfully  completed  is returned.
The scanf family of functions scans input according  to  a format  as  described below.  This format may contain con- version specifiers; the results from such conversions,  if any,  are stored through the pointer arguments.  The scanf function reads input from the standard input stream stdin, fscanf  reads  input  from  the stream pointer stream, and sscanf reads its input from the character  string  pointed to by str. The vfscanf function is analogous to <A HREF="/man3/vfprintf">vfprintf(3)</A> and reads input from the stream  pointer  stream  using  a  variable argument  list  of  pointers  (see  <A HREF="/man3/stdarg">stdarg(3)</A>.  The vscanf function scans a variable argument list from the  standard input  and  the  vsscanf  function scans it from a string; these are analogous to the vprintf and vsprintf  functions respectively. Each  successive pointer argument must correspond properly with each successive conversion specifier (but  see  `sup- pression' below).  All conversions are introduced by the % (percent sign) character.  The format string may also con- tain other characters.  White space (such as blanks, tabs, or newlines) in the format  string  match  any  amount  of white  space,  including  none,  in the input.  Everything else matches only itself.  Scanning stops  when  an  input character  does  not match such a format character.  Scan- ning also stops when an input conversion  cannot  be  made (see below). Following  the  % character introducing a conversion there may be a number of flag characters, as follows: *      Suppresses assignment.  The conversion that follows occurs as usual, but no pointer is used; the result of the conversion is simply discarded. h      Indicates that the conversion will be one of  dioux or  n  and the next pointer is a pointer to a short int (rather than int). l      Indicates either that the conversion will be one of dioux  or  n and the next pointer is a pointer to a long int (rather than int), or that the  conversion will  be  one  of  efg  and  the  next pointer is a pointer to double (rather than float). L      Indicates that the conversion will be efg  and  the next  pointer  is  a pointer to long double.  (This type is not implemented; the L  flag  is  currently ignored--this may not be true for Linux.) In addition to these flags, there may be an optional maxi- mum field width, expressed as a decimal  integer,  between the % and the conversion.  If no width is given, a default of `infinity' is used (with one exception, below);  other- wise  at most this many characters are scanned in process- ing the conversion.  Before conversion begins,  most  con- versions skip white space; this white space is not counted against the field width. The following conversions are available: %      Matches a literal `%'.  That is, `%%' in the format string  matches  a  single input `%' character.  No conversion is done, and assignment does not  occur. d      Matches  an  optionally signed decimal integer; the next pointer must be a pointer to int. D      Equivalent to ld; this exists  only  for  backwards compatibility. i      Matches  an  optionally  signed  integer;  the next pointer must be a pointer to int.  The  integer  is read  in base 16 if it begins with `0x' or `0X', in base 8 if it begins with `0', and in base 10 other- wise.   Only characters that correspond to the base are used. o      Matches an octal integer; the next pointer must  be a pointer to unsigned int. O      Equivalent to lo; this exists for backwards compat- ibility. u      Matches an optionally signed decimal  integer;  the next pointer must be a pointer to unsigned int. x      Matches an optionally a signed hexadecimal integer; the next pointer must be a pointer to unsigned int. X      Equivalent   to   lx;   this   violates   the  ANSI C3.159-1989 (``ANSI C'') but is backwards  compati- ble  with  previous UNIX systems--I don't know what Linux does with this. f      Matches an optionally signed floating-point number; the next pointer must be a pointer to float. e      Equivalent to f. g      Equivalent to f. E      Equivalent   to   lf;   this   violates   the  ANSI C3.159-1989 (``ANSI C'') but is backwards  compati- ble  with  previous UNIX systems--I don't know what Linux does with this. F      Equivalent to lf; this exists  only  for  backwards compatibility. s      Matches  a  sequence of non-white-space characters; the next pointer must be a pointer to char, and the array  must  be  large  enough  to  accept  all the sequence and the terminating  NUL  character.   The input string stops at white space or at the maximum field width, whichever occurs first. c      Matches  a  sequence  of  width  count   characters (default  1); the next pointer must be a pointer to char, and there must be enough  room  for  all  the characters  (no  terminating  NUL  is  added).  The usual skip of leading white  space  is  suppressed. To skip white space first, use an explicit space in the format. [      Matches a nonempty sequence of characters from  the specified  set  of  accepted  characters;  the next pointer must be a pointer to char, and  there  must be  enough  room  for  all  the  characters  in the string, plus  a  terminating  NUL  character.   The usual  skip  of  leading white space is suppressed. The string is to be made up of  characters  in  (or not in) a particular set; the set is defined by the characters between the open bracket [ character and a  close  bracket  ]  character.   The set excludes those characters if the first character  after  the open bracket is a circumflex ^.  To include a close bracket in the set, make  it  the  first  character after the open bracket or the circumflex; any other position will end the set.  The hyphen character  - is  also  special;  when  placed  between two other characters, it adds all intervening  characters  to the  set.   To  include  a hyphen, make it the last character before  the  final  close  bracket.   For instance,  `[^]0-9-]'  means  the  set  `everything except  close  bracket,  zero  through  nine,   and hyphen'.   The string ends with the appearance of a character not in the (or, with  a  circumflex,  in) set or when the field width runs out. p      Matches  a  pointer  value  (as  printed by `%p' in <A HREF="/man3/printf">printf(3)</A>; the next pointer must be  a  pointer  to void. n      Nothing is expected; instead, the number of charac- ters consumed thus far from  the  input  is  stored through  the  next pointer, which must be a pointer to int.  This is not a conversion, although it  can be suppressed with the * flag. For  backwards  compatibility, other conversion characters (except `\0') are taken as if they were `%d' or, if upper- case, `%ld', and a `conversion' of `%\0' causes an immedi- ate return of EOF.   The  F  and  X  conversions  will  be changed in the future to conform to the ANSI  C  standard,  after  which  they  will act like and respectively.  The behavior of Linux on  the  non-standard points is not known by this documenter.
/*/

#-setbuf@cstdio
int setbuf( FILE *stream, char *buf); int setbuffer( FILE *stream, char *buf, size_tsize); int setlinebuf( FILE *stream); int setvbuf( FILE *stream, char *buf, int mode , size_t size)
/*-p-
@b:setbuf,  setbuffer, setlinebuf, setvbuf - stream bufferingoperations
@r:
The  three  types  of  buffering available are unbuffered, block buffered, and line buffered.  When an output  stream is unbuffered, information appears on the destination file or terminal as soon as written; when it is block  buffered many  characters are saved up and written as a block; when it is line buffered characters are saved up until  a  new- line  is  output or input is read from any stream attached to a terminal  device  (typically  stdin).   The  function <A HREF="/man3/fflush">fflush(3)</A>  may be used to force the block out early.  (See <A HREF="/man3/fclose">fclose(3)</A>.)  Normally all files are block buffered.   When the  first  I/O  operation  occurs on a file, <A HREF="/man3/malloc">malloc(3)</A> is called, and a buffer is obtained.  If a stream refers to a terminal  (as  stdout  normally does) it is line buffered. The standard error stream stderr is always unbuffered. The setvbuf function may be used at any time on  any  open stream  to  change its buffer.  The mode parameter must be one of the following three macros: _IONBF unbuffered _IOLBF line buffered _IOFBF fully buffered Except for unbuffered files, the buf argument should point to  a buffer at least size bytes long; this buffer will be used instead of the current buffer.  If the  argument  buf is  NULL,  only the mode is affected; a new buffer will be allocated on  the  next  read  or  write  operation.   The setvbuf  function  may  be  used at any time, but can only change the mode of a stream when  it  is  not  ``active'': that  is,  before  any I/O, or immediately after a call to fflush. The other three calls are, in effect, simply  aliases  for calls  to setvbuf.  The setbuf function is exactly equiva- lent to the call setvbuf(stream,  buf,  buf  ?  _IOFBF   :   _IONBF, BUFSIZ); The  setbuffer  function is the same, except that the size of the buffer is up  to  the  caller,  rather  than  being determined by the default BUFSIZ.  The setlinebuf function is exactly equivalent to the call: setvbuf(stream, (char *)NULL, _IOLBF, 0);
/*/

#-setvbuf@cstdio
int setbuf( FILE *stream, char *buf); int setbuffer( FILE *stream, char *buf, size_tsize); int setlinebuf( FILE *stream); int setvbuf( FILE *stream, char *buf, int mode , size_t size)
/*-p-
@b:setbuf,  setbuffer, setlinebuf, setvbuf - stream bufferingoperations
@r:
The  three  types  of  buffering available are unbuffered, block buffered, and line buffered.  When an output  stream is unbuffered, information appears on the destination file or terminal as soon as written; when it is block  buffered many  characters are saved up and written as a block; when it is line buffered characters are saved up until  a  new- line  is  output or input is read from any stream attached to a terminal  device  (typically  stdin).   The  function <A HREF="/man3/fflush">fflush(3)</A>  may be used to force the block out early.  (See <A HREF="/man3/fclose">fclose(3)</A>.)  Normally all files are block buffered.   When the  first  I/O  operation  occurs on a file, <A HREF="/man3/malloc">malloc(3)</A> is called, and a buffer is obtained.  If a stream refers to a terminal  (as  stdout  normally does) it is line buffered. The standard error stream stderr is always unbuffered. The setvbuf function may be used at any time on  any  open stream  to  change its buffer.  The mode parameter must be one of the following three macros: _IONBF unbuffered _IOLBF line buffered _IOFBF fully buffered Except for unbuffered files, the buf argument should point to  a buffer at least size bytes long; this buffer will be used instead of the current buffer.  If the  argument  buf is  NULL,  only the mode is affected; a new buffer will be allocated on  the  next  read  or  write  operation.   The setvbuf  function  may  be  used at any time, but can only change the mode of a stream when  it  is  not  ``active'': that  is,  before  any I/O, or immediately after a call to fflush. The other three calls are, in effect, simply  aliases  for calls  to setvbuf.  The setbuf function is exactly equiva- lent to the call setvbuf(stream,  buf,  buf  ?  _IOFBF   :   _IONBF, BUFSIZ); The  setbuffer  function is the same, except that the size of the buffer is up  to  the  caller,  rather  than  being determined by the default BUFSIZ.  The setlinebuf function is exactly equivalent to the call: setvbuf(stream, (char *)NULL, _IOLBF, 0);
/*/

#-tmpfile@cstdio
FILE *tmpfile (void)
/*-p-
@b:tmpfile - create a temporary file
@r:The  tmpfile()  function  returns  a stream descriptor, or NULL if a unique  filename  cannot  be  generated  or  the unique file cannot be opened.
The  tmpfile() function generates a unique temporary file- name using the path prefix P_tmpdir defined in  &lt;stdio.h&gt;. The  temporary  file  is  then opened in binary read/write (w+b) mode.  The file will be automatically  deleted  when it is closed or the program terminates.
/*/

#-tmpnam@cstdio
char *tmpnam(char *s)
/*-p-
@b:tmpnam - create a name for a temporary file
@r:The tmpnam() function returns a pointer to the unique tem- porary filename, or NULL if a unique name cannot be gener- ated.
The  tmpnam()  function generates a unique temporary file- name using the path prefix P_tmpdir defined in  &lt;stdio.h&gt;. If the argument s is NULL, tmpnam() returns the address of an internal static area which holds the filename, which is overwritten  by subsequent calls to tmpnam().  If s is not NULL, the filename is returned in s.
/*/

#-ungetc@cstdio
int fgetc(FILE *stream); char *fgets(char *s, int size, FILE *stream); int getc(FILE *stream); int getchar(void); char *gets(char *s); int ungetc(int c, FILE *stream)
/*-p-
@b:fgetc, fgets, getc, getchar, gets, ungetc - input of char-acters and strings
@r:fgetc(), getc() and getchar() return the character read as an  unsigned  char cast to an int or EOF on end of file or error. gets() and fgets() return s on success, and NULL on end of file or error. ungetc() returns c on success, or EOF on error. ANSI - C, POSIX.1 Because  it is impossible to tell without knowing the data in advance how  many  characters  gets()  will  read,  and because  gets() will continue to store characters past the end of the buffer, it is extremely dangerous to  use.   It has  been  used  to  break computer security.  Use fgets() instead. It is not advisable to mix calls to input  functions  from the stdio library with low - level calls to read() for the file descriptor associated  with  the  input  stream;  the results  will  be undefined and very probably not what you want.
fgetc() reads the next character from stream  and  returns it  as  an  unsigned char cast to an int, or EOF on end of file or error. getc() is equivalent to fgetc()  except  that  it  may  be implemented  as  a  macro which evaluates stream more than once. getchar() is equivalent to getc(stdin). gets() reads a line from stdin into the buffer pointed  to by  s  until either a terminating newline or EOF, which it replaces with '\0'.  No check for buffer overrun  is  per- formed (see BUGS below). fgets()  reads  in at most one less than n characters from stream and stores them into the buffer pointed  to  by  s. Reading  stops after an EOF or a newline.  If a newline is read, it is stored into the  buffer.   A  '\0'  is  stored after the last character in the buffer. ungetc()  pushes  c back to stream, cast to unsigned char, where it is  available  for  subsequent  read  operations. Pushed  -  back  characters  will  be  returned in reverse order; only one pushback is guaranteed. Calls to the functions described here can  be  mixed  with each  other  and  with calls to other input functions from the stdio library for the same input stream.
/*/

##:cstdlib@cstdlib
#-abort@cstdlib
void abort(void)
/*-p-
@b:abort - cause abnormal program termination
@r:The abort() function never returns.
The  abort()  function causes abnormal program termination unless the signal SIGABORT is caught and the  signal  han- dler does not return.  If the abort() function causes pro- gram termination, all open streams are closed and flushed. If  the  SIGABORT  function  is  blocked  or  ignored, the abort() function will still override it.
/*/

#-abs@cstdlib
int abs(int j)
/*-p-
@b:abs - computes the absolute value of an integer.
@r:Returns the absolute value of the integer argument.
The  abs()  function  computes  the  absolute value of the integer argument j.
/*/

#-atexit@cstdlib
int atexit(void (*function")(void))
/*-p-
@b:atexit  -  register a function to be called at normal pro-gram termination.
@r:The atexit() function returns the value 0  if  successful; otherwise the value -1 is returned and the global variable errno is set to indicate the error.
The atexit() function registers the given function  to  be called  at normal program termination, whether via <A HREF="/man2/exit">exit(2)</A> or via return from the program's main.  Functions so  reg- istered are called in the reverse order of their registra- tion; no arguments are passed.
/*/

#-atof@cstdlib
double atof(const char *nptr)
/*-p-
@b:atof - convert a string to a double.
@r:The converted value.
The  atof()  function  converts the initial portion of the string pointed to by nptr to double.  The behaviour is the same as strtod(nptr, (char **)NULL); except that atof() does not detect errors.
/*/

#-atoi@cstdlib
int atoi(const char *nptr)
/*-p-
@b:atoi - convert a string to an integer.
@r:The converted value.
The  atoi()  function  converts the initial portion of the string pointed to by nptr to int.  The  behaviour  is  the same as strtol(nptr, (char **)NULL, 10); except that atoi() does not detect errors.
/*/

#-atol@cstdlib
long atol(const char *nptr)
/*-p-
@b:atol - convert a string to a long integer.
@r:The converted value.
The  atol()  function  converts the initial portion of the string pointed to by nptr to long.  The behaviour  is  the same as strtol(nptr, (char **)NULL, 10); except that atol() does not detect errors.
/*/

#-bsearch@cstdlib
void *bsearch(const void *key, const void *base, size_t nmemb, size_t size, int (*compar)(const void *, const void *))
/*-p-
@b:bsearch - binary search of a sorted array.
@r:The  bsearch()  function  returns  a pointer to a matching member of the array, or NULL if no  match  is  found.   If there  are  multiple elements that match the key, the ele- ment returned is unspecified.
The bsearch() function searches an array of nmemb objects, the initial member of which is pointed to by base,  for  a member  that  matches  the  object pointed to by key.  The size of each member of the array is specified by size. The contents of the array should be  in  ascending  sorted order  according  to the comparison function referenced by compar.  The compar routine is expected to have two  argu- ments  which  point to the key object and to an array mem- ber, in that order, and  should  return  an  integer  less than,  equal to, or greater than zero if the key object is found, respectively, to be less  than,  to  match,  or  be greater than the array member.
/*/

#-calloc@cstdlib
void *calloc(size_t nmemb, size_t size); void *malloc(size_t size); void free(void *ptr); void *realloc(void *ptr, size_t size)
/*-p-
@b:calloc,  malloc, free, realloc - Allocate and free dynamicmemory
@r:For calloc() and malloc(), the value returned is a pointer to the allocated memory, which is suitably aligned for any kind of variable, or NULL if the request fails. free() returns no value. realloc() returns a pointer to the newly allocated memory, which is suitably aligned for any kind of variable and may be  different from ptr, or NULL if the request fails or if size was equal to 0.   If  realloc()  fails  the  original block is left untouched - it is not freed or moved. ANSI - C
calloc() allocates memory for an array of  nmemb  elements of  size bytes each and returns a pointer to the allocated memory.  The memory is set to zero. malloc() allocates size bytes and returns a pointer to the allocated memory.  The memory is not cleared. free()  frees  the  memory  space pointed to by ptr, which must have been returned by a previous  call  to  malloc(), calloc()  or  realloc().   If ptr is NULL, no operation is performed. realloc() changes the size of the memory block pointed  to by  ptr  to size bytes.  The contents will be unchanged to the minimum of the old an new sizes; newly allocated  mem- ory  will  be  uninitialized.  If ptr is NULL, the call is equivalent to malloc(size); if size is equal to zero,  the call  is  equivalent to free(ptr).  Unless ptr is NULL, it must have been returned by an earlier  call  to  malloc(), calloc() or realloc().
/*/

#-div@cstdlib
div_t div(int numer, int denom)
/*-p-
@b:div - computes the quotient and remainder of integer divi-sion
@r:The div_t structure.
The div() function  computes  the  value  numer/denom  and returns  the  quotient  and remainder in a structure named div_t that contains two integer  members  named  quot  and rem.
/*/

#-exit@cstdlib
void exit(int status)
/*-p-
@b:exit - cause normal program termination
@r:The exit() function does not return.
The  exit() function causes normal program termination and the value of status is returned to the parent.  All  func- tions registered with atexit() and on_exit() are called in the reverse order of  their  registration,  and  all  open streams are flushed and closed.
/*/

#-free@cstdlib
void *calloc(size_t nmemb, size_t size); void *malloc(size_t size); void free(void *ptr); void *realloc(void *ptr, size_t size)
/*-p-
@b:calloc,  malloc, free, realloc - Allocate and free dynamicmemory
@r:For calloc() and malloc(), the value returned is a pointer to the allocated memory, which is suitably aligned for any kind of variable, or NULL if the request fails. free() returns no value. realloc() returns a pointer to the newly allocated memory, which is suitably aligned for any kind of variable and may be  different from ptr, or NULL if the request fails or if size was equal to 0.   If  realloc()  fails  the  original block is left untouched - it is not freed or moved. ANSI - C
calloc() allocates memory for an array of  nmemb  elements of  size bytes each and returns a pointer to the allocated memory.  The memory is set to zero. malloc() allocates size bytes and returns a pointer to the allocated memory.  The memory is not cleared. free()  frees  the  memory  space pointed to by ptr, which must have been returned by a previous  call  to  malloc(), calloc()  or  realloc().   If ptr is NULL, no operation is performed. realloc() changes the size of the memory block pointed  to by  ptr  to size bytes.  The contents will be unchanged to the minimum of the old an new sizes; newly allocated  mem- ory  will  be  uninitialized.  If ptr is NULL, the call is equivalent to malloc(size); if size is equal to zero,  the call  is  equivalent to free(ptr).  Unless ptr is NULL, it must have been returned by an earlier  call  to  malloc(), calloc() or realloc().
/*/

#-getenv@cstdlib
char *getenv(const char *name)
/*-p-
@b:getenv - get an environment variable
@r:The  getenv()  function  returns a pointer to the value in the environment, or NULL if there is no match.
The  getenv() function searches the environment list for a string that matches the string pointed to  by  name.   The strings are of the form name = value.
/*/

#-labs@cstdlib
long int labs(long int j)
/*-p-
@b:labs - computes the absolute value of a long integer.
@r:Returns the absolute value of the long integer argument.
The  labs()  function  computes  the absolute value of the long integer argument j.
/*/

#-ldiv@cstdlib
ldiv_t ldiv(long int numer, long int denom)
/*-p-
@b:ldiv - computes the quotient and remainder of long integerdivision.
@r:The ldiv_t structure.
The ldiv() function computes  the  value  numer/denom  and returns  the  quotient  and remainder in a structure named ldiv_t that contains two long integer members  named  quot and rem.
/*/

#-malloc@cstdlib
void *calloc(size_t nmemb, size_t size); void *malloc(size_t size); void free(void *ptr); void *realloc(void *ptr, size_t size)
/*-p-
@b:calloc,  malloc, free, realloc - Allocate and free dynamicmemory
@r:For calloc() and malloc(), the value returned is a pointer to the allocated memory, which is suitably aligned for any kind of variable, or NULL if the request fails. free() returns no value. realloc() returns a pointer to the newly allocated memory, which is suitably aligned for any kind of variable and may be  different from ptr, or NULL if the request fails or if size was equal to 0.   If  realloc()  fails  the  original block is left untouched - it is not freed or moved. ANSI - C
calloc() allocates memory for an array of  nmemb  elements of  size bytes each and returns a pointer to the allocated memory.  The memory is set to zero. malloc() allocates size bytes and returns a pointer to the allocated memory.  The memory is not cleared. free()  frees  the  memory  space pointed to by ptr, which must have been returned by a previous  call  to  malloc(), calloc()  or  realloc().   If ptr is NULL, no operation is performed. realloc() changes the size of the memory block pointed  to by  ptr  to size bytes.  The contents will be unchanged to the minimum of the old an new sizes; newly allocated  mem- ory  will  be  uninitialized.  If ptr is NULL, the call is equivalent to malloc(size); if size is equal to zero,  the call  is  equivalent to free(ptr).  Unless ptr is NULL, it must have been returned by an earlier  call  to  malloc(), calloc() or realloc().
/*/

#-mblen@cstdlib
int mblen(const char *s, size_t n)
/*-p-
@b:mblen - determine the number of bytes in a character
@r:The mblen() returns the number of bytes in a character  or -1 if the character is invalid or 0 if s is a NULL string.
The mblen() function scans the first n bytes of the string s and returns the number of bytes  in  a  character.   The mblen() function is equivalent to mbtowc((wchat_t *)0, s, n); except  that  the  shift state of the mbtowc() function is not affected.
/*/

#-mbstowcs@cstdlib
size_t mbstowcs(wchar_t *pwcs, const char *s, size_t n)
/*-p-
@b:mbstowcs  - convert a multibyte string to a wide characterstring.
@r:mbstowcs() returns the number of wide characters stored or -1 if s contains an invalid multibyte character.
The mbstowcs() function converts a sequence  of  multibyte characters  from the array s into a sequence of wide char- acters and stores up to n wide  characters  in  the  array pwcs.
/*/

#-mbtowc@cstdlib
int mbtowc(wchar_t *pwc, const char *s, size_t n)
/*-p-
@b:mbtowc  -  convert a multibyte character to a wide charac-ter.
@r:mbtowc()  returns  the  number  of  bytes in the multibyte character or -1 if the multibyte character is not valid.
The mbtowc() function converts a  multibyte  character  s, which  is  no  longer  than n bytes, into a wide character and, if pwc is not NULL, stores the wide character in pwc.
/*/

#-qsort@cstdlib
void qsort(void *base, size_t nmemb, size_t size, int (*compar)(const void *, const void *))
/*-p-
@b:qsort - sorts an array
@r:The qsort() function returns no value.
The qsort() function sorts an array with nmemb elements of size size.  The base argument points to the start  of  the array. The  contents  of  the array are sorted in ascending order according to a comparison function pointed to  by  compar, which  is  called  with  two  arguments  that point to the objects being compared. The comparison function must return an integer less  than, equal  to,  or  greater than zero if the first argument is considered to be respectively  less  than,  equal  to,  or greater than the second.  If two members compare as equal, their order in the sorted array is undefined.
/*/

#-rand@cstdlib
int rand(void); void srand(unsigned int seed)
/*-p-
@b:rand, srand - random number generator.
@r:The  rand()  function  returns  a  value  between  0   and RAND_MAX.  The srand() returns no value. The  versions of rand() and srand() in the Linux C Library use the same random number generator as random() and sran- dom(),  so the lower-order bits should be as random as the higher-order bits.  However, on older  rand()  implementa- tions,  the lower-order bits are much less random than the higher-order bits. In Numerical Recipes in C: The Art of Scientific Computing (William  H.  Press, Brian P. Flannery, Saul A. Teukolsky, William T.  Vetterling;  New  York:  Cambridge  University Press,  1990 (1st ed, p. 207)), the following comments are made: "If you want to generate a random integer between 1 and 10, you should always do it by j=1+(int) (10.0*rand()/(RAND_MAX+1.0)); and never by anything resembling j=1+((int) (1000000.0*rand()) % 10); (which uses lower-order bits)." Random-number  generation is a complex topic.  The Numeri- cal Recipes in C book (see reference  above)  provides  an excellent discussion of practical random-number generation issues in Chapter 7 (Random Numbers). For a more theoretical discussion which also  covers  many practical  issues  in  depth, please see Chapter 3 (Random Numbers) in Donald E. Knuth's The Art of Computer Program- ming,  volume 2 (Seminumerical Algorithms), 2nd ed.; Read- ing,  Massachusetts:  Addison-Wesley  Publishing  Company, 1981.
The   rand()  function  returns  a  pseudo-random  integer between 0 and RAND_MAX. The srand() function sets its argument as the seed  for  a new  sequence  of pseudo-random integers to be returned by rand().  These sequences are repeatable by calling srand() with the same seed value. If no seed value is provided, the rand() function is auto- matically seeded with a value of 1.
/*/

#-realloc@cstdlib
void *calloc(size_t nmemb, size_t size); void *malloc(size_t size); void free(void *ptr); void *realloc(void *ptr, size_t size)
/*-p-
@b:calloc,  malloc, free, realloc - Allocate and free dynamicmemory
@r:For calloc() and malloc(), the value returned is a pointer to the allocated memory, which is suitably aligned for any kind of variable, or NULL if the request fails. free() returns no value. realloc() returns a pointer to the newly allocated memory, which is suitably aligned for any kind of variable and may be  different from ptr, or NULL if the request fails or if size was equal to 0.   If  realloc()  fails  the  original block is left untouched - it is not freed or moved. ANSI - C
calloc() allocates memory for an array of  nmemb  elements of  size bytes each and returns a pointer to the allocated memory.  The memory is set to zero. malloc() allocates size bytes and returns a pointer to the allocated memory.  The memory is not cleared. free()  frees  the  memory  space pointed to by ptr, which must have been returned by a previous  call  to  malloc(), calloc()  or  realloc().   If ptr is NULL, no operation is performed. realloc() changes the size of the memory block pointed  to by  ptr  to size bytes.  The contents will be unchanged to the minimum of the old an new sizes; newly allocated  mem- ory  will  be  uninitialized.  If ptr is NULL, the call is equivalent to malloc(size); if size is equal to zero,  the call  is  equivalent to free(ptr).  Unless ptr is NULL, it must have been returned by an earlier  call  to  malloc(), calloc() or realloc().
/*/

#-srand@cstdlib
int rand(void); void srand(unsigned int seed)
/*-p-
@b:rand, srand - random number generator.
@r:The  rand()  function  returns  a  value  between  0   and RAND_MAX.  The srand() returns no value. The  versions of rand() and srand() in the Linux C Library use the same random number generator as random() and sran- dom(),  so the lower-order bits should be as random as the higher-order bits.  However, on older  rand()  implementa- tions,  the lower-order bits are much less random than the higher-order bits. In Numerical Recipes in C: The Art of Scientific Computing (William  H.  Press, Brian P. Flannery, Saul A. Teukolsky, William T.  Vetterling;  New  York:  Cambridge  University Press,  1990 (1st ed, p. 207)), the following comments are made: "If you want to generate a random integer between 1 and 10, you should always do it by j=1+(int) (10.0*rand()/(RAND_MAX+1.0)); and never by anything resembling j=1+((int) (1000000.0*rand()) % 10); (which uses lower-order bits)." Random-number  generation is a complex topic.  The Numeri- cal Recipes in C book (see reference  above)  provides  an excellent discussion of practical random-number generation issues in Chapter 7 (Random Numbers). For a more theoretical discussion which also  covers  many practical  issues  in  depth, please see Chapter 3 (Random Numbers) in Donald E. Knuth's The Art of Computer Program- ming,  volume 2 (Seminumerical Algorithms), 2nd ed.; Read- ing,  Massachusetts:  Addison-Wesley  Publishing  Company, 1981.
The   rand()  function  returns  a  pseudo-random  integer between 0 and RAND_MAX. The srand() function sets its argument as the seed  for  a new  sequence  of pseudo-random integers to be returned by rand().  These sequences are repeatable by calling srand() with the same seed value. If no seed value is provided, the rand() function is auto- matically seeded with a value of 1.
/*/

#-strtod@cstdlib
double strtod(const char *nptr, char **endptr
/*-p-
@b:strtod - convert ASCII string to double
@r:The strtod function returns the converted value, if any. If endptr is not NULL, a pointer to  the  character  after the last character used in the conversion is stored in the location referenced by endptr. If no conversion is performed, zero is  returned  and  the value  of  nptr  is  stored  in the location referenced by endptr. If the correct value would cause overflow, plus  or  minus HUGE_VAL is returned (according to the sign of the value), and ERANGE is stored in errno.  If the correct value would cause  underflow, zero is returned and ERANGE is stored in errno.
The  strtod  function  converts the initial portion of the string pointed to by nptr to double representation. The expected form  of  the  string  is  an  optional  plus (``+'')  or  minus  sign (``-'') followed by a sequence of digits optionally containing  a  decimal-point  character, optionally  followed by an exponent.  An exponent consists of an ``E'' or ``e'', followed  by  an  optional  plus  or minus sign, followed by a sequence of digits. Leading  white-space  characters in the string (as defined by the <A HREF="/man3/isspace">isspace(3)</A> function) are skipped.
/*/

#-strtol@cstdlib
long int strtol(const char *nptr, char **endptr, int base)
/*-p-
@b:strtol - convert a string to a long integer.
@r:The  strtol()  function  returns the result of the conver- sion, unless the value would underflow or overflow.  If an underflow  occurs, strtol() returns LONG_MIN.  If an over- flow occurs, strtol() returns LONG_MAX.   In  both  cases, errno is set to ERANGE.
The  strtol()  function  converts  the string in nptr to a long integer value according to the given base, which must be  between 2 and 36 inclusive, or be the special value 0. The string must begin with an arbitrary  amount  of  white space  (as  determined by <A HREF="/man3/isspace">isspace(3)</A>) followed by a single optional `+' or `-' sign.  If base  is  zero  or  16,  the string may then include a `0x' prefix, and the number will be read in base 16; otherwise, a zero base is taken as  10 (decimal)  unless the next character is `0', in which case it is taken as 8 (octal). The remainder of the string is converted  to  a  long  int value in the obvious manner, stopping at the first charac- ter which is not a valid digit in  the  given  base.   (In bases  above  10,  the letter `A' in either upper or lower case represents 10, `B' represents 11, and so forth,  with `Z' representing 35.) If  endptr is not NULL, strtol() stores the address of the first invalid character in *endptr.  If there were no dig- its  at all, strtol() stores the original value of nptr in *endptr.  (Thus, if *nptr is not `\0' but **endptr is `\0' on return, the entire string is valid.)
/*/

#-strtoul@cstdlib
unsigned long int strtol(const char *nptr, char **endptr, intbase)
/*-p-
@b:strtoul - convert a string to an unsigned long integer.
@r:The  strtoul()  function  returns either the result of the conversion or, if there was  a  leading  minus  sign,  the negation of the result of the conversion, unless the orig- inal (non-negated) value would  overflow;  in  the  latter case,  strtoul()  returns  ULONG_MAX  and  sets the global variable errno to ERANGE.
The  strtoul()  function converts the string in nptr to an unsigned long integer value according to the  given  base, which  must  be between 2 and 36 inclusive, or be the spe- cial value 0. The string must begin with an arbitrary  amount  of  white space  (as  determined by <A HREF="/man3/isspace">isspace(3)</A>) followed by a single optional `+' or `-' sign.  If base  is  zero  or  16,  the string may then include a `0x' prefix, and the number will be read in base 16; otherwise, a zero base is taken as  10 (decimal)  unless the next character is `0', in which case it is taken as 8 (octal). The remainder of the string is converted  to  an  unsigned long  int  value  in  the  obvious manner, stopping at the first character which is not a valid digit  in  the  given base.   (In bases above 10, the letter `A' in either upper or lower case represents 10, `B'  represents  11,  and  so forth, with `Z' representing 35.) If endptr is not NULL, strtoul() stores the address of the first invalid character in *endptr.  If there were no dig- its at all, strtoul() stores the original value of nptr in *endptr.  (Thus, if *nptr is not `\0' but **endptr is `\0' on return, the entire string is invalid.)
/*/

#-system@cstdlib
int system (const char * string)
/*-p-
@b:system - execute a shell command
@r:The value returned is 127 if the execve() call for /bin/sh fails,  -1  if there was another error and the return code of the command otherwise. If the value of string is NULL, system()  returns  nonzero if the shell is available, and zero if not. system()  does  not  affect  the  wait status of any other children. ANSI C, POSIX.1, proposed POSIX.2, BSD 4.3 Do not use system() from a program with suid or sgid priv- ileges,  because strange values for some environment vari- ables might be used to subvert system integrity.  Use  the <A HREF="/man2/exec">exec(2)</A>  family of functions instead, but not <A HREF="/man2/execlp">execlp(2)</A> or <A HREF="/man2/execvp">execvp(2)</A>. The check for the availability of /bin/sh is not  actually performed; it is always assumed to be available. It  is  possible  for  the shell command to return 127, so that code is not a sure indication that the execve()  call failed; check errno to make sure.
system() executes a command specified in string by calling /bin/sh -c string, and returns after the command has  been completed.   During execution of the command, SIGCHLD will be blocked, and SIGINT and SIGQUIT will be ignored.
/*/

#-wcstombs@cstdlib
size_t wcstombs(char *s, const wchar_t *pwcs, size_t n)
/*-p-
@b:wcstombs  - convert a wide character string to a multibytecharacter string.
@r:wcstombs()  returns  the number of bytes stored in s or -1 if s contains an invalid wide character.
The wcstombs() function converts a sequence of wide  char- acters  from  the  array pwcs into a sequence of multibyte characters and stores up to n bytes of  multibyte  charac- ters in the array s.
/*/

#-wctomb@cstdlib
int wctomb(char *s, wchar_t wchar)
/*-p-
@b:wctomb  -  convert a wide character to a multibyte charac-ter.
@r:wctomb() returns the number  of  bytes  in  the  multibyte character or -1 if the wide character is not valid.
The wctomb() function converts a wide character wchar into a  multibyte  character  and, if s is not NULL, stores the multibyte character representation in s.
/*/

##:cstring@cstring
#-memcpy@cstring
void *memcpy(void *dest, const void *src, size_t n)
/*-p-
@b:memcpy - copy memory area
@r:The memcpy() function returns a pointer to dest.
The  memcpy() function copies n bytes from memory area src to memory area dest.  The memory areas  may  not  overlap. Use <A HREF="/man3/memmove">memmove(3)</A> if the memory areas do overlap.
/*/

#-memmove@cstring
void *memmove(void *dest, const void *src, size_t n)
/*-p-
@b:memmove - copy memory area
@r:The memmove() function returns a pointer to dest.
The memmove() function copies n bytes from memory area src to memory area dest.  The memory areas may overlap.
/*/

#-memchr@cstring
void *memchr(const void *s, int c, size_t n)
/*-p-
@b:memchr - scan memory for a character
@r:The memchr() function returns a pointer  to  the  matching byte  or NULL if the character does not occur in the given memory area.
The  memchr() function scans the first n bytes of the mem- ory area pointed to by s for the character c.   The  first byte  to  match  c  (interpreted as an unsigned character) stops the operation.
/*/

#-memcmp@cstring
int memcmp(const void *s1, const void *s2, size_t n)
/*-p-
@b:memcmp - compare memory areas
@r:The  memcmp() function returns an integer less than, equal to, or greater than zero if the first n  bytes  of  s1  is found,  respectively,  to  be  less  than, to match, or be greater than the first n bytes of s2.
The  memcmp()  function  compares the first n bytes of the memory areas s1 and s2.  It returns an integer less  than, equal  to,  or  greater  than zero if s1 is found, respec- tively, to be less than, to match, or be greater than  s2.
/*/

#-memset@cstring
void *memset(void *s, int c, size_t n)
/*-p-
@b:memset - fill memory with a constant byte
@r:The memset() function returns a pointer to the memory area s.
The  memset() function fills the first n bytes of the mem- ory area pointed to be s with the constant byte c.
/*/

#-strcpy@cstring
char *strcpy(char *dest, const char *src); char *strncpy(char *dest, const char *src, size_t n)
/*-p-
@b:strcpy, strncpy - copy a string
@r:The  strcpy()  and strncpy() functions return a pointer to the destination string dest.
The  strcpy() function copies the string pointed to be src (including the terminating `\0' character)  to  the  array pointed  to by dest.  The strings may not overlap, and the destination string dest must be large  enough  to  receive the copy. The  strncpy()  function  is similar, except that only the first n bytes of src are copied. In the case where the length of src is less than  that  of n, the remainder of dest will be padded with nulls.
/*/

#-strncpy@cstring
char *strcpy(char *dest, const char *src); char *strncpy(char *dest, const char *src, size_t n)
/*-p-
@b:strcpy, strncpy - copy a string
@r:The  strcpy()  and strncpy() functions return a pointer to the destination string dest.
The  strcpy() function copies the string pointed to be src (including the terminating `\0' character)  to  the  array pointed  to by dest.  The strings may not overlap, and the destination string dest must be large  enough  to  receive the copy. The  strncpy()  function  is similar, except that only the first n bytes of src are copied. In the case where the length of src is less than  that  of n, the remainder of dest will be padded with nulls.
/*/

#-strcat@cstring
char *strcat(char *dest, const char *src); char *strncat(char *dest, const char *src, size_t n)
/*-p-
@b:strcat, strncat - concatenate two strings
@r:The strcat() and strncat() functions return a  pointer  to the resulting string dest.
The  strcat()  function appends the src string to the dest string overwriting the `\0' character at the end of  dest, and  then  adds a terminating `\0' character.  The strings may not overlap, and the  dest  string  must  have  enough space for the result. The  strncat()  function  is similar, except that only the first n characters of src are appended to dest.
/*/

#-strncat@cstring
char *strcat(char *dest, const char *src); char *strncat(char *dest, const char *src, size_t n)
/*-p-
@b:strcat, strncat - concatenate two strings
@r:The strcat() and strncat() functions return a  pointer  to the resulting string dest.
The  strcat()  function appends the src string to the dest string overwriting the `\0' character at the end of  dest, and  then  adds a terminating `\0' character.  The strings may not overlap, and the  dest  string  must  have  enough space for the result. The  strncat()  function  is similar, except that only the first n characters of src are appended to dest.
/*/

#-strcmp@cstring
int strcmp(const char *s1, const char *s2); int strncmp(const char *s1, const char *s2, size_t n)
/*-p-
@b:strcmp, strncmp - compare two strings
@r:The  strcmp()  and  strncmp()  functions return an integer less than, equal to, or greater than zero if  s1  (or  the first  n bytes thereof) is found, respectively, to be less than, to match, or be greater than s2.
The  strcmp() function compares the two strings s1 and s2. It returns an integer less than, equal to, or greater than zero  if  s1  is  found, respectively, to be less than, to match, or be greater than s2. The strncmp() function is similar, except it only compares the first n characters of s1.
/*/

#-strcoll@cstring
int strcoll(const char *s1, const char *s2)
/*-p-
@b:strcoll - compare two strings using the current locale
@r:The strcoll() function returns an integer less than, equal to, or greater than zero if s1 is found, respectively,  to be  less  than, to match, or be greater than s2, when both are interpreted as appropriate for the current locale.
The strcoll() function compares the two strings s1 and s2. It returns an integer less than, equal to, or greater than zero  if  s1  is  found, respectively, to be less than, to match, or be greater than s2.  The comparison is based  on strings  interpreted as appropriate for the program's cur- rent locale for category LC_COLLATE.  (See  <A HREF="/man3/setlocale">setlocale(3)</A>).
/*/

#-strncmp@cstring
int strcmp(const char *s1, const char *s2); int strncmp(const char *s1, const char *s2, size_t n)
/*-p-
@b:strcmp, strncmp - compare two strings
@r:The  strcmp()  and  strncmp()  functions return an integer less than, equal to, or greater than zero if  s1  (or  the first  n bytes thereof) is found, respectively, to be less than, to match, or be greater than s2.
The  strcmp() function compares the two strings s1 and s2. It returns an integer less than, equal to, or greater than zero  if  s1  is  found, respectively, to be less than, to match, or be greater than s2. The strncmp() function is similar, except it only compares the first n characters of s1.
/*/

#-strxfrm@cstring
size_t strxfrm(char *dest, const char *src, size_t n)
/*-p-
@b:strxfrm - string transformation
@r:The  strxfrm()  function  returns  the  number  of   bytes required to store the transformed string in dest excluding the terminating `\0' character.  If the value returned  is n or more, the contents of dest are indeterminate.
The  strxfrm()  function  transforms the src string into a form such that the result of strcmp() on two strings  that have  been  transformed  with strxfrm() is the same as the result of strcoll() on the two strings before their trans- formation.   The  first  n  characters  of the transformed string are placed in dest.  The transformation is based on the  program's  current  locale  for  category LC_COLLATE. (See <A HREF="/man3/setlocale">setlocale(3)</A>).
/*/

#-strchr@cstring
char *strchr(const char *s, int c); char *strrchr(const char *s, int c)
/*-p-
@b:strchr, strrchr - locate character in string
@r:The  strchr()  and strrchr() functions return a pointer to the matched character or NULL  if  the  character  is  not found.
The  strchr()  function  returns  a  pointer  to the first occurrence of the character c in the string s. The strrchr() function  returns  a  pointer  to  the  last occurrence of the character c in the string s.
/*/

#-strcspn@cstring
size_t strspn(const char *s, const char *accept); size_t strcspn(const char *s, const char *reject)
/*-p-
@b:strspn, strcspn - search a string for a set of characters
@r:The  strspn() function returns the number of characters in the initial segment of s which consist only of  characters from accept. The strcspn() function returns the number of characters in the initial segment of s  which  are  not  in  the  string reject.
The strspn() function calculates the length of the initial segment of s which  consists  entirely  of  characters  in accept. The  strcspn()  function calculates the length of the ini- tial segment of s which consists  entirely  of  characters not in reject.
/*/

#-strpbrk@cstring
char *strpbrk(const char *s, const char *accept)
/*-p-
@b:strpbrk - search a string for any of a set of characters
@r:The strpbrk() function returns a pointer to the  character in s that matches one of the characters in accept.
The strpbrk() function locates the first occurrence in the string s of any of the characters in the string accept.
/*/

#-strrchr@cstring
char *strchr(const char *s, int c); char *strrchr(const char *s, int c)
/*-p-
@b:strchr, strrchr - locate character in string
@r:The  strchr()  and strrchr() functions return a pointer to the matched character or NULL  if  the  character  is  not found.
The  strchr()  function  returns  a  pointer  to the first occurrence of the character c in the string s. The strrchr() function  returns  a  pointer  to  the  last occurrence of the character c in the string s.
/*/

#-strspn@cstring
size_t strspn(const char *s, const char *accept); size_t strcspn(const char *s, const char *reject)
/*-p-
@b:strspn, strcspn - search a string for a set of characters
@r:The  strspn() function returns the number of characters in the initial segment of s which consist only of  characters from accept. The strcspn() function returns the number of characters in the initial segment of s  which  are  not  in  the  string reject.
The strspn() function calculates the length of the initial segment of s which  consists  entirely  of  characters  in accept. The  strcspn()  function calculates the length of the ini- tial segment of s which consists  entirely  of  characters not in reject.
/*/

#-strstr@cstring
char *strstr(const char *haystack, const char *needle)
/*-p-
@b:strstr - locate a substring
@r:The  strstr()  function returns a pointer to the beginning of the substring, or NULL if the substring is not found.
The  strstr()  function  finds the first occurrence of the substring needle in the string haystack.  The  terminating `\0' characters are not compared.
/*/

#-strtok@cstring
char *strtok(char *s, const char *delim)
/*-p-
@b:strtok - extract token from string
@r:The strtok() function returns a pointer to the next token, or NULL if there are no more tokens.
The strtok() function breaks the string s into a series of tokens delimited by delim.  The  first  call  to  strtok() should  have  s  as  the first argument.  Subsequent calls should have the first argument set to NULL.  The separator string delim may be different for each call. The first call to strtok() returns the first token in s or NULL if delim is not found in s.  A pointer  to  the  next character  is  saved  for the next call to strtok.  Subse- quent calls return subsequent tokens  until  delim  is  no longer found, when NULL is returned.
/*/

#-strerror@cstring
char *strerror(int errnum)
/*-p-
@b:strerror - return string describing error code
@r:The  strerror()  function returns the appropriate descrip- tion string, or an unknown error message if the error code is unknown.
The  strerror()  function  returns a string describing the error code passed in the argument errno.  The  string  can only be used until the next call to strerror().
/*/

#-strlen@cstring
size_t strlen(const char *s)
/*-p-
@b:strlen - calculate the length of a string
@r:The strlen() function returns the number of characters  in s.
The  strlen() function calculates the length of the string s, not including the terminating `\0' character.
/*/

##:ctime@ctime
#-clock@ctime
clock_t clock(void)
/*-p-
@b:clock - Determine processor time
@r:The value returned is the  CPU  time  used  so  far  as  a clock_t;  to  get  the  number  of seconds used, divide by CLOCKS_PER_SEC. ANSI C The C standard allows for arbitrary values at the start of the   program;  take  the  difference  between  the  value returned from a call to clock() at the start of  the  pro- gram and the end to get maximum portability. The times() function call returns more information.
The clock() function returns an approximation of processor time used by the program.
/*/

#-difftime@ctime
double difftime(time_t time1, time_t time0)
/*-p-
@b:difftime - calculate time difference
@r:
The  difftime()  function  returns  the  number of seconds elapsed between time time1 and time time0.  The two  times are  specified in calendar time, which represents the time elapsed since 00:00:00 on  January  1,  1970,  Coordinated Universal Time (UTC).
/*/

#-time@ctime
time_t time ( time_t * timer )
/*-p-
@b:Get the current calendar time as a time_t object
@p:Pointer to an object of type time_t, where the time value is stored or null.
@r:The current calendar time as a time_t object.
Alternativelly, this parameter can be a null pointer, in which case the parameter is not used, but the a time_t object is still returned by the function.
If the argument is not a null pointer, the return value is the same as the one stored in the location pointed by the argument.

If the function could not retrieve the calendar time, it returns a -1 value.
/*/

#-asctime,  ctime,  gmtime,  localtime,  mktime@ctime
char *asctime(const struct tm *timeptr); char *ctime(const time_t *timep); struct tm *gmtime(const time_t *timep); struct tm *localtime(const time_t *timep); time_t mktime(struct tm *timeptr); extern char *tzname[2]; long int timezone; extern int daylight
/*-p-
@b:asctime,  ctime,  gmtime,  localtime,  mktime  - transformbinary date and time to ASCII
@r:
The ctime(), gmtime() and localtime() functions  all  take an  argument of data type time_t which represents calendar time.  When interpreted as an absolute time value, it rep- resents  the  number  of seconds elapsed since 00:00:00 on January 1, 1970, Coordinated Universal Time (UTC). The asctime() and mktime() functions both take an argument representing  broken-down time which is a binary represen- tation separated into year, month, day, etc.   Broken-down time  is  stored  in  the structure tm which is defined in &lt;time.h&gt; as follows: struct tm { int     tm_sec;         /* seconds */ int     tm_min;         /* minutes */ int     tm_hour;        /* hours */ int     tm_mday;        /* day of the month */ int     tm_mon;         /* month */ int     tm_year;        /* year */ int     tm_wday;        /* day of the week */ int     tm_yday;        /* day in the year */ int     tm_isdst;       /* daylight saving time */ }; The members of the tm structure are: tm_sec The number of seconds after the minute, normally in the range 0 to 59, but can be up to 61 to allow for leap seconds. tm_min The number of minutes after the hour, in the  range 0 to 59. tm_hour The  number  of hours past midnight, in the range 0 to 23. tm_mday The day of the month, in the range 1 to 31. tm_mon The number of months since January, in the range  0 to 11. tm_year The number of years since 1900. tm_wday The  number of days since Sunday, in the range 0 to 6. tm_yday The number of days since January 1, in the range  0 to 365. tm_isdst A  flag that indicates whether daylight saving time is in effect at the time described.  The  value  is positive if daylight saving time is in effect, zero if it is not, and negative if  the  information  is not available. The ctime() function converts the calendar time timep into a string of the form "Wed Jun 30 21:49:08 1993\n" The abbreviations for the days  of  the  week  are  `Sun', `Mon',  `Tue', `Wed', `Thu', `Fri', and `Sat'.  The abbre- viations for the months are `Jan',  `Feb',  `Mar',  `Apr', `May',  `Jun',  `Jul',  `Aug',  `Sep',  `Oct',  `Nov', and `Dec'.  The return value points to a statically  allocated string  which  might be overwritten by subsequent calls to any of the date and time  functions.   The  function  also sets  the  external variable tzname with information about the current time zone. The gmtime() function converts the calendar time timep  to broken-down  time representation, expressed in Coordinated Universal Time (UTC). The localtime() function converts the calendar time  timep to  broken-time  representation, expressed relative to the user's specified  time  zone.     The  function  sets  the external  variables tzname with information about the cur- rent time  zone,  timezone  with  the  difference  between Coordinated  Universal  Time (UTC) and local standard time in seconds, and daylight to a non-zero value  if  standard US daylight savings time rules apply. The asctime() function converts the broken-down time value timeptr into a string with the  same  format  as  ctime(). The  return  value points to a statically allocated string which might be overwritten by subsequent calls to  any  of the date and time functions. The  mktime()  function converts a broken-down time struc- ture  to  calendar  time  representation.   The   function ignores  the  specified  contents of the structure members tm_wday and tm_yday and recomputes  them  from  the  other information  in  the  broken-down time structure.  Calling mktime() also  sets  the  external  variable  tzname  with information about the current time zone.  If the specified broken-down time cannot be represented as  calendar  time, mktime()  returns  a  value  of  (time_t)(-1) and does not alter the tm_wday and tm_yday members of  the  broken-down time structure.
/*/

#-strftime@ctime
size_t strftime(char *s, size_t max, const char *format, const struct tm *tm)
/*-p-
@b:strftime - format date and time
@r:The  strftime()  function returns the number of characters placed in the array s, not including the terminating  NULL character.   If  the  value  equals max, it means that the array was too small.
The  strftime()  function  formats the broken-down time tm according to the format specification  format  and  places the result in the character array s of size max. Ordinary characters placed in the format string are copied to s without conversion.  Conversion specifiers are intro- duced  by  a  `%' character, and are replaced in s as fol- lows: %a     The abbreviated weekday name according to the  cur- rent locale. %A     The  full  weekday  name  according  to the current locale. %b     The abbreviated month name according to the current locale. %B     The  full  month  name  according  to  the  current locale. %c     The preferred date and time representation for  the current locale. %d     The  day  of the month as a decimal number (range 0 to 31). %H     The hour as a decimal number using a 24-hour  clock (range 00 to 23). %I     The  hour as a decimal number using a 12-hour clock (range 01 to 12). %j     The day of the year as a decimal number (range  001 to 366). %m     The month as a decimal number (range 10 to 12). %M     The minute as a decimal number. %p     Either  `am'  or  `pm'  according to the given time value, or the corresponding strings for the current locale. %S     The second as a decimal number. %U     The  week  number  of the current year as a decimal number, starting with the first Sunday as the first day of the first week. %W     The  week  number  of the current year as a decimal number, starting with the first Monday as the first day of the first week. %w     The day of the week as a decimal, Sunday being 0. %x     The  preferred  date representation for the current locale without the time. %X     The preferred time representation for  the  current locale without the date. %y     The  year  as  a  decimal  number without a century (range 00 to 99). %Y     The year as a decimal number including the century. %Z     The time zone or name or abbreviation. %%     A literal `%' character. The  broken-down  time structure tm is defined in &lt;time.h&gt; as follows: struct tm { int     tm_sec;         /* seconds */ int     tm_min;         /* minutes */ int     tm_hour;        /* hours */ int     tm_mday;        /* day of the month */ int     tm_mon;         /* month */ int     tm_year;        /* year */ int     tm_wday;        /* day of the week */ int     tm_yday;        /* day in the year */ int     tm_isdst;       /* daylight saving time */ }; The members of the tm structure are: tm_sec The number of seconds after the minute, normally in the range 0 to 59, but can be up to 61 to allow for leap seconds. tm_min The number of minutes after the hour, in the  range 0 to 59. tm_hour The  number  of hours past midnight, in the range 0 to 23. tm_mday The day of the month, in the range 1 to 31. tm_mon The number of months since January, in the range  0 to 11. tm_year The number of years since 1900. tm_wday The  number of days since Sunday, in the range 0 to 6. tm_yday The number of days since January 1, in the range  0 to 365. tm_isdst A  flag that indicates whether daylight saving time is in effect at the time described.  The  value  is positive if daylight saving time is in effect, zero if it is not, and negative if  the  information  is not available.
/*/

#-strftime@ctime
size_t strftime(char *s, size_t max, const char *format, const struct tm *tm)
/*-p-
@b:strftime - format date and time
@r:The  strftime()  function returns the number of characters placed in the array s, not including the terminating  NULL character.   If  the  value  equals max, it means that the array was too small.
The  strftime()  function  formats the broken-down time tm according to the format specification  format  and  places the result in the character array s of size max. Ordinary characters placed in the format string are copied to s without conversion.  Conversion specifiers are intro- duced  by  a  `%' character, and are replaced in s as fol- lows: %a     The abbreviated weekday name according to the  cur- rent locale. %A     The  full  weekday  name  according  to the current locale. %b     The abbreviated month name according to the current locale. %B     The  full  month  name  according  to  the  current locale. %c     The preferred date and time representation for  the current locale. %d     The  day  of the month as a decimal number (range 0 to 31). %H     The hour as a decimal number using a 24-hour  clock (range 00 to 23). %I     The  hour as a decimal number using a 12-hour clock (range 01 to 12). %j     The day of the year as a decimal number (range  001 to 366). %m     The month as a decimal number (range 10 to 12). %M     The minute as a decimal number. %p     Either  `am'  or  `pm'  according to the given time value, or the corresponding strings for the current locale. %S     The second as a decimal number. %U     The  week  number  of the current year as a decimal number, starting with the first Sunday as the first day of the first week. %W     The  week  number  of the current year as a decimal number, starting with the first Monday as the first day of the first week. %w     The day of the week as a decimal, Sunday being 0. %x     The  preferred  date representation for the current locale without the time. %X     The preferred time representation for  the  current locale without the date. %y     The  year  as  a  decimal  number without a century (range 00 to 99). %Y     The year as a decimal number including the century. %Z     The time zone or name or abbreviation. %%     A literal `%' character. The  broken-down  time structure tm is defined in &lt;time.h&gt; as follows: struct tm { int     tm_sec;         /* seconds */ int     tm_min;         /* minutes */ int     tm_hour;        /* hours */ int     tm_mday;        /* day of the month */ int     tm_mon;         /* month */ int     tm_year;        /* year */ int     tm_wday;        /* day of the week */ int     tm_yday;        /* day in the year */ int     tm_isdst;       /* daylight saving time */ }; The members of the tm structure are: tm_sec The number of seconds after the minute, normally in the range 0 to 59, but can be up to 61 to allow for leap seconds. tm_min The number of minutes after the hour, in the  range 0 to 59. tm_hour The  number  of hours past midnight, in the range 0 to 23. tm_mday The day of the month, in the range 1 to 31. tm_mon The number of months since January, in the range  0 to 11. tm_year The number of years since 1900. tm_wday The  number of days since Sunday, in the range 0 to 6. tm_yday The number of days since January 1, in the range  0 to 365. tm_isdst A  flag that indicates whether daylight saving time is in effect at the time described.  The  value  is positive if daylight saving time is in effect, zero if it is not, and negative if  the  information  is not available.
/*/

