#include "main.h"


int Application::Main(const ArgumentList& args)
{
	MainForm mfrm;
	mfrm.Show();
	return 0;
}



void MainForm::Initialize()
{
	//!init
	this->SetBounds(0, 0, 600, 400);
	this->onContextMenu.Add_H(this, &This::DoOnContextMenu);
	//!<ToolBar>
	toolBar.dock = Dock::cTop;
	toolBar.height = 40;
	this->Add(&toolBar);
	//!<StatusBar>
	statusBar.dock = Dock::cBootom;
	statusBar.height = 23;
	this->Add(&statusBar);
	//!end
}

enum {cFileNew=1000, cFileOpen, cFileClose, cFileExit, cEditDelLine, cEditCopy, cEditCut, cEditPaste, cEditUndo, cEditSpecial, cHelpAbout};
	//this->onContextMenu.Add_H(this, &This::DoOnCtxMenu);


void MainForm::CreateActionList() {
	//Action *fileNew, *openNew;
	fileActs.Init(this, &This::HandleCmd);
	fileNew = fileActs.Add(cFileNew, "New",  aCtrl+'N', Icons::icoNew);
	fileActs.Separator();
	fileActs.Add( cFileOpen, "Open file", vkF12, Icons::icoOpen);
	fileActs.Add( cFileClose, "Close file", vkF11, Icons::icoCloseAll);
	fileActs.Add( cFileExit, "Exit", aNone, Icons::icoQuit);

	// -- Edit actions -->
	editActs.Init(this, &This::HandleCmd);
	editActs.Add(cEditDelLine, "&Del Line",  aNone);
	editActs.Block("A submenu");
	editActs.Add(cEditCopy , "&Copy",  aCtrl+'C', Icons::icoCopy);
	editActs.Add(cEditPaste, "Cut",  aCtrl+'X' , Icons::icoCut);
	editActs.Add(cEditPaste, "&Paste",  aCtrl+'V' , Icons::icoPaste);
	editActs.End();
	editUndo= editActs.Add(cEditUndo, "&Undo",  aCtrl+'Z', Icons::icoUndo );
	editActs.Add(cEditSpecial, "Use special handler",  aNone, cImgNone,  &This::SpecialHandleCmd);

	// -- Help actions -->
	helpActs.Init(this);
	helpActs.Add(cHelpAbout, "&About",  aNone, cImgNone, &This::SpecialHandleCmd);

	//Context menu actions
	ctxActs.Init(this);
	ctxActs.Add(fileNew);  // if you have pointer (safe use) or ...
	ctxActs.Add(editActs[2]);
	ctxActs.Add(editActs[3]);
}


void MainForm::SetupMenu()
{
	// file menu
	fileMenu.Load(fileActs);
	fileMenu.SetLabel("File");
	menuBar.Add(&fileMenu);
	// edit menu
	editMenu.Load(editActs);
	editMenu.SetLabel("Edit");
	menuBar.Add(&editMenu);
	// help menu
	helpMenu.Load(helpActs);
	helpMenu.SetLabel("Help");
	menuBar.Add(&helpMenu);

	this->SetMenu(&menuBar);
}

void MainForm::SetupToolbar()
{
	toolBar.items.Add(fileNew);  // if you have pointer (safe use) or ...
	toolBar.items.Add(fileActs[2]);
	toolBar.items.Add(_separator_);
	toolBar.items.Add(editActs[2]);
	toolBar.items.Add(editActs[3]);
	toolBar.items.Add(editActs[4]);
	//toolBar.items.Add(&edit);
}

void MainForm::SetupStatusbar()
{
	statusBar.AddPane(cFillPane, "Ready");
	statusBar.AddPane(200, "Ln 0   Col 0");
}



void MainForm::HandleCmd(int cmd)
{
	MsgBox("The command is:"+ToStr(cmd));
}


void MainForm::SpecialHandleCmd(int cmd)
{
	statusBar.SetPaneText(1, "The special command: "+ToStr(cmd));
}



void MainForm::DoOnContextMenu(MouseArgs &e)
{
	static ContextMenu edCtxMenu(ctxActs);
	this->contextMenu = &edCtxMenu;
}

