#include "main.h"


int Application::Main(const ArgumentList& args)
{
	MainForm mfrm;
	mfrm.Show();
	return 0;
}


void MainForm::Initialize()
{
	//!init
	this->SetBounds(0, 0, 892, 596);
	this->text = "Draw test";
	//!<PaintBox>
	paintBox.dock = Dock::cFill;
	paintBox.onMouseMove.Add_H(this, &This::PaintBoxDoOnMouseMove);
	paintBox.onMouseDown.Add_H(this, &This::PaintBoxDoOnMouseDown);
	paintBox.onDblClick.Add_H(this, &This::PaintBoxDoOnDblClick);
	paintBox.onPaint.Add_H(this, &This::PaintBoxDoOnPaint);
	this->Add(&paintBox);
	//!<Panel>
	panel.dock = Dock::cRight;
	panel.width = 151;
	this->Add(&panel);
	//!<Button>
	closeBtn.SetBounds(21, 481, 114, 42);
	closeBtn.text = "Close";
	closeBtn.icon = Icons::icoQuit;
	closeBtn.onClick.Add_H(this, &This::CloseBtnDoOnClick);
	panel.Add(&closeBtn);
	//!<Panel>
	panel1.dock = Dock::cTop;
	panel1.height = 32;
	this->Add(&panel1);
	//!<Label>
	label1.SetBounds(23, 5, 326, 18);
	label1.text = "Press left mouse key and move cursor to draw a line";
	panel1.Add(&label1);
	//!<RadioGroup>
	typeSel.SetBounds(18, 108, 113, 65);
	typeSel.text = "Type:";
	typeSel.items.Add("Curve");
	typeSel.items.Add("Segments");
	typeSel.itemIndex = 0;
	panel.Add(&typeSel);
	//!<SpinEdit>
	spinEdit1.SetBounds(18, 68, 112, 23);
	spinEdit1.value = 3;
	spinEdit1.onChange.Add_H(this, &This::SpinEdit1DoOnChange);
	panel.Add(&spinEdit1);
	//!<Label>
	label2.SetBounds(18, 52, 71, 18);
	label2.text = "Pen width:";
	panel.Add(&label2);
	//!<Panel>
	panel2.SetBounds(45, 199, 55, 23);
	panel2.backColor = Color::cBlack;
	panel.Add(&panel2);
	//!<Button>
	button1.SetBounds(21, 227, 114, 42);
	button1.text = "Line color";
	button1.icon = Icons::icoPicker;
	button1.onClick.Add_H(this, &This::Button1DoOnClick);
	panel.Add(&button1);
	//!end
}


void MainForm::CloseBtnDoOnClick(NotifyArgs &e)
{
  Close();
}


void MainForm::PaintBoxDoOnMouseMove(MouseArgs &e)
{
	if((e.buttons&MouseArgs::cLeftBtn)!=0 && typeSel.itemIndex==0) {
		Graphics g(paintBox);
		g.MoveTo(pen_x, pen_y);
		g.SetPen(drawPen);
		g.LineTo(e.x, e.y);
		pen_x = e.x;
		pen_y = e.y;
		paintBox.Redraw();
	}
	m_x = e.x;
	m_y = e.y;
	paintBox.Invalidate();
}


void MainForm::PaintBoxDoOnMouseDown(MouseArgs &e)
{
	if(e.buttons&MouseArgs::cLeftBtn) {
		if(typeSel.itemIndex==1) {
			Graphics g(paintBox);
			g.MoveTo(pen_x, pen_y);
			if(!first_click) {
				g.SetPen(drawPen);
				g.LineTo(e.x, e.y);
				paintBox.Redraw();
			}
			first_click = false;
		}
		pen_x = e.x;
		pen_y = e.y;
	}
}





void MainForm::Button1DoOnClick(NotifyArgs &e)
{
	ChooseColorDialog dlg(drawPen.GetColor());
	if(dlg.Execute()) {
		panel2.backColor = dlg.color;
		drawPen.SetColor(dlg.color);
	}
}




void MainForm::PaintBoxDoOnDblClick(NotifyArgs &e)
{
	first_click = true;
}



void MainForm::PaintBoxDoOnPaint(PaintArgs &e)
{
	if(typeSel.itemIndex==1 && !first_click) {
		e.gdc->MoveTo(pen_x,pen_y);
		e.gdc->LineTo(m_x,m_y);
	}
}



void MainForm::SpinEdit1DoOnChange(NotifyArgs &e)
{
	drawPen.SetWidth(spinEdit1.value);
}

