#include "MyEditor.h"
#include "main.h"
#include <cassert>
#include <lexer/CppDefs.h>
#include <lexer/PasDefs.h>
#include <lexer/JavaDefs.h>
#include <lexer/HtmlDefs.h>
#include <lexer/CssDefs.h>
#include <lexer/LaTeXDefs.h>
#include <lexer/TxtDefs.h>
#include <lexer/FortranDefs.h>


static SciStyleDef  cdefStyles[] ={
{ 0,0x0},    				//Default
{ 1,0x0},    				//Default
{uint(-1)}
};

static SciLexerDef defaultLexer(cdefStyles, "");


class LexerDefs
{
	vector<SciLexerDef*> regs;
public:
	LexerDefs(){
		Register(&CppDefs::sciEditDef);
		Register(&PasDefs::sciEditDef);
		Register(&JavaScriptDefs::sciEditDef);
		Register(&JavaDefs::sciEditDef);
		Register(&HtmlDefs::sciEditDef);
		Register(&CssDefs::sciEditDef);
		Register(&LaTeXDefs::sciEditDef);
		Register(&PropsDefs::sciEditDef);
		Register(&BatchDefs::sciEditDef);
		Register(&FortranDefs::sciEditDef);
	}
	void Register(SciLexerDef *def);
	SciLexerDef* GetLang(const string ext)
	{
		for(int i=0; i<regs.size(); ++i) {
			string pat = regs[i]->fileFilter;
			pat+=";";
			int ix = pat.find(ext);
			if(ix>0)return regs[i];
		}
		return &defaultLexer;
	}
};


void LexerDefs::Register(SciLexerDef *def){
	regs.push_back(def);
}


static LexerDefs lexerTabs;


MyEditor::MyEditor()
{
	static MarkerDef ideMarkers[4] = {
		MarkerDef(SC_MARK_CIRCLE, colourDesired(0x7f, 0, 0), colourDesired(0,0,0xff)),
		MarkerDef(SC_MARK_ROUNDRECT, colourDesired(0x7f, 0, 0), colourDesired(0xff, 0, 0)),
		MarkerDef(SC_MARK_SHORTARROW, colourDesired(0x7f, 0, 0), colourDesired(0, 0, 0xff)),
		MarkerDef(SC_MARK_ARROW, colourDesired(0x7f, 0, 0), colourDesired(0, 0xff, 0))
	};
	markers = ideMarkers;
	markers[cMarkBreak].single = false;
	maxMarkers = 4;
	SetLexerDef(&defaultLexer);
	props.propFontSize = 11;
	props.monoFontSize = 11;
	//Configure(userCfg, false);
}



SciLexerDef* MyEditor::GetLexerDef(const string fn)
{
	string ext;
	int pix = fn.rfind('.');
	if(pix>0) {
		ext = fn.substr(pix, fn.length()-pix)+";";
		return lexerTabs.GetLang(ext);
	}
	return 0;
}



void MyEditor::InsertStrList(StringList &lines) {
	string s;
	for(int n=0; n<lines.count(); ++n) {
		s += lines[n]+"\r\n";
	}
	InsertText(s.c_str());
}



int MyEditor::GetMarginWidth() {
	if(props.margin) return props.marginWidth;
	else return 0;
}


int MyEditor::ToggleBreakpoint(int &line, int oper) {
  if(line<0)
    line = GetCurrentLineNumber();
  bool Add = (GetMarker(line) & 8)==0;
	if(oper) Add = oper>0;
  int sciH = HighlightLine(line, cMarkBreak, Add);
  if(!Add) sciH =-sciH;
  return sciH;
}



void MyEditor::Open(string fn)
{
	filePath = fn;
	SciLexerDef *def = GetLexerDef(fn);
	if(def) {
		SetLexerDef(def);
	} else
		MsgBox("Problem");
	OpenFile(fn.c_str());
}


bool MyEditor::Save()
{
	bool res = SaveBuffer(filePath.c_str());
	SciLexerDef *def = GetLexerDef(filePath);
	if(def) {
		SetLexerDef(def);
	}
	return res;
}


void MyEditor::OnChangeState() {
	if(modified)
		isBrowsed = false;
}


void MyEditor::OnCharAdded() {
}


bool MyEditor::FillCompleteList(string &pat, string &list) {
	return false;
}

bool MyEditor::FillCallTipList(string &pat, string &list) {
  return false;
}


void MyEditor::UpdateDisplay(int type) {
	int caretPos = SendEditor(SCI_GETCURRENTPOS);
	int lineNum = SendEditor(SCI_LINEFROMPOSITION, caretPos);
	int colNum = SendEditor(SCI_GETCOLUMN, caretPos);
	//propsStatus.Set("OverType", SendEditor(SCI_GETOVERTYPE) ? "OVR" : "INS");
	mainForm->ViewCurPos(colNum, lineNum);
	mainForm->SetMacroBtn(recording);
}


void MyEditor::OnWordTooltip(int pos) {
}





