#include "main.h"

MainForm *mainForm = 0;

string loadFilePath;

int Application::Main(const ArgumentList& args)
{
	MainForm mfrm;
	mainForm = &mfrm;
    for(int ix=0; ix<args.size(); ++ix){
		string s = args[ix];
        loadFilePath = s;
    }

	mfrm.Show();
	return 0;
}



void MainForm::Initialize()
{
	//!init
	this->SetBounds(0, 0, 1188, 844);
	this->onContextMenu.Add_H(this, &This::DoOnContextMenu);
	this->onCreate.Add_H(this, &This::DoOnCreate);
	this->onClose.Add_H(this, &This::DoOnClose);
	//!<ToolBar>
	toolBar.dock = Dock::cTop;
	toolBar.height = 31;
	this->Add(&toolBar);
	//!<StatusBar>
	statusBar.dock = Dock::cBootom;
	statusBar.height = 23;
	this->Add(&statusBar);
	//!<Memo>
	editor.dock = Dock::cFill;
	this->Add(&editor);
	//!end
	this->focusCtrl = &editor;
}

enum {cFileNew=1000, cFileOpen, cFileSave, cFileSaveAs, cFileClose,  cHelpAbout};

const char *editorFiters = "All files|*.*||";

void MainForm::CreateActionList() {
	//Action *fileNew, *openNew;
	fileActs.Init(this, &This::HandleCmd);
	fileNew = fileActs.Add(cFileNew, "New",  aCtrl+'N', Icons::icoNew);
	fileActs.Separator();
	fileOpen = fileActs.Add( cFileOpen, "Open file", aCtrl+'O', Icons::icoOpen);
	fileSave = fileActs.Add( cFileSave, "Save", aCtrl+'S', Icons::icoSave);
	fileActs.Add( cFileSaveAs, "Save", aCtrl+'S');
	fileActs.Add( cFileClose, "Exit", aNone, Icons::icoQuit);

	// -- Help actions -->
	helpActs.Init(this);
	helpActs.Add(cHelpAbout, "&About",  aNone, cImgNone, &This::SpecialHandleCmd);

	//Context menu actions
	ctxActs.Init(this);
	ctxActs.Add(fileNew);
	//ctxActs.Add(editActs[2]);
	//ctxActs.Add(editActs[3]);
}


void MainForm::SetupMenu()
{
	// file menu
	fileMenu.Load(fileActs);
	fileMenu.SetLabel("File");
	menuBar.Add(&fileMenu);
	// edit menu
	editorActs.Link(&editor);

	//editMenu.Load(editActs);
	editMenu.Load(editorActs);

	editMenu.SetLabel("Edit");
	menuBar.Add(&editMenu);
	// help menu
	helpMenu.Load(helpActs);
	helpMenu.SetLabel("Help");
	menuBar.Add(&helpMenu);

	this->SetMenu(&menuBar);
}

void MainForm::SetupToolbar()
{
	toolBar.items.Add(_separator_);
	toolBar.items.Add(fileNew, Icons::icoNew);
	toolBar.items.Add(fileOpen, Icons::icoOpen);
	toolBar.items.Add(fileSave, Icons::icoSave);
	toolBar.items.Add(_separator_);
	toolBar.items.Add(editorActs.edCut, Icons::icoCut);
	toolBar.items.Add(editorActs.edCopy, Icons::icoCopy);
	toolBar.items.Add(editorActs.edPaste, Icons::icoPaste);
	toolBar.items.Add(_separator_);
	toolBar.items.Add(editorActs.edRecMacro, Icons::icoRedBall);
	toolBar.items.Add(editorActs.edPlayMacro, Icons::icoGo);
}

void MainForm::SetupStatusbar()
{
	statusBar.AddPane(cFillPane, "Ready");
	statusBar.AddPane(200, "Ln 0   Col 0");
}

void MainForm::ViewCurPos(int x, int y)
{
  char buf[32];
  sprintf(buf, "Ln %d   Col %d", y+1, x+1);
  statusBar.SetPaneText(1, buf);
}

void MainForm::SetMacroBtn(bool on) {
	if(on) {
		toolBar.items.SetState(9, bstPress);
		//toolBar.SetState(11, bstCheck);
		statusBar.SetPaneText(0, "Macro recording");
	} else {
		toolBar.items.SetState(9, -bstPress);
		statusBar.SetPaneText(0, " ");
	}
}


void MainForm::OpenFile(string fn)
{
  //userCfg.recentFiles.Add(fn);
	editor.Open(fn);
	text = editor.filePath;
}


void MainForm::CmdNewFile()
{
	editor.New();
	editor.filePath.clear();
	text = "?New-?";
}


void MainForm::CmdOpenFile()
{
  FileDialog fd("Open...");
  fd.filter = editorFiters;
  //fd.defaultExt = "cpp";
  if(fd.Execute())
    OpenFile(fd.directory + fd.fileName);
}


bool MainForm::CmdSaveFile(bool  as)
{
	if (as || editor.filePath.length()==0 ) {
		string info =  "Save";
		if(as) info += " as";
		FileDialog fd(info, FileDialog::cSave);
		fd.filter = editorFiters;
		fd.defaultExt = "txt";
		if(!fd.Execute()) return false;
		const string fn = fd.directory+fd.fileName;
		editor.filePath=fn;
		text = fd.fileName;
	}
	if(!editor.Save()){
		string s="Error on save file:";
		MsgBox( s+editor.filePath);
		return false;
	}
	return true;
}


void MainForm::CmdAbout()
{
}


void MainForm::CmdClose()
{
}


void MainForm::HandleCmd(int cmd)
{
	switch(cmd)
	{
		case cFileNew:
				CmdNewFile();
		break;
		case cFileOpen:
				CmdOpenFile();
		break;
		case cFileSave:
		case cFileSaveAs:
			CmdSaveFile(cmd==cFileSaveAs);
		break;
		case cFileClose:
			CmdClose();
		break;
		case cHelpAbout:
			CmdAbout();
		break;
	}
}


void MainForm::SpecialHandleCmd(int cmd)
{
	statusBar.SetPaneText(1, "The special command: "+ToStr(cmd));
}



void MainForm::DoOnContextMenu(MouseArgs &e)
{
	static ContextMenu edCtxMenu(ctxActs);
	this->contextMenu = &edCtxMenu;
}


void MainForm::DoOnCreate(NotifyArgs &e)
{
	if(!loadFilePath.empty())
		OpenFile(loadFilePath);
}



void MainForm::DoOnClose(NotifyArgs &e)
{
	if(editor.IsModified()){
		string s="File:   ";
		s+=editor.filePath;
		s+="\n\nwas modified!\tSave?";
		bool cancel;
		if(ConfBox(s, &cancel))
			CmdSaveFile(false);
		if(cancel) e.code=0;
	}
}

