#include "main.h"


MainForm *mainForm=NULL;

int Application::Main(const ArgumentList& args)
{
	MainForm mainF;
	mainForm = &mainF;
	mainF.Show();
}



bool doRun = false;
int tn;

double xp0, yp0;

const int cMaxPoints = 1000;

struct MyPoint
{
	double x, y;
} points[cMaxPoints];


MainForm::MainForm()
{
	Initialize();
	plot.AddSeries(&series1);
	Randomize();
}


void MainForm::Initialize() {
	//!init
	this->SetBounds(0, 0, 862, 565);
	this->text = "MainForm";
	//!<PaintBox>
	paintBox.dock = Dock::cFill;
	paintBox.onDraw.Add_H(this, &This::DoOnDraw);
	this->Add(&paintBox);
	//!<Button>
	startBtn.SetBounds(191, 429, 84, 34);
	startBtn.text = "Start";
	startBtn.kind = Button::cRun;
	startBtn.onClick.Add_H(this, &This::OnStartBtnClick);
	panel1.Add(&startBtn);
	//!<PlotXY>
	plot.SetBounds(6, 70, 295, 282);
	panel1.Add(&plot);
	//!<Label>
	label2.SetBounds(23, 380, 80, 16);
	label2.text = "Step:";
	panel1.Add(&label2);
	//!<Edit>
	stepVal.SetBounds(25, 398, 80, 20);
	stepVal.backColor = Color::cLightGray;
	stepVal.text = "0";
	panel1.Add(&stepVal);
	//!<Panel>
	diode.SetBounds(209, 411, 36, 14);
	diode.borderStyle = BorderStyle::cRised;
	panel1.Add(&diode);
	//!<Panel>
	panel1.dock = Dock::cRight;
	panel1.width = 304;
	panel1.borderStyle = BorderStyle::cFixed3d;
	this->Add(&panel1);
	//!end
}

void MainForm::DoOnDraw(NotifyArgs &e)
{
	Graphics g(paintBox);
	g.Clear();
	const double rd =3;
	for(int i=0; i<cMaxPoints; ++i) {
		g.Circle(points[i].x,  points[i].y, rd);
	}
	g.ApplyCache();
}


void MainForm::BeforeStart()
{
	tn =0;
	sysClock.Reset();
	series1.Clear();
	xp0 = paintBox.width/2;
	yp0 = paintBox.height/2;
	for(int i=0; i<cMaxPoints; ++i) {
		points[i].x = xp0;
		points[i].y = yp0;
	}
}



void CountData()
{
	double avgR = 0;
	for(int i=0; i<cMaxPoints; ++i) {
		double dx = points[i].x - xp0;
		double dy = points[i].y - yp0;
		avgR += dx*dx + dy*dy;
	}
	avgR = sqrt(avgR/cMaxPoints);
	mainForm->series1.AddXY(tn, avgR);
}


void RunProc(void *p) {
	while(doRun){
		for(int i=0; i<cMaxPoints; ++i) {
			double dx = Random()-0.5;
			double dy = Random()-0.5;
			points[i].x += dx;
			points[i].y += dy;
		}
		++tn;
		if(tn%10==0){
			CountData();
			ThreadJob::StartLock();
			mainForm->plot.Redraw();
			mainForm->stepVal.text = IntToStr(tn);
			mainForm->paintBox.Redraw();
		    //ThreadJob::sleep(1);
			ThreadJob::StopLock();
		}
	}
	ThreadJob::StartLock();
	mainForm->diode.backColor = Color::cDefault;
	ThreadJob::StopLock();
}


ThreadJob tjob(RunProc);



void MainForm::OnStartBtnClick(NotifyArgs &e)
{
	if(!doRun) {
		doRun = true;
		diode.backColor = Color::cRed;
		startBtn.text = "Stop";
		BeforeStart();
		tjob.Run(0);
	}
	else
	{
		startBtn.text = "Start";
		doRun = false;
	}
}



