#include "main.h"
#include "dynam2d.h"


MainForm *mainForm=NULL;
bool doRun = false;


int Application::Main(const ArgumentList& args)
{
	MainForm mainF;
	mainForm = &mainF;
	mainF.Show();
}


MainForm::MainForm()
{
	Initialize();
}


void MainForm::Initialize()
{
	//!init
	this->SetBounds(0, 0, 755, 565);
	this->text = "MainForm";
	//!<PaintBox>
	paintBox.dock = Dock::cFill;
	paintBox.onDraw.Add_H(this, &This::DoOnDraw);
	this->Add(&paintBox);
	//!<Button>
	startBtn.SetBounds(7, 419, 84, 34);
	startBtn.text = "Start";
	startBtn.kind = Button::cRun;
	startBtn.onClick.Add_H(this, &This::OnStartBtnClick);
	panel1.Add(&startBtn);
	//!<Label>
	label2.SetBounds(9, 32, 80, 16);
	label2.text = "Step:";
	panel1.Add(&label2);
	//!<Edit>
	stepVal.SetBounds(9, 50, 80, 20);
	stepVal.backColor = Color::cLightGray;
	stepVal.text = "0";
	panel1.Add(&stepVal);
	//!<Panel>
	diode.SetBounds(25, 401, 36, 14);
	diode.borderStyle = BorderStyle::cRised;
	panel1.Add(&diode);
	//!<Panel>
	panel1.dock = Dock::cRight;
	panel1.width = 100;
	panel1.borderStyle = BorderStyle::cFixed3d;
	this->Add(&panel1);
	//!end
}


// System definition
const double bwd=500, bht=500;

class Pudlo {
public:
	double x1,x2,y1,y2;
	Pudlo() {
		x1=5, y1=5, x2=bwd+5, y2=bht+5;
	}
	void Rysuj(Graphics &g) {
		g.Rectangle(x1,y1,x2,y2);
	}
};

class Kulka {
public:
	double x,y,r, vx,vy, m;
	Kulka() {
		x=Random(bwd-20)+15; y=Random(bwd-20)+15;
		m=4;
		vx=2, vy=2; r=4;
	}
	void Ustaw(double _vx, double _vy, double _m=4) {
		r=m=_m;
		vx=_vx; vy=_vy;
	}
	void Rysuj(Graphics &g) {
		g.Circle(x,y,r);
	}
	void Przesun(double dt, Pudlo &p){
		x+=vx*dt;
		if(x+r>p.x2) {
			x = p.x2-r;
			vx = -vx;
		}
		if(x-r<p.x1) {
			x = p.x1+r;
			vx = -vx;
		}
		y+=vy*dt;
		if(y+r>p.y2) {
			y = p.y2-r;
			vy = -vy;
		}
		if(y-r<p.y1) {
			y = p.y1+r;
			vy = -vy;
		}
	}
};


Pudlo pudlo;
const int Nk=250; //Liczba kulek
const int Np=1000; //Max dlugosc sladu

Kulka ktab[Nk], k0;

Point pt[Np];  //slad
int pn=0;

void MainForm::DoOnDraw(NotifyArgs &e)
{
	Graphics g(paintBox);
	g.Clear();
	// Draw system state
	pudlo.Rysuj(g);
	static Brush bred(Color::cRed), bgrn(Color::cGreen);
	g.SetBrush(bred);
	for(int n=0; n<Nk; ++n) {
		ktab[n].Rysuj(g);
	}
	g.SetBrush(bgrn);
	k0.Rysuj(g);
	g.Polyline(pt, pn);
}

int tn;  //kroki

void MainForm::BeforeStart()
{
	// Set initial state of your system
	tn = 0;
	Randomize();
	for(int n=0; n<Nk; ++n) {
			ktab[n].Ustaw(Random(4)-2, Random(4)-2);
	}
	k0.vx=k0.vy=0;
	k0.m= 10;
	k0.r=12;
}


void RunProc(void *p)
{
	const double dt=0.02;
	while(doRun){
		//calculation code
		for(int n=0; n<Nk; ++n) {
			ktab[n].Przesun(dt, pudlo);
		}
		k0.Przesun(dt, pudlo);
		for(int n=0; n<Nk; ++n) {
			for(int m=n+1; m<Nk; ++m) {
				Collision2D(ktab[m],ktab[n]);
			}
			Collision2D(k0,ktab[n]);
		}
		if(tn%200==0) {
			if(pn<Np) {
				pt[pn].x = k0.x;
				pt[pn].y = k0.y;
				++pn;
			} else {
				pn=0;
			}
			ThreadJob::StartLock();
			mainForm->paintBox.Redraw();
			mainForm->stepVal.text=ToStr(tn);
			//ThreadJob::Sleep(1);
			ThreadJob::StopLock();
		}
		++tn;
	}
	ThreadJob::StartLock();
	mainForm->diode.backColor = Color::cDefault;
	ThreadJob::StopLock();
}


ThreadJob tjob(RunProc);



void MainForm::OnStartBtnClick(NotifyArgs &e)
{
	if(!doRun) {
		doRun = true;
		diode.backColor = Color::cRed;
		startBtn.text = "Stop";
		BeforeStart();
		tjob.Run(0);
	}
	else
	{
		startBtn.text = "Start";
		doRun = false;
	}
}


