#include "main.h"
#include "dynam2d.h"


MainForm *mainForm=NULL;
bool doRun = false;

Series ser1,ser2;

int Application::Main(const ArgumentList& args)
{
	MainForm mainF;
	mainForm = &mainF;
	mainF.Show();
}


MainForm::MainForm()
{
	Initialize();
}


void MainForm::Initialize()
{
	//!init
	this->SetBounds(0, 0, 774, 565);
	this->text = "MainForm";
	//!<PaintBox>
	paintBox.SetBounds(0, 0, 521, 529);
	paintBox.onDraw.Add_H(this, &This::DoOnDraw);
	this->Add(&paintBox);
	//!<Button>
	startBtn.SetBounds(114, 447, 84, 34);
	startBtn.text = "Start";
	startBtn.kind = Button::cRun;
	startBtn.onClick.Add_H(this, &This::OnStartBtnClick);
	panel1.Add(&startBtn);
	//!<Label>
	label2.SetBounds(15, 429, 80, 16);
	label2.text = "Step:";
	panel1.Add(&label2);
	//!<Edit>
	stepVal.SetBounds(15, 447, 80, 20);
	stepVal.backColor = Color::cLightGray;
	stepVal.text = "0";
	panel1.Add(&stepVal);
	//!<Panel>
	diode.SetBounds(132, 429, 36, 14);
	diode.borderStyle = BorderStyle::cRised;
	panel1.Add(&diode);
	//!<Panel>
	panel1.dock = Dock::cRight;
	panel1.width = 227;
	panel1.borderStyle = BorderStyle::cFixed3d;
	this->Add(&panel1);
	//!<Edit>
	ed1.SetBounds(15, 47, 80, 20);
	ed1.text = "0";
	panel1.Add(&ed1);
	//!<Edit>
	ed2.SetBounds(110, 47, 80, 22);
	ed2.text = "0";
	panel1.Add(&ed2);
	//!<Label>
	label3.SetBounds(19, 29, 61, 16);
	label3.foreColor = Color::cBlue;
	label3.text = "Ek1";
	panel1.Add(&label3);
	//!<Label>
	label4.SetBounds(114, 30, 80, 16);
	label4.foreColor = Color::cRed;
	label4.text = "Ek2";
	panel1.Add(&label4);
	//!<PlotXY>
	plotXY1.SetBounds(4, 139, 215, 187);
	panel1.Add(&plotXY1);
	//!end
	plotXY1.AddSeries(&ser1, Color::cBlue);
	plotXY1.AddSeries(&ser2, Color::cRed);
}


// System definition
const double bwd=500, bht=500;

class Pudlo {
public:
	double x1,x2,y1,y2;
	Pudlo() {
		x1=5, y1=5, x2=bwd+5, y2=bht+5;
	}
	void Rysuj(Graphics &g) {
		g.Rectangle(x1,y1,x2,y2);
	}
};

class Kulka {
public:
	double x,y,r, vx,vy, m;
	Kulka() {
		x=Random(bwd-20)+15; y=Random(bwd-20)+15;
		m=r=4;
		vx=4*Random()-2, vy=4*Random()-2;
	}
	void Ustaw(double _vx, double _vy, double _m=4) {
		r=m=_m;
		vx=_vx; vy=_vy;
	}
	void Rysuj(Graphics &g) {
		g.Circle(x,y,r);
	}
	void Przesun(double dt, Pudlo &p){
		x+=vx*dt;
		if(x+r>p.x2) {
			x = p.x2-r;
			vx = -vx;
		}
		if(x-r<p.x1) {
			x = p.x1+r;
			vx = -vx;
		}
		y+=vy*dt;
		if(y+r>p.y2) {
			y = p.y2-r;
			vy = -vy;
		}
		if(y-r<p.y1) {
			y = p.y1+r;
			vy = -vy;
		}
	}
};


Pudlo pudlo;
const int Nk=200; //Liczba kulek
const int Np=1000; //Max dlugosc sladu

Kulka ktab[Nk];

void MainForm::DoOnDraw(NotifyArgs &e)
{
	Graphics g(paintBox);
	g.Clear();
	// Draw system state
	pudlo.Rysuj(g);
	static Brush bred(Color::cRed), bble(Color::cBlue);
	g.SetBrush(bble);
	for(int n=0; n<Nk; ++n) {
		if(n==Nk/2) {
			g.SetBrush(bred);
		}
		ktab[n].Rysuj(g);
	}
}

int tn;  //kroki

void MainForm::BeforeStart()
{
	// Set initial state of your system
	ser1.Clear();
	ser2.Clear();
	tn = 0;
	Randomize();
	for(int n=0; n<Nk; ++n) {
		Kulka &k=ktab[n];
		if(n<Nk/2) {
			k.Ustaw(0.2*Random()-0.1, 0.2*Random()-0.1);
		}
	}
}


void RunProc(void *p)
{
	const double dt=0.02;
	while(doRun){
		//calculation code
		for(int n=0; n<Nk; ++n) {
			ktab[n].Przesun(dt, pudlo);
		}
		for(int n=0; n<Nk; ++n) {
			for(int m=n+1; m<Nk; ++m) {
				Collision2D(ktab[m],ktab[n]);
			}
		}
		if(tn%200==0) {
			double Ek,Ek1=0, Ek2=0;
			for(int n=0; n<Nk; ++n) {
				Kulka &k=ktab[n];
				Ek=sqr(k.vx) + sqr(k.vy);
				if(n<Nk/2)
					Ek1+=Ek;
				else
					Ek2+=Ek;
			}
			ThreadJob::StartLock();
			ser1.AddXY(tn, Ek1);
			ser2.AddXY(tn, Ek2);
			mainForm->stepVal.text = ToStr(tn);
			mainForm->ed1.text = ToStr(Ek1);
			mainForm->ed2.text = ToStr(Ek2);
			mainForm->paintBox.Redraw();
			mainForm->plotXY1.Redraw();
			//ThreadJob::Sleep(1);
			ThreadJob::StopLock();
		}
		++tn;
	}
	ThreadJob::StartLock();
	mainForm->diode.backColor = Color::cDefault;
	ThreadJob::StopLock();
}


ThreadJob tjob(RunProc);



void MainForm::OnStartBtnClick(NotifyArgs &e)
{
	if(!doRun) {
		doRun = true;
		diode.backColor = Color::cRed;
		startBtn.text = "Stop";
		BeforeStart();
		tjob.Run(0);
	}
	else
	{
		startBtn.text = "Start";
		doRun = false;
	}
}


