#include "main.h"
#include "Dynam2d.h"

//simple demo by
//W.Olchawa
//R.Olchawa


MainForm *mainForm=NULL;
bool doRun = false;


int Application::Main(const ArgumentList& args)
{
	MainForm mainF;
	mainForm = &mainF;
	mainF.Show();
}


MainForm::MainForm()
{
	Initialize();
	Randomize();
}


void MainForm::Initialize()
{
	//!init
	this->SetBounds(0, 0, 668, 565);
	this->text = "Verlet";
	//!<PaintBox>
	paintBox.dock = Dock::cFill;
	paintBox.onDraw.Add_H(this, &This::DoOnDraw);
	this->Add(&paintBox);
	//!<Button>
	startBtn.SetBounds(12, 422, 84, 34);
	startBtn.text = "Start";
	startBtn.kind = Button::cRun;
	startBtn.onClick.Add_H(this, &This::OnStartBtnClick);
	panel1.Add(&startBtn);
	//!<Panel>
	diode.SetBounds(30, 404, 36, 14);
	diode.borderStyle = BorderStyle::cRised;
	panel1.Add(&diode);
	//!<Panel>
	panel1.dock = Dock::cRight;
	panel1.width = 108;
	panel1.borderStyle = BorderStyle::cFixed3d;
	this->Add(&panel1);
	//!<DialCtrl>
	dialCtrl1.SetBounds(1, 244, 104, 124);
	dialCtrl1.pos = 0.001;
	dialCtrl1.min = 0.0004;
	dialCtrl1.max = 0.0016;
	dialCtrl1.showPos = false;
	dialCtrl1.labels = DialCtrl::cNone;
	dialCtrl1.text = "dt";
	panel1.Add(&dialCtrl1);
	//!<SpinEdit>
	Ned.SetBounds(30, 151, 72, 21);
	Ned.min = 2;
	Ned.max = 10;
	Ned.value = 3;
	panel1.Add(&Ned);
	//!<Label>
	label1.SetBounds(8, 154, 20, 18);
	label1.text = "N=";
	panel1.Add(&label1);
	//!end
}




// System definition
struct Body{
	double x, y;  		//position
	double  vx, vy; 	//velocity
	double m;  			//mass
	double r;
	int wall;
	void InitState() {
		x= Random(200)+120;
		y= Random(200)+120;
		vx=vy=0;
		m=8000;
		r=10;
		wall = false;
	}
} ;





const int MAXN=10;

Body btb[MAXN];
int N=2;

double t=0, dt=0.001;

double k1 = 0.5;
int xLewa = 10, xPrawa=480, yGora=10, yDol=460;


void CalcAcceleration(Vect2D a[], Body b[], double t)
{
	//Calc acceleration of all bodies
	for(int i=0; i<N; ++i) {
		double Fx = 0;
		double Fy = 0;
		int j = i+1;
		if(j<N) {
			Fx += -k1 * ( btb[i].x - b[j].x);
			Fy += -k1 * ( btb[i].y - b[j].y);
		}
		j = i - 1;
		if(j>=0) {
			Fx += -k1 * ( btb[i].x - b[j].x);
			Fy += -k1 * ( btb[i].y - b[j].y);
		}
		a[i].x = Fx/b[i].m;
		a[i].y = Fy/b[i].m;
	}
}


void DrawSpring(Graphics& g, int x0, int y0, int x1, int y1, int n)
{
	const double b=15;
	if (n%2 !=0) n++;
	double sx = x1-x0, sy = y1-y0;
	double L   = sqrt( sx*sx + sy*sy );
	sx /= L;  sy/=L;
	double nx = 10*sy, ny = -10*sx;
	double dx = sx*(L-2.*b)/(2.*n),  dy = sy*(L-2.*b)/(2.*n);

	int xs,ys;

	g.MoveTo(x0,y0);
	x0 += lround(b*sx);  y0 += lround(b*sy);
	g.LineTo(x0, y0);
	x0 -= lround(dx/2); y0 -= lround(dy/2);
	for (int i=0; i<n; i++)
	{
		x0 += lround(dx); y0 += lround(dy);
		xs = lround(x0+nx);  ys = lround(y0+ny);
		g.LineTo(xs, ys);
		nx *= -1; ny *= -1;
	}
	g.MoveTo(x1,y1);
	x1 -= lround(b*sx);   y1 -= lround(b*sy);
	g.LineTo(x1, y1);
	x1 += lround(dx/2);   y1 += lround(dy/2);
	for (int i=0; i<n; i++)
	{
		nx *= -1; ny *= -1;
		x1 -= lround(dx); y1 -= lround(dy);
		g.LineTo(lround(x1+nx), lround(y1+ny));
	}
	g.LineTo(xs, ys);
}



enum {isCollision, isPenetrance, noCollision};
double epsZderz = dt*0.05;
Vect2D normalna;
double vNormalna;

int TestCollision(Body& c1, Body& c2)
{
	Vect2D vWzgl;
	normalna.x = c1.x - c2.x;
	normalna.y = c1.y - c2.y;
	double nDl = sqrt(  normalna.x*normalna.x + normalna.y*normalna.y );
	double s = nDl - (c1.r + c2.r);
    normalna.x /= nDl;   normalna.y /= nDl;
	vWzgl.x = c1.vx - c2.vx;
	vWzgl.y = c1.vy - c2.vy;
	vNormalna = normalna.x * vWzgl.x    +    normalna.y * vWzgl.y;
	if ( (abs(s) <= epsZderz)  &&  (vNormalna < 0.0) )
		return isCollision;
	if(s < -epsZderz)
		return isPenetrance;
	return noCollision;
}

int TestWallCollision(Body btb[])
{
	int wynik=noCollision;
	double s;

	for (int i=0; i<N; i++)
	{
		btb[i].wall = 0;
		s = xPrawa -  (btb[i].x + btb[i].r);
		if ( (abs(s) <= epsZderz)  &&  (btb[i].vx > 0.0) )
		{
			wynik = isCollision;
			btb[i].wall = 1;   // ograniczenie  na x
		}
		if (s < -epsZderz)
			wynik = isPenetrance;

		s =   (btb[i].x - btb[i].r) - xLewa;
		if ( (abs(s) <= epsZderz)  &&  (btb[i].vx < 0.0) )
		{
			wynik = isCollision;
			btb[i].wall = 1;   // ograniczenie  na x
		}
		if (s < -epsZderz)
			wynik = isPenetrance;

		s =   (btb[i].y - btb[i].r) -yGora;
		if ( (abs(s) <= epsZderz)  &&  (btb[i].vy < 0.0) )
		{
			wynik = isCollision;
			btb[i].wall = 2;   // ograniczenie  na x
		}
		if (s < -epsZderz)
			wynik = isPenetrance;

		s = yDol -  (btb[i].y + btb[i].r);
		if ( (abs(s) <= epsZderz)  &&  (btb[i].vy > 0.0) )
		{
			wynik = isCollision;
			btb[i].wall = 2;   // ograniczenie  na x
		}
		if (s < -epsZderz)
			wynik = isPenetrance;
	} // for
	return wynik;
}



void InteractImpulse(Body& c1, Body& c2)
{
	double j = -2. * vNormalna / (1./c1.m + 1./c2.m );
	c1.vx += j * normalna.x / c1.m;
	c1.vy += j * normalna.y / c1.m;
	c2.vx -= j * normalna.x / c2.m;
	c2.vy -= j * normalna.y / c2.m;
}

void WallInteractImpulse(Body btb[])
{
	for (int i=0; i<N; i++)
	{
		if (btb[i].wall>0)
			if (btb[i].wall==1)
				btb[i].vx = -btb[i].vx;
			else
				btb[i].vy = -btb[i].vy;
	}
}


void SimulationStep(double dt)
{
	bool probujPonownie = true;
	int sprawdzenie_para = 0;
	int sprawdzenie_sciana = 0;
	int usedBytes = N*sizeof(Body);
	Body cKlon[N];
	while (probujPonownie && dt > 0.5*epsZderz)
	{
		probujPonownie = false;
		memcpy(&cKlon, &btb, usedBytes);
		Verlet(CalcAcceleration, t, dt, cKlon, N);

		sprawdzenie_para = TestCollision( cKlon[0], cKlon[1]);
		sprawdzenie_sciana = TestWallCollision( cKlon);
		if ((sprawdzenie_para == isPenetrance) || (sprawdzenie_sciana == isPenetrance) )
		{
			dt /= 2;
			probujPonownie = true;
		}else if ((sprawdzenie_para == isCollision) || (sprawdzenie_sciana == isCollision) )
			if ((sprawdzenie_para == isCollision) )
				InteractImpulse(cKlon[0], cKlon[1]);
			else
				WallInteractImpulse(cKlon);
	}
	memcpy( &btb,  &cKlon, usedBytes );
}



void MainForm::DoOnDraw(NotifyArgs &e)
{
	Graphics g(paintBox);
	static Brush bcz(Color::cRed);
	static Brush bnb(Color::cBlue);
	g.Clear();
	// Draw system state
	g.MoveTo(xLewa,yGora);
	g.LineTo(xLewa,yDol);
	g.LineTo(xPrawa,yDol);
	g.LineTo(xPrawa,yGora);
	g.LineTo(xLewa,yGora);
	for(int i=0; i<N-1; ++i)
		DrawSpring(g, lround(btb[i].x), lround(btb[i].y), lround(btb[i+1].x) ,lround(btb[i+1].y), 4);
	g.SetBrush(bcz);
	g.Circle(lround(btb[0].x), lround(btb[0].y), 10);
	g.SetBrush(bnb);
	for(int i=1; i<N; ++i)
		g.Circle(lround(btb[i].x), lround(btb[i].y), 10);
}

void MainForm::BeforeStart()
{
	// Set initial state of your system
	N = mainForm->Ned.value;
	assert(N<=MAXN);
	for(int i=0; i<N; ++i) {
		btb[i].InitState();
	}
	btb[0].vx = -0.5;   btb[0].vy = 1;
}


void RunProc(void *p)
{
	int tn = 0;
	while(doRun){
		SimulationStep(dt);
		t += dt;
		if(tn%500==0) {
			ThreadJob::StartLock();
			mainForm->paintBox.Redraw();
			dt = mainForm->dialCtrl1.pos;
			ThreadJob::StopLock();
		}
		++tn;
	}
	ThreadJob::StartLock();
	mainForm->diode.backColor = Color::cDefault;
	ThreadJob::StopLock();
}


ThreadJob tjob(RunProc);


void MainForm::OnStartBtnClick(NotifyArgs &e)
{
	if(!doRun) {
		doRun = true;
		diode.backColor = Color::cRed;
		startBtn.text = "Stop";
		BeforeStart();
		tjob.Run(0);
	}
	else
	{
		startBtn.text = "Start";
		doRun = false;
	}
}


