#include "main.h"
#include "threadjob.h"
#include "dynam2d.h"
#include "cstring"



MainForm *mainForm=NULL;
bool doRun = false;


int Application::Main(const ArgumentList& args)
{
	MainForm mainF;
	mainForm = &mainF;
	mainF.Show();
}


MainForm::MainForm()
{
	Initialize();
	Randomize();
}


void MainForm::Initialize()
{
	//!init
	this->SetBounds(0, 0, 682, 565);
	this->text = "Verlet";
	//!<PaintBox>
	paintBox.dock = Dock::cFill;
	paintBox.onDraw.Add_H(this, &This::DoOnDraw);
	this->Add(&paintBox);
	//!<Button>
	startBtn.SetBounds(12, 422, 84, 34);
	startBtn.text = "Start";
	startBtn.kind = Button::cRun;
	startBtn.onClick.Add_H(this, &This::OnStartBtnClick);
	panel1.Add(&startBtn);
	//!<Panel>
	diode.SetBounds(30, 404, 36, 14);
	diode.borderStyle = BorderStyle::cRised;
	panel1.Add(&diode);
	//!<Panel>
	panel1.dock = Dock::cRight;
	panel1.width = 109;
	panel1.borderStyle = BorderStyle::cFixed3d;
	this->Add(&panel1);
	//!<DialCtrl>
	dialCtrl1.SetBounds(1, 244, 104, 124);
	dialCtrl1.pos = 0.001;
	dialCtrl1.min = 0.0004;
	dialCtrl1.max = 0.0016;
	dialCtrl1.showPos = false;
	dialCtrl1.labels = DialCtrl::cNone;
	dialCtrl1.text = "dt";
	panel1.Add(&dialCtrl1);
	//!end
}




// System definition
struct Body{
	double x, y;  		//position
	double  vx, vy; 	//velocity
	double m;  			//mass
} ;





const int N=1;

Body btb[N];

double t=0, dt=0.001;

void CalcAcceleration(Vect2D a[], Body b[], double t)
{
	//Calc acceleration of all bodies
	a[0].x = 0;
	a[0].y = 0;
}


void SimulationStep(double dt)
{
	Verlet(CalcAcceleration, t, dt, btb, N);
	//Apply boundary conditions
}



void MainForm::DoOnDraw(NotifyArgs &e)
{
	Graphics g(paintBox);
	g.Clear();
	//Draw your system state
}

void MainForm::BeforeStart()
{
	// Set initial state of your system
	btb[0].x = 100; btb[0].y = 100;
	btb[0].vx = 0;   btb[0].vy = 0;
	btb[0].m = 1000;
}


void RunProc(void *p)
{
	int tn = 0;
	while(doRun){
		SimulationStep(dt);
		t += dt;
		if(tn%500==0) {
			ThreadJob::StartLock();
			mainForm->paintBox.Redraw();
			dt = mainForm->dialCtrl1.pos;
			ThreadJob::StopLock();
		}
		++tn;
	}
	ThreadJob::StartLock();
	mainForm->diode.backColor = Color::cDefault;
	ThreadJob::StopLock();
}


ThreadJob tjob(RunProc);


void MainForm::OnStartBtnClick(NotifyArgs &e)
{
	if(!doRun) {
		doRun = true;
		diode.backColor = Color::cRed;
		startBtn.text = "Stop";
		BeforeStart();
		tjob.Run(0);
	}
	else
	{
		startBtn.text = "Start";
		doRun = false;
	}
}


