#pragma once

struct Vect2D{ double x, y; };

/*
//TB
class ABody{
public:
	double x, y, vx, vy, m;
	...
};

//AF
void AnAccelFun(Vect2D acc[],  ABody body[], double t)
{
	acc[0].x = -body[0].x;
	acc[0].y = 0;
	...
}

*/




template<typename AF, class TB>
void RungeKutta2(AF AccelFun, double t, double dt, TB body[], int n)
{
	Vect2D k1v[n], k1r[n];
	Vect2D a[n];
	TB s[n];
	AccelFun(a, body, t);
	for (int i=0;  i<n;  i++)
	{
		k1v[i].x  = dt * a[i].x;
		k1v[i].y  = dt * a[i].y;
		k1r[i].x   = dt * body[i].vx;
		k1r[i].y   = dt * body[i].vy;

		s[i].x  = body[i].x + k1r[i].x;
		s[i].y  = body[i].y + k1r[i].y;
		s[i].vx  = body[i].vx + k1v[i].x;
    	s[i].vy  = body[i].vy + k1v[i].y;
		s[i].m = body[i].m;
	}
	AccelFun(a, s, t+dt);
	for (int i=0;  i<n;  i++)
	{
		body[i].x +=  0.5*( k1r[i].x  + dt * s[i].vx);
		body[i].y +=  0.5*( k1r[i].y  + dt * s[i].vy);
		body[i].vx +=  0.5*( k1v[i].x + dt * a[i].x);
    	body[i].vy +=  0.5*( k1v[i].y + dt * a[i].y);
	}
}


template<typename AF, class TB>
void RungeKutta4(AF AccelFun, double t, double dt, TB body[], int n)
{
	Vect2D k1r[n],  k1v[n], k2r[n],  k2v[n], k3r[n],  k3v[n];
	Vect2D a[n];
	TB s[n];
	AccelFun(a, body, t);
	for (int i=0;  i<n;  i++)
	{
		k1v[i].x  = dt * a[i].x;
		k1v[i].y  = dt * a[i].y;
		k1r[i].x   = dt * body[i].vx;
		k1r[i].y   = dt * body[i].vy;

		s[i].x  = body[i].x + 0.5*k1r[i].x;
		s[i].y  = body[i].y + 0.5*k1r[i].y;
		s[i].vx  = body[i].vx + 0.5*k1v[i].x;
    	s[i].vy  = body[i].vy + 0.5*k1v[i].y;

		s[i].m = body[i].m;
	}
	AccelFun(a, s, t+0.5*dt);
	for (int i=0;  i<n;  i++)
	{
		k2v[i].x  = dt * a[i].x;
		k2v[i].y  = dt * a[i].y;
		k2r[i].x   = dt * s[i].vx;
		k2r[i].y   = dt * s[i].vy;

		s[i].x  = body[i].x + 0.5*k2r[i].x;
		s[i].y  = body[i].y + 0.5*k2r[i].y;
		s[i].vx  = body[i].vx + 0.5*k2v[i].x;
    	s[i].vy  = body[i].vy + 0.5*k2v[i].y;
	}
	AccelFun(a, s, t+0.5*dt);
	for (int i=0;  i<n;  i++)
	{
		k3v[i].x  = dt * a[i].x;
		k3v[i].y  = dt * a[i].y;
		k3r[i].x   = dt * s[i].vx;
		k3r[i].y   = dt * s[i].vy;

		s[i].x  = body[i].x + k3r[i].x;
		s[i].y  = body[i].y + k3r[i].y;
		s[i].vx  = body[i].vx + k3v[i].x;
    	s[i].vy  = body[i].vy + k3v[i].y;
	}
	AccelFun(a, s, t+dt);
	for (int i=0;  i<n;  i++)
	{
		body[i].x   +=  1.0/6.0*(k1r[i].x + 2.*k2r[i].x + 2.*k3r[i].x + dt * s[i].vx);
		body[i].y   +=  1.0/6.0*(k1r[i].y + 2.*k2r[i].y + 2.*k3r[i].y + dt * s[i].vy);
		body[i].vx   +=  1.0/6.0*(k1v[i].x + 2.*k2v[i].x + 2.*k3v[i].x + dt * a[i].x);
		body[i].vy   +=  1.0/6.0*(k1v[i].y + 2.*k2v[i].y + 2.*k3v[i].y + dt * a[i].y);
	}
}


template<typename AF, class TB>
void Verlet(AF AccelFun, double t, double dt, TB body[], int n)
{
	Vect2D a[n];

	AccelFun(a, body, t);
	for (int i=0;  i<n;  i++)
	{
		body[i].vx  += 0.5 * dt * a[i].x;
		body[i].vy  += 0.5 * dt * a[i].y;
		body[i].x  += dt * body[i].vx;
		body[i].y  += dt * body[i].vy;
	}
	AccelFun(a, body, t);
	for (int i=0;  i<n;  i++)
	{
		body[i].vx  += 0.5 * dt * a[i].x;
		body[i].vy  += 0.5 * dt * a[i].y;
	}
}



template<class TB>
bool Collision2D(TB &s1, TB &s2)
{
	double  m21,dvx2,a,x21,y21,vx21,vy21,fy21,r21,sign;
	m21=s2.m/s1.m;
	x21=s2.x-s1.x;
	y21=s2.y-s1.y;
	r21=s2.r+s1.r;
	if(x21*x21 + y21*y21>r21*r21)
			return false;
	vx21=s2.vx-s1.vx;
	vy21=s2.vy-s1.vy;
	if ( (vx21*x21 + vy21*y21) >= 0) return false;
	// to avoid a zero divide:
	fy21=1.0E-12*fabs(y21);
	if ( fabs(x21)<fy21 ) {
		if (x21<0) { sign=-1; } else { sign=1;}
		x21=fy21*sign;
	}
	a=y21/x21;
	dvx2= -2*(vx21 +a*vy21)/((1+a*a)*(1+m21)) ;
	s2.vx=s2.vx+dvx2;
	s2.vy=s2.vy+a*dvx2;
	s1.vx=s1.vx-m21*dvx2;
	s1.vy=s1.vy-a*m21*dvx2;
	return true;
}




