// This file is part of Hermes2D.
//
// Copyright 2005-2008 Jakub Cerveny <jakub.cerveny@gmail.com>
// Copyright 2005-2008 Lenka Dubcova <dubcova@gmail.com>
// Copyright 2005-2008 Pavel Solin <solin@utep.edu>
//
// Hermes2D is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// Hermes2D is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Hermes2D.  If not, see <http://www.gnu.org/licenses/>.

// $Id: space_h1.h 1086 2008-10-21 09:05:44Z jakub $

#ifndef __HERMES2D_SPACE_H1_H
#define __HERMES2D_SPACE_H1_H

#include "space.h"


/// H1Space represents a space of continuous scalar functions over a domain (mesh).
///
///
///
class H1Space : public Space
{
public:

  H1Space(Mesh* mesh = NULL, Shapeset* shapeset = NULL);
  virtual ~H1Space();

  virtual Space* dup(Mesh* mesh) const;

protected:
  
  virtual void assign_vertex_dofs();
  virtual void assign_edge_dofs();
  virtual void assign_bubble_dofs();

  virtual void get_vertex_assembly_list(Element* e, int iv, AsmList* al);
  virtual void get_edge_assembly_list_internal(Element* e, int ie, AsmList* al);
  virtual void get_bubble_assembly_list(Element* e, AsmList* al);

  static double** h1_proj_mat;
  static double*  h1_chol_p;
  static int      h1_proj_ref;

  virtual scalar* get_bc_projection(EdgePos* ep, int order);

  struct EdgeInfo
  {
    Node* node;
    int part;
    int ori;
    double lo, hi;
  };
    
  inline void output_component(BaseComponent*& current, BaseComponent*& last, BaseComponent* min, 
                               Node*& edge, BaseComponent*& edge_dofs);
  
  BaseComponent* merge_baselists(BaseComponent* l1, int n1, BaseComponent* l2, int n2, 
                                 Node* edge, BaseComponent*& edge_dofs, int& ncomponents);                               
  
  void update_constrained_nodes(Element* e, EdgeInfo* ei0, EdgeInfo* ei1, EdgeInfo* ei2, EdgeInfo* ei3);
  virtual void update_constraints();
  
  //void dump_baselist(NodeData& nd);
  
};


#endif
