#pragma once

#include <cstdlib>
#include <cstring>
#include <cstdio>
#include <string>
#include <vector>
#include <map>
#include <SysDefs.h>


//$linux
typedef unsigned int uint;
typedef signed char sbyte;
typedef unsigned char byte;
typedef unsigned short ushort;
typedef long unsigned  int ulong;
typedef signed long long slong;

using namespace std;


namespace rofrm {


//----------- Point -------------

struct Point {
	int x,y;
	Point(): x(0), y(0)
	{}
	Point(int x, int y): x(x), y(y)
	{}
	bool operator==(const Point& rhs) const {
		return x == rhs.x && y == rhs.y;
	}
	bool operator!=(const Point& rhs) const {
		return !(*this == rhs);
	}
};


//---------- Size -----------------

struct Size {
	int width;
	int height;

	Size(int w = 0, int h = 0): height(h), width(w)
	{}
	Size getSize() const {
		return *this;
	}
	void SetSize(const Size& sz) {
		*this = sz;
	}
	void SetSize(int w, int h) {
		width = w;
		height = h;
	}
};

//---------------  Rect -------------

struct Rect: Point,  Size {
	Rect(int x, int y, int width, int height)
		: Point(x, y), Size(width, height) {
    }
	Rect() {
	}

	const Rect& GetBounds() const {
		return *this;
	}

	void SetBounds(const Rect& r) {
		SetBounds(r.x, r.y, r.width, r.height);
	}

	void SetBounds(int x, int y, int width, int height) {
		this->x = x;
		this->y = y;
		this->width = width;
		this->height = height;
	}

	bool Contains(const Point& p) const {
		return Contains(p.x, p.y);
	}

	bool Contains(int X, int Y) const {
		int w = this->width;
		int h = this->height;
		if ((w | h) < 0) {
			return false;
		}
		int x = this->x;
		int y = this->y;
		if (X < x || Y < y) {
			return false;
		}

		w += x;
		h += y;
		// overflow || intersect
		return (w < x || w > X) && (h < y || h > Y);
	}

	bool Empty() const {
		return width <= 0 || height <= 0;
	}
};


struct Range
{
Range(int lt=0, int tp=0, int rt=0, int bt=0) :
    left(lt), top(tp), right(rt), bottom(bt)
	{ }

	int left, top;
	int right, bottom;
	int Height() {return bottom - top;}
	int Width() {return right - left;}
};

inline bool operator==(const Range& lhs, const Range& rhs) {
	return lhs.left == rhs.left && lhs.top == rhs.top &&
		lhs.right == rhs.right && lhs.bottom == rhs.bottom;
}

inline bool operator!=(const Range& lhs, const Range& rhs) {
	return !(lhs == rhs);
}




//--------------- Color ------------------

class Color {
	friend bool operator==(const Color& lhs, const Color& rhs);
	friend bool operator!=(const Color& lhs, const Color& rhs);
public:
	const static Color
		cBlack, cBlue, cCyan, cDarkGray, cGray,
			cGreen, cLightGray, cMagenta, cOrange, cPink,
			cRed, cWhite, cYellow, cDefault;
	Color(int rgb=-1): value(rgb)
	{};
	Color(int r, int g, int b){
		SetRGB(r,g,b);
	}
	int GetRed() const      {return value & 0xFF;}
	int GetGreen() const    {return ( value >> 8 ) & 0xFF;}
	int GetBlue() const     {return ( value >> 16 ) & 0xFF;}
	int GetRGB() const      {return value&0x00FFFFFF;}
	void SetRGB(int r, int g, int b){
		value = ( (r & 0xFF) << 0 )|
			( (g & 0xFF) << 8 )|
				( (b & 0xFF) << 16 );
	}
	Color Brighter(float factor=0.7) const;
	Color Darker(float factor=0.7) const {
		return Color(static_cast<int>(GetRed()*factor),
			static_cast<int>(GetGreen()*factor),
			static_cast<int>(GetBlue()*factor));
	}
	bool IsDefault() { return value==-1;}
	int GetAttr() {
		return attr;
	}
protected:
	union {
		int value;
		struct {
			byte red,green,blue, attr;
		};
	};
};

inline bool operator==(const Color& lhs, const Color& rhs) {
	return lhs.value == rhs.value;
}

inline bool operator!=(const Color& lhs, const Color& rhs) {
	return !(lhs == rhs);
}


class object
{
protected:
	virtual ~object(){};
};


//=============================  property ====================================

template<typename T, bool full=false>
class property
{
protected:
	typedef void (object::*TSetF)(T);
	typedef T (object::*TGetF)();
	TSetF setF;
	TGetF getF;
	object *hobj;
public:
	template<class TS, class TG>
	property(object *h, TG gF, TS sF)
		: hobj(h), getF(static_cast<TGetF>(gF)),
			setF(static_cast<TSetF>(sF))
			{}
	T get() const {
		return  (hobj ->* getF)();
	}
	void set (const T v){
		(hobj ->* setF)(v);
	}
	void operator=(const T v) {set(v);}
	void operator=(const property<T,full> p) {set(p.get());}
	//bool operator==(T v) {return v==value;}
	operator T(void) const {return get();}
	template<class TS, class TG>
	void replaceHandlers(object *h, TG gF, TS sF){
		hobj = h;
		getF = static_cast<TGetF>(gF);
		setF = static_cast<TSetF>(sF);
	}
};


template<typename T>
class property<T, false>
{
protected:
	typedef void (object::*TSetF)();
	TSetF setF;
	object *hobj;
	T &value;
public:
	template<class TS>
	property(object *h, T &v, TS sF)
		: hobj(h), value(v), setF(static_cast<TSetF>(sF))
		{}
	T& get() const {
		return  value;
	}
	void set (const T v){
		if (value != v) {
			value = v;
			(hobj ->* setF)();
		}
	}
	//template<class TS>
	//void replace_upd(TS sF) {updF=static_cast<TUpdF>(sF);}
	void operator=(T v) {set(v);}
	void operator=(const property<T> p) {set(p.get());}
	bool operator==(T v) {return v==value;}
	operator T(void) const {return get();}
	template<class TS>
	void replaceHandler(object *h, TS sF){
		hobj = h;
		setF = static_cast<TSetF>(sF);
	}
};

template<class TA>
class event
{
public:
	typedef void (object::*T)(TA&);
	struct Handler
	{
    Handler(object *b, T f):
		base(b), fun(f), next(0)
		{}
		object *base;
		T fun;
		Handler *next;
	} *list;
public:
	event() { list=NULL; }
	~event() {
		Handler *p=list, *q;
		while(p) {q=p; p=p->next; delete q;}
	}
	bool empty() {return list==NULL;}
	void remove(object *c);
	void exec(TA &e){
		for(Handler *p=list; p; p=p->next)
			(p->base ->* p->fun)(e);
	}
	template<class TH>
	void Add_H(object *b, TH hF){
		link(b, static_cast<T>(hF));
	}
protected:
	void link(object *b, T f){
		Handler *p=list, *h = new Handler(b, f);
		if(list==NULL) list = h;
		else {
			Handler *p=list;
			while(p->next) p=p->next;
			p->next = h;
		}
	}
};



template<class TA>
void event<TA>::remove(object *c)
{
	Handler *p=list, *prv=NULL, *tmp;
	while(p)
	{
		if(p->base ==c )
		{
			tmp=p->next;
			if(prv) prv->next=tmp;
			else list=tmp;
			delete p;
			p=tmp;
		}
		else
		{
			prv=p;
			p=p->next;
		}
	}
};


class StringList
{
public:
	size_t count() const {return lines.size();}
	void Add(const string& item){
		lines.push_back(item);
	}
	void Insert(const string& item, int index){
		lines.insert(lines.begin()+index, item);
	}
	void Remove(int index) {
		lines.erase(lines.begin()+index);
	}
	void Replace(int n, const string& item){
		lines[n] = item;
	}
	void Clear() {lines.clear();}
	bool LoadFromFile(const string name);
	bool SaveToFile(const string name);
	const string& operator[](int n)  const {return lines[n];}
protected:
	vector<string> lines;
};

class ParamList: public std::map<string, string>
{
public:
	void ToStringList(StringList &list){
		const_iterator it;
		for(it=begin(); it!=end(); ++it){
			if(!it->second.empty())
				list.Add(it->first+"="+it->second);
		}
	}
	bool Exists(string key){
		return find(key)!=end();
	}
	void AddStringList(StringList &list){
		for(int i=0; i<list.count(); ++i){
			string s = list[i]; int ix;
			if((ix=s.find('='))>-1){
				(*this)[s.substr(0, ix)]=s.substr(ix+1);
			}
		}
	}
	void Save(const string name){
		StringList list;
		ToStringList(list);
		list.SaveToFile(name);
	}
	void Load(const string name){
		StringList list;
		list.LoadFromFile(name);
		AddStringList(list);
	}
	string GetVal(string key, string def="") {
		if(Exists(key)) return (*this)[key];
		return def;
	}
};


inline string IntToStr(int v){
	char buf[16];
	sprintf(buf, "%d",  v);
	return buf;
}


inline string ToStr(const double v, int prec=0) {
	char buf[32];
	if(prec>0)
		sprintf(buf, "%0.*lf", prec, v);
	else
		sprintf(buf, "%g",  v);
	return buf;
}


inline long ToInt(const string s){
	return atoi(s.c_str());
}


inline double ToFloat(const string s){
	return atof(s.c_str());
}

bool IsInt(const string s);

bool IsFloat(const string s);

void TrimStr(string &s);

};

