#pragma once
#include <string>
#include <vector>
#include <Classes.h>

using namespace std;

namespace rofrm {

struct Record;

class Field {
protected:
	Record *owner;
public:
	enum {vtNone, vtInt, vtFloat, vtString};
	const short type;
	const short size;
	const string name;
	virtual string GetText()=0;
	virtual void SetText(const string s)=0;
	Field(Record *rec, short type, short size, const string name);
	Record* GetOwner() {
		return owner;
	}
};


class IntField: public Field {
	int value;
public:
	IntField(Record *r, int v,  const string n): Field(r, vtInt, sizeof(int), n), value(v)  {
	}
	string GetText();
	void SetText(const string s);
	operator int() {return value;}
	int operator=(int v) {return value=v;}
};


class FloatField: public Field {
	double value;
public:
	int width, prec;
	FloatField(Record *r, double v, const string n): Field(r, vtFloat, sizeof(double), n),
			value(v), width(0), prec(2)  {
	}
	string GetText();
	void SetText(const string s);
	operator double() {return value;}
	double operator=(double v) {return value=v;}
};


class StringField: public Field {
	char value[64];
public:
	StringField(Record *r, string s, const string n): Field(r, vtString, 64, n) {
		SetText(s);
	}
	string GetText();
	void SetText(const string s);
};


struct Record{
	vector<Field*> fields;
	string GetText();
	void SetText(const string s);
	void Import(ParamList &list);
	void Export(ParamList &list);
};


class DataSet: public Record
{
protected:
	int recSize;
	vector<string> data;
	bool active;
	int recCount;
	char* FieldData(Field *fld);
public:
	const int recNo;
	DataSet(const char *n=0);
	void Open();
	void Close();
	bool Eof();
	void Append();
	void Delete();
	void Post();
	void SetRecNo(int ix);
	void First() { SetRecNo(0); }
	void Last() { SetRecNo(recCount); }
	void Prior() { SetRecNo(recNo-1); }
	void Next() { SetRecNo(recNo+1); }
	bool SaveToFile(string fn);
	bool LoadFromFile(string fn);
};

}
