#pragma once

#include <Classes.h>
#include <cassert>
#include <vector>
#include <algorithm>


namespace rofrm {



struct Cmds {
enum T {cOk=1, cCancel, cClose, cYes, cNo};
};



class Control;
class Container;


struct NotifyArgs
{
  NotifyArgs(Control *s, int cd=0): source(s), code(cd) {};
  Control *source;
  int code;
};

//============================== InputArgs ====================================

namespace Keys
{
enum {
	cShiftMask =1, cCtrlMask =  2, cMetaMask = 4, cAltMask = 8, cAltGrMask = 8,
	cButton1Mask =16, cButton2Mask =8, cButton3Mask =4
};
enum{
	  	KEY_DOWN  = 0x1,
		KEY_UP        = 0x2,
		KEY_CHAR   =  0x4
 };
}


using namespace Keys;

class InputArgs : public NotifyArgs
{
public:
  InputArgs(Control *s, int mf): NotifyArgs (s),
      modifiers(mf),
      consumed(false) {
    }
  bool consumed;
  int modifiers;
};


//=============================== Key =========================================

class KeyArgs :public InputArgs {
public:
	KeyArgs(Control *s, int sig, int mf, int cd): InputArgs(s, mf), keyCode(cd) {
		code = sig;
	}
  int keyCode;
};


//=============================== Mouse =========================================

class MouseArgs : public InputArgs
{
public:

  enum MouseBtn {
    cNoneBtn=0,
    cLeftBtn=1,
    cRightBtn=2,
    cMidleBtn=4
  };

  MouseArgs(Control *s, int x, int y, MouseBtn btns, int modifiers)
    : InputArgs(s, modifiers), x(x), y(y), buttons(btns){
  }
  int buttons;
  int x,y;
};


class SystemArgs : public NotifyArgs
{
public:
	SystemArgs(Control *s, int sig, int cd): NotifyArgs (s, cd), signal(sig) {
	  code = cd;
  }
  int signal;
};



//class Window;

//=============================== ItemArgs =======================================
class GDC;

struct PaintArgs: NotifyArgs {
  PaintArgs(Control *src, GDC *gdc)
    :NotifyArgs(src, 0), gdc(gdc) {
    }
  GDC *gdc;
};

//=============================== ItemArgs =======================================

struct ItemArgs: NotifyArgs {
    string text;
    int item;
  ItemArgs(Control *src, const string& txt, int it)
    :NotifyArgs(src, 1), text(txt), item(it) {
    }
    void Reject() {code=-1;};
};

struct ListArgs{

    int oper, index;
    ListArgs(int op, int ix):
      oper(op), index(ix){
      //:NotifyArgs(src, cd), text(txt), index(ix) {
    }
};


class StringListProp
{
public:
  enum {cAdd, cRemove, cClear};
  typedef  std::vector<string> T;
  size_t count() const {return list.size();}
  void Add(const string& item){
    list.push_back(item);
    Notify(cAdd, list.size()-1);
  }
  void Remove(int index) {
    Notify(cRemove, index);
    list.erase(list.begin()+index);
  }
  void Clear() {
    Notify(cClear, list.size());
    list.clear();
  }
  void InsertNew(const string& item) {
    if(item.empty()) return;
    T::iterator i = std::find(list.begin(), list.end(), item);
    if(i != list.end()) list.erase(i);
	list.insert(list.begin(), item);
  }
  int Find(const string& item) {
    T::iterator i = std::find(list.begin(), list.end(), item);
    if(i != list.end()) return i-list.begin();
	return -1;
  }
  void SetLines(const string& lines);
  const string& operator[](int n)  const {return list[n];}
  event<ListArgs> onOperate;
protected:
  //Control *ctrl;
  void Notify(int op, int ix){
    ListArgs e(op,ix);
    onOperate.exec(e);
  }
  T list;
};



struct ScrollArgs: NotifyArgs
{
  ScrollArgs(Control *src, int code, int pos)
    :NotifyArgs(src, code), pos(pos)
  {}
  int pos;
  void Reject() {code=-1;};
};


struct TreeArgs: NotifyArgs
{
   TreeArgs(Control *src, int code,  TreeItem *itm)
    :NotifyArgs(src, code), item(itm)
  {}
  TreeItem *item;
  void Reject() {code=-1;};
};



struct ScrollData {
    int     min;
    int     max;
    uint    page;
    int     pos;
    int     trackPos;
};


class ScrollProperty
{
public:
  bool enabled;
  void SetPos(int pos);
  void SetData(ScrollData &data);
  void GetData(ScrollData &data);
  ScrollProperty(Control *ctrl, int dir):
    ctrl(ctrl), dir(dir), enabled(false)
  {};
private:
  Control *ctrl;
  const int dir;
};



//======================  Timer ===========================


class Timer {
	int id,ix;
	Control *ctrl;
public:
	int interval;
	Timer();
	virtual ~Timer();
	void Start();
	void Stop();
	event<NotifyArgs> onTick;
	int GetId() {return id;}
	bool IsEnabled() {return id>-1;}
};

typedef unsigned int  Shortcut;

const int cImgNone = -2;
class Action {
	friend class ActionList;
	typedef void (object::*TActH)(int);
public:
enum{act_end=0, act_sep=-1, act_block=-2};
	string title;
	Shortcut shortcut;
	int id;
	int imgIdx;
  //void Add_H(object *b, TH hF){
    //link(b, static_cast<T>(hF));
 // }

	template<class TH>
	Action(const string tit,  object *obj, TH hF, int img=-2, int id=0, int acc=0):
		id(id), title(tit), shortcut(acc), obj(obj), handler(static_cast<TActH>(hF)), imgIdx(img){
	}
	Action(int id, const string tit, int acc, int img=-2): id(id), title(tit), shortcut(acc), handler(0), imgIdx(img){
	}
	//Action(const string tit): title(tit), id(-2), shortcut(0), handler(0), imgIdx(0){
	//}
	Action(int id=0):shortcut(0), id(id), handler(0), imgIdx(-2), objID(-1){
	}
	void Link(object *o) {
		obj = o;
	}
	bool Exec() {
		if(obj && handler) {
			(obj ->* handler)(id);
			return true;
		}else return false;
	}
	static Action end, _sep_;
private:
	int objID;
	object *obj;
	TActH handler;
};

#define _separator_ (Action*)0

class ActionList: public object
{
	vector<Action*> mem,list;
	int objID;
	object *obj;
	Action::TActH defaultH;
public:
	Action *last;
	ActionList():obj(0), objID(0), last(0),defaultH(0) {
	}
	~ActionList();
	void Init(object *ref, int refID=0){
		obj = ref;
		objID = refID;
	}
	template<class TH>
	void Init(object *ref,  TH hF, int refID=0){
		obj = ref;
		objID = refID;
		defaultH = static_cast<Action::TActH>(hF);
	}
	Action* Add(int id, const string title, int acc=0,  int img=-2);
	template<class TH>
	Action* Add(int id, const string title, int acc,  int img,  TH hF) {
		Add(id, title, acc, img);
		last->handler = static_cast<Action::TActH>(hF);
		return last;
	}
	Action* Add(Action* act) {
		list.push_back(act);
	}
	void Separator(){
		list.push_back(&Action::_sep_);
	}
	void Block(const string title){
		list.push_back(last=new Action(-2, title, 0));
		mem.push_back(last);
	}
	void End(){
		list.push_back(&Action::end);
	}
	void Link(object *ref, int refID=0) {
		for(int i=0; i<list.size(); ++i) {
			if(list[i]->objID==refID)
				list[i]->obj=ref;
		}
	}
	template<class TH>
	ActionList*  Add(const string title, TH hF, int acc=0,  int img=-2) {
		return Add(0,title, acc, hF,img);
	}
	int Size(){
		return list.size();
	}
	Action* operator[](int ix){
		return list[ix];
	}
};


namespace Keys
{
  enum {
    UNDEFINED=0x0
  };

  int GetModifiers(int mouseButton);
  int MapToSystemKey(int Key);
  int MapFromSystemKey(uint sKey, int modifiers);
}




}

