#pragma once
#include <Core.h>



namespace rofrm {

class Pen {
	friend class GDC;
public:
  enum Style{cSolid, cDash, cDot, cDashDot, cDashDotDot, cCustom};
  Pen(Color c = Color::cBlack, int w=1)
    : color(c), width(w), hpen(NULL) {
  }
  ~Pen(){
    Destroy();
  }
  void SetColor(Color c){
	color = c;
	Destroy();
  }
  void SetWidth(int wd){
	width = wd;
	Destroy();
  }
  void SetStyle(Style st){
	style = st;
	Destroy();
  }
  //HPen Get() const;
  Color GetColor() { return color;}
  void Destroy();
  static HPen GetPredefined(int index);
private:
  int width;
  Style style;
  Color color;
  mutable HPen hpen;
};


class Brush {
	friend class GDC;
public:
  Brush(Color c = Color::cWhite)
    : color(c), hbrush(NULL) {
    }
  ~Brush();
  void SetColor(Color c);
  HBrush Get() const;
  static HBrush GetPredefined(int index);
private:
  Color color;
  mutable HBrush hbrush;
};


class Bitmap
{
HBitmap hbitmap;
public:
  Bitmap(HBitmap hb = 0) : hbitmap(hb)
  {}
  ~Bitmap(){
    Destroy();
  }
  bool Empty() {return hbitmap==0;}
  void CreateCompatible(GDC *dc, int width, int height);
  void Destroy();
  void Replace(Bitmap &bmp) {
    HBitmap tmp=hbitmap;
    hbitmap=bmp.hbitmap;
    bmp.hbitmap = tmp;
  }
  operator HBitmap() {return hbitmap;}
};


class DCImpl;


class GDC  {
public:
  Range client;
  GDC();
  ~GDC();
  void SetPen(const Pen& pen);
  void SetBrush(const Brush& brush);
  void SetFont(const Font& font);
  void MoveTo(int x, int y);
  void LineTo(int x, int y);
  void Rectangle(int x1, int y1, int x2, int y2);
  void Polyline(const Point p[], int count);
  void Polygon(const Point p[], int count);
  void FillRect(int x1, int y1,  int x2, int y2);
  void RoundRect(int x1, int y1, int x2, int y2, int wd, int ht);
  void Ellipse(int x1, int y1, int x2, int y2);
  void Circle(int x, int y, int r) {
		Ellipse(x-r, y-r, x+r, y+r);
  }
  void Arrow(int x1, int y1, int x2, int y2);
  void SetPixel(int x, int y, Color col);
  Color GetPixel(int x, int y);
  void TextOut(int x, int y, const string &s);
  bool ExtTextOut(int x, int y, uint options, Range &rect, const char *txt,  int count=-1, int *lpdxwd=NULL);
  int SetBkColor(const Color crColor);
  int SetTextColor(const Color  crColor);
  void FillSolidRect(Range &rect, const Color  clr);
  void FillSolidRect(int x, int y, int cx, int cy, const Color  clr){
		Range rc(x, y, x + cx, y + cy);
		FillSolidRect(rc, clr);
  }
  void Draw(int left, int top, int width, int height, Bitmap &src);
  void Clear(Color color=Color::cWhite) {
    FillSolidRect(client, color);
  }
  HFont SelectFont(HFont hFont);
  DCImpl& GetDC() { return dcimpl;}
protected:
  DCImpl &dcimpl;
};


class PaintDC : public GDC {
public:
  PaintDC(Control* ctrl);
  ~PaintDC();
	Control* GetCtrl() {return ctrl;}
private:
	//void *ps;
  Control* ctrl;
};


class CachedDC:public GDC
{
  HBitmap oldbitmap;
  Bitmap *cache;
public:
  PaintDC paintDC;
  CachedDC(Control *ctrl, Bitmap *bmp);
  ~CachedDC();
  void ApplyCache();
};

}

