#pragma once

#include <Core.h>
#include <vector>


namespace rofrm {

class AccelMenu : public ActionMenu
{
	friend class MenuItem;
	HAccel haccel;
public:
	void Create();
	void Destroy();
	void Setup(Control *ctrl);
};


//================== MenuItem ====================
class Menu;

class MenuItem: public object
{
	friend class Menu;
public:
	const bool isMenu;
	bool IsEnabled() const;
	virtual void SetEnabled(bool);
protected:
	Action *action;
	bool enabled, valid;
	Menu *parent;
	MenuItem(Action *act);
	MenuItem(): enabled(true), isMenu(true), valid(false), action(0){
	}
	virtual void Create(ActionMenu *ab);
	virtual void Destroy();
	void Update();
};


//================== Menu ========================
typedef std::vector<MenuItem*> MenuItemList;

class Menu : public MenuItem  {
	friend class MenuBar;
public:
	Menu(const string label = "");
	Menu(ActionList &actList) {
		Load(actList);
	}
	~Menu();
	void Create(ActionMenu *ab);
	void Destroy();
	int GetItemCount() const;
	MenuItem* AddMenuItem(Action *act);
	Menu* AddSubMenu(const string &lab);
	void Load(ActionList &actList);
	void AddSeparator();
	void SetEnabled(bool);
	void Remove(int index);   // deprecated
	void RemoveAll();
	void SetLabel(const string &s) { label=s; Update(); }
	operator HMenu() {return hmenu;}
protected:
	HMenu hmenu;
	string label;
	MenuItemList items;
	ActionMenu *base;
};



class ContextMenu: public ActionMenu {
	Menu menu;
	HMenu hmenu;
public:
	ContextMenu(){
	}
	ContextMenu(ActionList &actList){
		menu.Load(actList);
	}
	~ContextMenu();
	int Load(ActionList &actList){
		menu.Load(actList);
	}
	void Setup(Control *ctrl);
};


//========================= MenuBar =================================

class Container;

class MenuBar : public AccelMenu
{
	HMenu hmenu;
public:
	void Setup(Control *ctrl);
	Menu* Add(Menu* m);
	void Remove(int index);
	void Remove(Menu* m);
	int GetMenuCount() const;
	Menu* GetMenu(int i) const;
private:
	typedef std::vector<Menu*> MenuList;
	MenuList menus;
	Menu* helpMenu;
	Container *parent;
	void AddMenu(Menu* m);
};



}

