#pragma once

#include <vector>
#include "CustomCtrl.h"
#include "Mathex.h"


namespace rofrm{

using namespace std;


typedef double (*Fun)(double);

class Series
{
public:
	struct Node{
		Node(double x, double y):
			x(x), y(y), attr(0){}
		double x,y;
		int attr;
	};
	vector<Node*> data;
public:
	string name;
	bool sorted;
	double xmin, xmax;
	double ymin, ymax;
	bool visible, select;
	int color;
	int Count() const {return data.size();}
	Series();
	virtual ~Series();
	void Clear();
	Node* AddXY(double x, double y);
	void FillData(double a, double b, int n, Fun *fun=0);
	int LocateX(double x);
	int LocateNearestX(double x);
	Node& operator[](int n) {
		assert(n<data.size());
		return *data[n];
	}
	bool SaveToFile(const string name);
};


class DistribSeries: protected Series
{
	double scaleX;
public:
  DistribSeries():scaleX(0) {
  }
  void AddX(double x, double weight=1);
  void SetRange(double a, double b, int num=100);
  void Clear();
};


class ErrSeries  :public Series
{
public:
  //ErrSeries() {type=stBars;}
  void AddErrBar(double x, double y, double dy);
};





enum{ MAX_MARKS=20 };

class PlotXY;


class Markers  :protected Series
{
public:
  //Markers() {type=stBars;}
  int AddMarker(bool vert=true);
  void MoveMarker(int n, double v);
  int MarkerCount() {return Count()/2;}
};


enum TRangeMode
{
  rgStatic, rgGrowing, rgAutoFit
};

struct PointSelector {
	Series *series;
	int idx;
	PointSelector (): series(0), idx(0) {
	}
};


class PlotXY :public CustomCtrl
{
	struct AxisDef {
		int len;
		int expon;
		double scale;
		string format;
		string label;
		double marks[MAX_MARKS];
		void Init(double &va, double &vb);
		string MarkText(int i);
	};
	struct SeriesCont {
		int penIdx;
		Series *series;
		SeriesCont(Series *ser): series(ser) {
		}
	};
	PointSelector  selector;
public:
	TRangeMode rangeMode;
	PlotXY(int penN=8);
	void AddSeries(Series *sr, Color cl=-1);
	void AddSeries(DistribSeries *sr) {
		AddSeries(reinterpret_cast<Series*>(sr));
	}
	void RemoveSeries();
	void SetXRange(double xmin, double xmax);
	void SetYRange(double ymin, double ymax);
	void SetView(bool lab, bool grid, int mL=-1);
	void SetLabels(string sX, string sY);
	bool IsActive(){return  state!=0;}
	void Update(bool fit=true);
	PointSelector GetSelector() { return selector;}
	void SaveAsWmf(string FN, bool fit=false);
	virtual ~PlotXY();
	int GetSeriesCount() { return seriesList.size(); }
	Series* GetSeries(int n) { return seriesList[n].series; }
	event<NotifyArgs> onSelectPoint;
protected:
	bool bLabs, bGrid;
	int mLeft, mBottom;
	int penNum;
	Pen *pens;
	Pen penGrid;
	Pen penAxies;
	Pen penCursor;
	Font fontScale;
	Font fontLabels;
	double xmin, xmax;
	double ymin, ymax;
	vector<SeriesCont> seriesList;
	void Create();
protected:  // methods
	void SetRanges(bool fit);
	void CountParams(bool fit);
	int GetX(double x) {return int(aX*x+bX);}
	int GetY(double y) {return int(aY*y+bY);}
	int RetX(int x) {return -(-x+bX)/aX ;}
	int RetY(int y) {return -(-y+bY)/aY;}
	void DrawGrid(GDC &g);
	void DrawAxies(GDC &g);
	void DrawLegend(GDC &g);
	void DrawSeries(GDC &g, SeriesCont &src);
	void DrawCursor(GDC &g);
	void Draw(GDC &g);
	void DoOnDraw(NotifyArgs &e);
	void DoOnMouseUp(MouseArgs &e);
private:
	int ekrXa, ekrXb;
	int ekrYa, ekrYb;
	AxisDef axisX, axisY;
	// linear transformations data
	double aX, bX;
	double aY, bY;
};

}//namespace
