#pragma once
#include <Core.h>

namespace rofrm {

struct TimeSpan
{
  int hour, minute, second, milliseconds;
  TimeSpan(int hr, int mn, int sc, int ms=0):
    hour(hr), minute(mn), second(sc), milliseconds(ms)
  {
  }
};

struct DateTime
{
  ushort year, month, day;
  ushort hour, minute, second, milliseconds;
  DateTime(){
    *this = Now();
  }
  DateTime(ushort yr, ushort mc, ushort dy, ushort hr=0, ushort mn=0, ushort sc=0, ushort ms=0);
  void AddDays(int dy);
  void AddMonths(int mc);
  int Compare(const DateTime &dtm);
  void operator+=(const TimeSpan &ts);
  bool operator<=(const DateTime &dtm){
    return Compare(dtm)<0;
  }
  bool operator>=(const DateTime &dtm){
    return Compare(dtm)>0;
  }
  static DateTime Now();
  static DateTime Today();
  static int DaysInMonth(int year, int month);
  static bool IsLeapYear (int year);
};


struct FileAttributes
{
	bool readOnly;
	bool hidden;
	bool system;
	bool directory;
};

class Path
{
public:
  static string ChangeExtension(string path, string ext);
  static string Combine(string path1, string path2);
  static string GetDirectoryName(string path);
  static string GetExtension(string path);
  static string GetFileName(string path);
  static string GetFileNameWithoutExtension(string path);
  static string GetFullPath(string path);
};


class Directory
{
public:
  static string GetCurrentDir();
  static void SetCurrentDir(string path);

  static bool Exists(const string path);
};

class File
{
public:
  static bool Exists(const string path){
    return Directory::Exists(path);
  }
  static bool Copy(const string src, const string dst, bool ovr=false);
};


class FileSystemInfo
{
public:
  string fullPath;
  FileSystemInfo(const string path):fullPath(path){
  };
  string Name(){
    return Path::GetFileName(fullPath);
  }
  void Change(const string path){
    fullPath = path;
  };
};

typedef std::vector<FileSystemInfo> FileSystemList;

class DirectoryInfo: public FileSystemInfo
{
public:
  bool Exists(){
    return Directory::Exists(fullPath);
  }
  void Create();
  void Remove();
  void RemoveFiles(const string pat);
  DirectoryInfo(const string path):FileSystemInfo(path){
  };
  DirectoryInfo():FileSystemInfo(Directory::GetCurrentDir()){
  };
  DirectoryInfo CreateSubdirectory(string name);
  void GetDirectories(FileSystemList &list, string pat="*.*");
  void GetFiles(FileSystemList &list, string pat="*.*");
};




int GetFileIconIndex(const string& fnpath);

enum {cMyDoc=0x0005, cAppData=0x001A, cWindows=0x0024, cSystem, cProgram, cPictures, cProgComm};

std::string GetSpecialFolder(int type=cAppData);

extern
	bool lastIOResult;

};

