#include <Core.h>
#include <cassert>
#include <gtk/gtk.h>

#define MENU_ID_OFFSET 3400
#define GTK_IMPL 1

//struct SnmHdr : NMHDR{};

namespace rofrm {

const int cTbButtonId    =  0x1000;
const int cControlId    = 0x2000;

const int cTypeIdMask   = 0xF000;
const int cCtrlIdMask   = 0x0FFF;
extern
GtkWindow *topLevelWindow;

class Control;
class Container;

enum{dgnInsert=1, dgnRemove};
enum {sigCmd, sigSetRes, sigClose};


struct WndID:GtkWidget{
  operator GtkWidget() const {return (GtkWidget)(*this);}
};


class ImgListImpl {
friend class ImageResource;
	ImageResource *res;
	XPMIcon *xpmList;
	struct ImgItem{
		GdkPixmap *pix;
		union{
			GtkIconSize size;
			GdkBitmap *mask;
		};
		const char *stock;
		XPMIcon xpm;
	};
	vector<ImgItem> list;
	void CreateXPMImage(ImgItem &itm, XPMIcon xpm);
public:
	ImgListImpl():res(0){
	}
	void AddXPMImage(XPMIcon xpm);
	GtkWidget* Get(int idx);
	GdkPixbuf* GetPixbuf(int idx);
	void InitRes(ImageResource *r, ResRef ref) {
		res = r; xpmList=ref.xpm;
	}
	bool Create();
};


struct FontID:GdkFont{
  operator GdkFont() const {return (GdkFont)(*this);}
};


struct GColor : GdkColor
{
	GColor(Color cl);
};


struct DCImpl{
	int xp,yp;
	GdkDrawable* drw;
	GdkGC *gc0, *gc, *gcfill, *gctxt;
	PangoFontDescription *font_desc;
	Font font;
	Control* ctrl;
	DCImpl():ctrl(0), drw(0), xp(0), yp(0), font_desc(0){
	}
	void Init(Control* c, GdkDrawable* d)
	{
		ctrl=c; drw=d;
		gc0 = gdk_gc_new (drw);
		GColor col(ctrl->foreColor);
		gdk_gc_set_rgb_fg_color (gc0, &col);
		gc = gc0;
		gcfill = gc0;
		gctxt = gc0;
	}
	~DCImpl(){
		if(gc0){
			g_object_unref (gc0);
		}
	}
};


class Application::Impl :public object{
public:
	Impl (){//:hmod(0) {
	}
	//void Initialize(HMODULE hm);
	//void StartEventLoop() {
	  //eventQueue.Enter();
	//}
	//void StopEventLoop() {
	  //eventQueue.Leave();
	//}
	void AddDisplayable(Control*); //Syst
	void RemoveDisplayable(Control*);
	Control* GetControl(int id) const;
	static Size GetScreenSize() {
		return Size(gdk_screen_width(), gdk_screen_height());
	}
	static int GetScreenResolution(){
		long result = 97;
		return result;
	}
	static int MsgBox(const string &msg, const string &tit, int flags);
	static void ProcessMessages() {
		if(gtk_events_pending())
			gtk_main_iteration();
	}
	//HMODULE hmod;
	//EventQueue eventQueue;
private:
	std::vector<Control*> displayableComponents;
};


enum PackType{cpAuto,cpFixed, cpVBox, cpHBox, cpTable};
enum {stCanFocus, stClickHandle, stOutFocus, stKeyPress};
enum {ntwSize, ntwExit};

class SysImpl :public object{
public:
	GtkWidget *hwnd;
	PackType packType;
	GtkWidget *hpack;
	GtkWidget *harea;
	Control *ctrl;
	bool autopack;
	uint style;
	int shiftx,shifty;
	const CtrlType type;
	mutable bool setcolor;
	int cmdBtn;
	Font vfont;
	bool needKeys, designMode;
	void InitCtrl(GtkWidget *handle);
	void UpdVisible();
	GtkWidget* GetPacker();
	void PackCtrl(Control &c);
	virtual void CreateWnd() = 0;
	virtual void ConfigWnd();
	virtual void NotifyWnd(int cmd, int par1=0);
	virtual void DestroyWnd();
	void ValidateWnd();
	void Recreate() {
		DestroyWnd();
		CreateWnd();
	}
	void ValidateAll();
	void ShowWnd(bool b);
	void SetPacker(){
		hpack = GetPacker();
		gtk_container_add(GTK_CONTAINER(harea), hpack);
	}
	bool IsFocused() {
		return (GTK_WIDGET_HAS_FOCUS(harea) || GTK_WIDGET_HAS_FOCUS(hwnd));
	}
	virtual void DesignerClick(int x, int y);
	//void BuildWnd();
	//mutable DWORD exStyle;
  //mutable HBRUSH backgroundBrush;
  bool traceToolTip;
	DataLink *dataLink;
	int event_time;
	int ctx_x,  ctx_y;
	SysImpl(CtrlType t=cControl);
	virtual ~SysImpl();
	virtual void NotifyCtrl(uint signal, int cmd);

	void UpdColors(ColorProperty *cp) const;
	void InvalidateRect(const Rect *pR, bool erase);
  //HBRUSH getBackgroundBrush() const;
  void updateWindow() { //paintDamagedAreaImmediately()
     //::UpdateWindow(*hwnd);
  }
  void ToFront() {
	  gtk_window_present (GTK_WINDOW(hwnd));
  }
  void ToBack() {
    //::SetWindowPos(*hwnd, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE);
  }
  //void setIcon(HICON hIcon, bool big = true) {
    //::SendMessage(*hwnd, WM_SETICON, big, (LPARAM)hIcon);
  //}
  void DefaultPaint()
  {
    //::CallWindowProc(defaultWndProc, *hwnd, WM_PAINT, 0, 0);
  }
	bool CanFocus() {
		return (style&stCanFocus)!=0; ///????
	}

	void ScreenToClient(Point &pt)
	{
		//::ScreenToClient(impl, toPOINT(&pt));
	}

	void ClientToScreen(Point &pt)
	{
		//::ClientToScreen(impl, toPOINT(&pt));
	}

	void GetClientRect(Rect &cr);
	void SetControlID(int id) {
			//  ::SetWindowLong(impl, GWL_ID, id);
	}

	void SetDefaultFont(bool narrow);

  void wmGotFocus(){
    NotifyArgs e(ctrl);
    ctrl->onGotFocus.exec(e);
  }

  void wmLostFocus(){
    NotifyArgs e(ctrl);
    ctrl->onLostFocus.exec(e);
  }

  void UpdToolTip();
  //void SetTooltip (const char *text, int x, int y, bool trackmod);
  //void HideTooltip ();
  //--prop-text
  string text_buf;
  void SetWindowText(const string txt){
       text_buf = txt;
  }
  string GetWindowText();
  //borderStyle
  int vborderStyle;
  void UpdExStyle(){
  }
  //--prop-font
  Font GetFont();
  void SetFont(const Font &font);
  //--prop-cursor
  Cursor vcursor;
  void UpdCursor();
  //--prob-visible
  bool vvisible;
  //--prob-enabled
  bool venabled;
  string vtoolTip;
  void UpdEnabled() {
    //if(hwnd)
      //::EnableWindow(*hwnd, venabled);
  }
  //--prob-bounds
  void UpdBounds();
  //virtual void SizeNotify();

  void setParent(Control *parent){
     ctrl->parent = parent;
  }
  void SetFocus() {
		gtk_widget_grab_focus(harea);
  }
  Control* GetFocused() {
	  GtkWidget *hw = gtk_window_get_focus(GTK_WINDOW(harea));
    return 0;//???GetControl(hw);
  }

  void SetHWND(GtkWidget * h){
    hwnd = (WndID*)h;
  }
  operator GtkWidget*() const {return hwnd;}
  Rect& GetBounds() {return ctrl->bounds;}

	template<class T>
	T& GetCtrl() {return (T&)*ctrl;}

	template <class T>
  typename T::Impl& operator()(T *ctrl) {
	return  static_cast<typename T::Impl&>(ctrl->impl);
  }
	static gint MousePressCallback (GtkWidget *widget, GdkEventButton *event, Control *ctrl);
	static gboolean KeyPressCallback (GtkWidget  *widget, GdkEventKey *event, Control *ctrl);
	static gboolean MouseMoveCallback (GtkWidget  *widget, GdkEventMotion *event, Control *ctrl);
	static gboolean MouseLeaveCallback (GtkWidget  *widget, GdkEventMotion *event, Control *ctrl);
};



class Container::Impl : public SysImpl{
public:
	Impl(CtrlType t=cContainer);
	void CreateWnd();
	Container* getMe() const {return (Container*)ctrl;}
	static gboolean ExposeEvent(GtkWidget *widget, GdkEventExpose  *event,  Container* cnt);
};



class Form::Impl: public Container::Impl{
public:
	static int numOfFrames;
	int borderh, borderv;
	GtkWidget *vbox;
	bool init;
	Impl(): Container::Impl(cForm), init(true)
	{
		vvisible = false;
	}
	void NotifyCtrl(uint signal, int cmd);
	void SetWindowText(const string txt);
	string GetWindowText() {
		return text_buf;
	}

	Form*  getCtrl() const {return (Form*)ctrl;}
	bool IsModal() const {
		return static_cast<Form*>(ctrl)->modal;
	}
  	bool CheckHotButtons(Control *c, int res);
	void DoCmdEvent(int cmd){
		GetCtrl<Form>().result= cmd;
		CheckHotButtons(ctrl, cmd);
	}
	void CreateForm();
	void CreateWnd();
	void ConfigWnd();
	void NotifyWnd(int cmd, int par1);
	void DestroyForm() {
		--numOfFrames;
		init = true;
	}
	void ShowModalForm();
	void ShowForm() {
		//ShowWnd(true);
		if(IsModal()) {
			ShowModalForm();
		} else {
			ShowWnd(true);
			//gtk_widget_show_all(hwnd);
			ValidateAll();
			if ( numOfFrames == 1 ) {
				gtk_main();
				//Destroy();
			}
		}
    }

	bool FindAppWindow(const string &title, bool close) {
		return false;
	}
	void SetForeground() {
	}

	static gboolean ResizeWindowEvent(GtkWidget *widget, GdkEventConfigure *event,  Form * frm);
	static gboolean CloseWindowEvent(GtkWidget *widget, GdkEventConfigure *event,  Form *frm);
};


HFont GetFontImpl(const Font& font);


template <class T>
inline typename T::Impl& getimpl(T *ctrl) {
	return  static_cast<typename T::Impl&>(ctrl->impl);
}


inline string TransformLabs(string s)
{
	for(int i=0; i<s.length(); ++i)
		if(s[i]=='&') s[i]='_';
	return s;
}


#  ifndef NDEBUG
#    define verify(f) assert(f)
#  else
#    define verify(f) ((void)(f))
#  endif


}

