#pragma once

#define _WIN32_IE  0x0600
#define WIN32_LEAN_AND_MEAN
#define NOSERVICE
#define NOMCX
#define NOIME
#define NOSOUND
#define NOCOMM
#define NORPC
#undef NOMINMAX
#define NOMINMAX
#include <windows.h>
#ifdef TEXT
#undef TEXT
#endif

#include <commctrl.h>
#include <core.h>
#include <cassert>

#define MENU_ID_OFFSET 3400
#define WM_USER_READY WM_USER+40
#define WM_USER_NESTEDCTRL WM_USER+41
#define WM_USER_RETURN WM_USER+42

namespace rofrm {
	const int cTbButtonId    =  0x1000;
	const int cControlId    = 0x2000;
	const int cNestedCtrlId = 0x3000;
	const int cTypeIdMask   = 0xF000;
	const int cCtrlIdMask   = 0x0FFF;

	enum {evKeys=1, evMouse=2};
	enum {sigCmd, sigSetRes, sigClose};

	class Control;
	class Container;


class ImgListImpl {
friend class ImageResource;
	ImageResource *res;
	int colorBits;
	bool map3d;
public:
	HIMAGELIST himglist;
	ImgListImpl():res(0),himglist(0){
	}
	~ImgListImpl(){
		Destroy();
	}
	void InitRes(ImageResource *r, ResRef ref) {
		res = r;
		colorBits = ref.cbits;
		map3d=ref.map3d;
	}
	int Add(HBitmap hBitmap, HBitmap hBitmapMask=NULL);
	bool Create();
	void Destroy(){
		if (himglist != NULL)
			ImageList_Destroy(himglist);
		himglist = NULL;
	}
};


class DCImpl {
public:
	HPEN oldPen;
	HBRUSH oldBrush;
	PAINTSTRUCT ps;
	HDC hdc;
	DCImpl():hdc(0), oldPen(0), oldBrush(0) {
	}
	~DCImpl(){
		if(oldPen!=NULL)
			::SelectObject(hdc, oldPen);
		if(oldBrush!=NULL)
			::SelectObject(hdc, oldBrush);
	}
	operator HDC() {return hdc;}
};



class DCSelector {
public:
	DCSelector(HDC dc, HGDIOBJ obj) : dc(dc) {
		old = ::SelectObject(dc, obj);
	}
	~DCSelector() {
		::SelectObject(dc, old);
	}
private:
	HDC dc;
	HGDIOBJ old;
};

inline POINT* toPOINT(Point *p)  {return reinterpret_cast<POINT*>(p);}
inline RECT* toRECT(Range *r)  {return reinterpret_cast<RECT*>(r);}



class Application::Impl :public object{
public:
	Impl ():hmod(0) {
	}
	void Initialize(HMODULE hm);
	void StartEventLoop();
	void StopEventLoop() {
		::PostQuitMessage(0);
	}
	void AddDisplayable(Control*); //Syst
	void RemoveDisplayable(Control*);
	Control* GetControl(int id) const;
	static Size GetScreenSize() {
		int cx = ::GetSystemMetrics(SM_CXSCREEN);
		int cy = ::GetSystemMetrics(SM_CYSCREEN);
		return Size(cx, cy);
	}
	static int GetScreenResolution(){
		HWND hWnd = ::GetDesktopWindow();
		HDC hDC = ::GetDC(hWnd);
		long result = ::GetDeviceCaps(hDC, LOGPIXELSX);
		::ReleaseDC(hWnd, hDC);
		return result;
	}
	static int MsgBox(const string &msg, const string &tit, int flags);
	static void ProcessMessages();
	HMODULE hmod;
private:
	std::vector<Control*> displayableComponents;
private:
	bool OnIdle();
	int result;
};


class SysImpl :public object{
public:
	HWND hwnd;
	HGDIOBJ hfont;
	HWND htooltip; // ToolTip window
	WNDPROC defaultWndProc;
	Control *ctrl;
	const CtrlType type;
	mutable DWORD style;
	mutable DWORD exStyle;
	mutable HBRUSH backgroundBrush;
	bool subclassed;
	bool eraseBackground;
	bool hasText;
	bool textEditable;
	int cmdBtn;
	DataLink *dataLink;
	int ctx_x, ctx_y;
	int vborderStyle;
	string vtoolTip;
	Cursor vcursor;
	bool vvisible;
	bool venabled;
	bool designMode;
	string text_buf;
	SysImpl(CtrlType t=cControl);
	virtual ~SysImpl();
	HWND NewCtrl(const  LPCTSTR cls,  uint sty);
	void InitCtrl(const  LPCTSTR cls, uint evmask=0);
	virtual void CreateWnd() = 0;
	virtual void ConfigWnd();
	virtual void DestroyWnd();
	virtual void ShowWnd(bool b);
	virtual void SetStyle() const;
	virtual int EvCommand(uint cmd, uint ref=0);
	virtual LRESULT HandleNotify(NMHDR* nmHdr);
	virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
	virtual void DesignerClick(int x, int y);
	virtual void NotifyCtrl(uint signal, int cmd);
	bool IsFocused() {
		return ::GetFocus()==hwnd;
	}

	long SendMessage(uint msg, WPARAM wParam, LPARAM lParam) const {
		return ::SendMessage(hwnd, msg, wParam, lParam);
	}
	void InvalidateRect(const Rect *pR, bool erase);
		HBRUSH getBackgroundBrush() const;
	void Recreate() {
		if(hwnd) {
			DestroyWnd();
			CreateWnd();
		}
	}
	void UpdateWindow() { //paintDamagedAreaImmediately()
		::UpdateWindow(hwnd);
	}
	void ToFront() {
		::SetWindowPos(hwnd, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE);
		::SetForegroundWindow(hwnd);
	}
	void ToBack() {
		::SetWindowPos(hwnd, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE);
	}
	void SetIcon(HICON hIcon, bool big = true) {
		::SendMessage(hwnd, WM_SETICON, big, (LPARAM)hIcon);
	}
	void DefaultPaint() {
		::CallWindowProc(defaultWndProc, hwnd, WM_PAINT, 0, 0);
	}
	void ScreenToClient(Point &pt) {
		::ScreenToClient(hwnd, toPOINT(&pt));
	}
	void ClientToScreen(Point &pt)	{
		::ClientToScreen(hwnd, toPOINT(&pt));
	}
	void GetClientRect(Rect &cr){
		assert(sizeof(Rect)==sizeof(RECT));
		::GetClientRect(hwnd, (RECT*)&cr);
	}
	void SetControlID(int id) {
		::SetWindowLong(hwnd, GWL_ID, id);
	}
	void UpdColors(ColorProperty *cp){
		if (ctrl->state==stShow)
			ctrl->Invalidate();
	}
	void UpdExStyle();
	bool CanFocus() {return (style&WS_TABSTOP)!=0;}
	bool WMSetCursor(WPARAM wParam, LPARAM lParam);
	bool WMMouseMove(WPARAM wParam, LPARAM lParam);
	bool WMMouseLeave();
	bool WMMouseDown(WPARAM wParam, LPARAM lParam, int button);
	bool WMMouseUp(WPARAM wParam, LPARAM lParam, int button);
	void WMShowWindow(WPARAM wParam, LPARAM lParam);
	void WMEraseBkg(HDC hdc);
	void WMDblClick(){
		NotifyArgs e(ctrl);
		ctrl->onDblClick.exec(e);
	}
	bool WMKey(WPARAM wParam, LPARAM lParam, uint type);
	void WMGotFocus(){
		NotifyArgs e(ctrl);
		ctrl->onGotFocus.exec(e);
	}
	void WMLostFocus(){
		NotifyArgs e(ctrl);
		ctrl->onLostFocus.exec(e);
	}
	void WMCtxMenu(WPARAM, LPARAM lParam);
	void WMMove(WPARAM, LPARAM lParam);
	void WMSize(WPARAM wParam, LPARAM lParam){
		NotifyArgs e(ctrl);
		ctrl->onResize.exec(e);
	}
	void SubClassWindow();
	void UpdToolTip();
	void OpenPopupMenu(HMENU hmenu, int x , int y){
		TrackPopupMenu(hmenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, x, y, 0, hwnd, NULL);
	}
	void SetWindowText(const string txt){
		text_buf = txt;
		if(hwnd && hasText)
			::SetWindowText(hwnd, txt.c_str());
	}
	string GetWindowText();
	Font GetFont();
	void SetFont(const Font &font);
	void UpdCursor();
	void UpdVisible();
	void UpdEnabled() {
		if(hwnd)
			::EnableWindow(hwnd, venabled);
	}
	void UpdBounds();
	void SetParent(Control *parent){
		ctrl->parent = parent;
	}
	void SetFocus() {
		::SetFocus(hwnd);
	}
	Control* GetFocused() {
		HWND hwnd=::GetFocus();
		//HWND hpar=::GetParent(hwnd);
		return NULL;//GetControl(hwnd);
	}
	void SetHWND(HWND h){
		hwnd = h;
	}
	Rect& GetBounds() {return ctrl->bounds;}
	template<class T>
	T& GetCtrl() {return (T&)*ctrl;}
	//operators
	operator HWND() const {return hwnd;}
	//static
	static Control* GetWndCtrl(HWND hwnd);
	static LRESULT CALLBACK ROfrmWndProc( HWND hWnd, UINT msg, WPARAM wPar, LPARAM lPar);
};



inline LRESULT SysImpl::HandleNotify(NMHDR* /*nmHdr*/) { return 0; }
inline void SysImpl::DesignerClick(int x, int y){}



class Container::Impl : public SysImpl{
public:
	Impl(CtrlType t=cContainer);
	void ConfigWnd();
	void WMCommand(WPARAM wParam, LPARAM lParam);
	LRESULT WMNotify(WPARAM wParam, LPARAM lParam);
	LRESULT WMDrawItem(WPARAM wParam, LPARAM lParam);
	HBRUSH WMCtlColor(HDC dc, HWND hwnd);
	void WMScroll(WPARAM wParam, LPARAM lParam, bool vertical);
	LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
	void CreateWnd();
};


const static LPCSTR frameCls= "roForm";
const static LPCTSTR dlgclassName = _T("#32770");


class Form::Impl : public Container::Impl {
	mutable std::vector<HWND> disabledWindows;
public:
	HACCEL haccel;
	int idIcon;
	static int numOfFrames;
	Impl(): Container::Impl(cForm), haccel(0)
	{
		idIcon = 100;
		vvisible = false;
		eraseBackground =  true;
	}
	bool IsModal() const {
		return static_cast<Form*>(ctrl)->modal;
	}
	int EvCommand(uint cmd, uint ref);
	void NotifyCtrl(uint signal, int cmd);
	const Color& getDefaultForeColor() const {
		return SystemColor::cWindowText;
	}
	const Color& getDefaultBackColor() const {
		return SystemColor::cWindow;
	}
	static BOOL CALLBACK windowsDisableProc(HWND hwnd, LPARAM lParam) {
		Form* pThis = (Form*)lParam;
		if ( hwnd != pThis->impl.hwnd && ::IsWindowEnabled(hwnd) ) {
			HWND ph=	::GetParent(hwnd);
			if(ph==NULL){
				::EnableWindow(hwnd, FALSE);
				static_cast<Form::Impl*>(&pThis->impl)->disabledWindows.push_back(hwnd);
			}
		}
		return TRUE;
	}
    void ModalEnableParentWindows(bool b) {
		if (b ) {
			::EnumThreadWindows(::GetCurrentThreadId(), &windowsDisableProc, (LPARAM)ctrl);
		} else {
			std::vector<HWND>::iterator i;
			for ( i = disabledWindows.begin();
				i != disabledWindows.end(); ++i ) {
				if ( ::IsWindow(*i) ) {
					::EnableWindow(*i, TRUE);
				}
			}
			disabledWindows.clear();
		}
	}
	void CreateForm() ;
	void DestroyForm(){
		 --numOfFrames;
		if(IsModal()) {
				ModalEnableParentWindows(false);
				application.impl.StopEventLoop();
		} else {
			  if ( Impl::numOfFrames == 0 ) {
				 application.impl.StopEventLoop();
			  }
		  }
	}
	void ShowForm(){
		ShowWnd(true);
		if(IsModal()){
			ModalEnableParentWindows(true);
			application.impl.StartEventLoop();
		} else {
			if(numOfFrames == 1 ) {
				application.impl.StartEventLoop();
			}
		}
	}
  	bool CheckHotButtons(Control *c, int res);
	void DoCmdEvent(int cmd) {
		GetCtrl<Form>().result= cmd;
		CheckHotButtons(ctrl, cmd);
	}
	bool FindAppWindow(const string &title, bool close) {
		 HWND hApp=::FindWindow(frameCls, title.c_str());//NULL);"English-Polish dictionary"
		 if(hApp) {
				if(close)
					::CloseWindow(hApp);
				else
					::SetForegroundWindow(hApp);
				return false;
		 }
		 return true;
	}
	void SetForeground()
	{
		HWND hCurrWnd;
		int iMyTID;
		int iCurrTID;
		BOOL bRetVal;
		DWORD dwPid;
		hCurrWnd = GetForegroundWindow ();
		iMyTID   = GetCurrentThreadId ();
		iCurrTID = GetWindowThreadProcessId (hCurrWnd,&dwPid);
		AttachThreadInput (iMyTID, iCurrTID, TRUE);
		bRetVal = SetForegroundWindow (hwnd);
		AttachThreadInput (iMyTID, iCurrTID, FALSE);
	}
	void SetStyle() const;
	void WMSize(WPARAM wParam, LPARAM lParam);
	void WMActivate(WPARAM wParam, LPARAM lParam);
	virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
	Control *LocateControl(HWND h);
	void CreateWnd();
};







HFont GetFontImpl(const Font& font);

inline HDC toHDC(HDc h) {return reinterpret_cast<HDC>(h);}
inline HFONT toHFONT(HFont h) {return reinterpret_cast<HFONT>(h);}
inline HBITMAP toHBITMAP(HBitmap h) {return reinterpret_cast<HBITMAP>(h);}



template <class T>
inline typename T::Impl& getimpl(T *ctrl) {
	return  static_cast<typename T::Impl&>(ctrl->impl);
}


}
